@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -19,15 +19,21 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
 
-#include <stdlib.h>
+/* This interface is OBSOLETE -- please do not use it in your
+   application code, as it will be removed from Coin 3 and onwards. */
 
+#include <stdlib.h>
 #include <Inventor/C/basic.h>
 
+#if defined(COIN_INTERNAL) && !defined(COIN_ALLOW_CC_HASH)
+#error prefer cc_dict over cc_hash for internal code
+#endif /* COIN_INTERNAL */
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
