@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -19,7 +19,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -362,9 +362,10 @@ COIN_DLL_API void cc_glglue_glDrawRangeElements(const cc_glglue * glue,
                                                 GLenum mode, GLuint start, GLuint end, GLsizei count, GLenum type, const GLvoid * indices);
 COIN_DLL_API void cc_glglue_glArrayElement(const cc_glglue * glue, GLint i);
 
-/* OpenGL vertex array with multitexture */
 COIN_DLL_API SbBool cc_glglue_has_multitexture_vertex_array(const cc_glglue * glue);
 COIN_DLL_API int cc_glglue_max_texture_units(const cc_glglue * glue);
+COIN_DLL_API int cc_glglue_has_multidraw_vertex_arrays(const cc_glglue * glue);
+
 COIN_DLL_API void cc_glglue_glMultiDrawArrays(const cc_glglue * glue, GLenum mode, const GLint * first, 
                                               const GLsizei * count, GLsizei primcount);
 COIN_DLL_API void cc_glglue_glMultiDrawElements(const cc_glglue * glue, GLenum mode, const GLsizei * count, 
@@ -526,9 +527,11 @@ COIN_DLL_API SbBool cc_glglue_can_do_sortedlayersblend(const cc_glglue * glue);
 COIN_DLL_API int cc_glglue_get_max_lights(const cc_glglue * glue);
 COIN_DLL_API const float * cc_glglue_get_line_width_range(const cc_glglue * glue);
 COIN_DLL_API const float * cc_glglue_get_point_size_range(const cc_glglue * glue);
-COIN_DLL_API SbBool cc_glglue_is_texture_size_legal(const cc_glglue * glw, int xsize, int ysize, 
-                                                    int zsize, int bytespertexel, SbBool mipmap);
-
+/* DEPRECATED for internal use. Kept for ABI compatibility */
+COIN_DLL_API SbBool cc_glglue_is_texture_size_legal(const cc_glglue * glw, 
+                                                    int xsize, int ysize, int zsize, 
+                                                    int bytespertexel, 
+                                                    SbBool mipmap);
 
 /* ********************************************************************** */
 
