@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -19,7 +19,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -28,16 +28,17 @@
 #error this is a private header file
 #endif /* ! COIN_INTERNAL */
 
-/* NOTE: do *not* include this header file outside of the gl.c OpenGL
-   wrapper -- even if you're inside Coin library implementation
-   code. */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /* HAVE_CONFIG_H */
 
+/* ********************************************************************** */
+
 #include <Inventor/system/gl.h>
-#include <Inventor/C/base/hash.h>
+#include <Inventor/C/base/dict.h>
+#include <Inventor/C/base/string.h>
+
+/* ********************************************************************** */
 
 #ifdef __cplusplus
 extern "C" {
@@ -447,10 +448,66 @@ typedef void (APIENTRY * COIN_PFNGLGETVERTEXATTRIBIVARBPROC)(GLuint index, GLenu
 typedef void (APIENTRY * COIN_PFNGLGETVERTEXATTRIBPOINTERVARBPROC)(GLuint index, GLenum pname, 
                                                                    GLvoid **pointer);
 
+/* Typedefs for shader objects -- GL_ARB_shader_objects */
+
+/* FIXME: according to the GL_ARB_shader_objects doc, these types must
+   be at least 8 bits wide and 32 bits wide, respectively. Apart from
+   that, there does not seem to be any other limitations on them, so
+   these types may not match the actual types used on the platform
+   (these were taken from NVIDIA's glext.h for their 32-bit Linux
+   drivers). How should this be properly handled? Is there any way at
+   all one could possibly pick up these at the correct size in a
+   dynamic manner? 20050124 mortene. */
+typedef char COIN_GLchar;
+typedef unsigned long COIN_GLhandle;
+
+typedef int (APIENTRY * COIN_PFNGLGETUNIFORMLOCATIONARBPROC)(COIN_GLhandle,
+                                                             const COIN_GLchar *);
+typedef void (APIENTRY * COIN_PFNGLGETACTIVEUNIFORMARBPROC)(COIN_GLhandle,
+                                                            GLuint index,
+                                                            GLsizei maxLength,
+                                                            GLsizei * length,
+                                                            GLint * size,
+                                                            GLenum * type,
+                                                            COIN_GLchar * name);
+typedef void (APIENTRY * COIN_PFNGLUNIFORM1FARBPROC)(GLint location, GLfloat v0);
+typedef void (APIENTRY * COIN_PFNGLUNIFORM2FARBPROC)(GLint location, GLfloat v0, GLfloat v1);
+typedef void (APIENTRY * COIN_PFNGLUNIFORM3FARBPROC)(GLint location, GLfloat v0, GLfloat v1, GLfloat v2);
+typedef void (APIENTRY * COIN_PFNGLUNIFORM4FARBPROC)(GLint location, GLfloat v0, GLfloat v1, GLfloat v2, GLfloat v3);
+typedef COIN_GLhandle (APIENTRY * COIN_PFNGLCREATESHADEROBJECTARBPROC)(GLenum);
+typedef void (APIENTRY * COIN_PFNGLSHADERSOURCEARBPROC)(COIN_GLhandle, GLsizei, const COIN_GLchar **, const GLint *);
+typedef void (APIENTRY * COIN_PFNGLCOMPILESHADERARBPROC)(COIN_GLhandle);
+typedef void (APIENTRY * COIN_PFNGLGETOBJECTPARAMETERIVARBPROC)(COIN_GLhandle, GLenum, GLint *);
+typedef void (APIENTRY * COIN_PFNGLDELETEOBJECTARBPROC)(COIN_GLhandle);
+typedef void (APIENTRY * COIN_PFNGLATTACHOBJECTARBPROC)(COIN_GLhandle, COIN_GLhandle);
+typedef void (APIENTRY * COIN_PFNGLDETACHOBJECTARBPROC)(COIN_GLhandle, COIN_GLhandle);
+typedef void (APIENTRY * COIN_PFNGLGETINFOLOGARBPROC)(COIN_GLhandle, GLsizei, GLsizei *, COIN_GLchar *);
+typedef void (APIENTRY * COIN_PFNGLLINKPROGRAMARBPROC)(COIN_GLhandle);
+typedef void (APIENTRY * COIN_PFNGLUSEPROGRAMOBJECTARBPROC)(COIN_GLhandle);
+typedef COIN_GLhandle (APIENTRY * COIN_PFNGLCREATEPROGRAMOBJECTARBPROC)(void);
+typedef void (APIENTRY * COIN_PFNGLUNIFORM1FVARBPROC)(COIN_GLhandle, GLsizei, const GLfloat *);
+typedef void (APIENTRY * COIN_PFNGLUNIFORM2FVARBPROC)(COIN_GLhandle, GLsizei, const GLfloat *);
+typedef void (APIENTRY * COIN_PFNGLUNIFORM3FVARBPROC)(COIN_GLhandle, GLsizei, const GLfloat *);
+typedef void (APIENTRY * COIN_PFNGLUNIFORM4FVARBPROC)(COIN_GLhandle, GLsizei, const GLfloat *);
+typedef void (APIENTRY * COIN_PFNGLUNIFORM1IARBPROC)(COIN_GLhandle, const GLint);
+typedef void (APIENTRY * COIN_PFNGLUNIFORM2IARBPROC)(COIN_GLhandle, const GLint, GLint);
+typedef void (APIENTRY * COIN_PFNGLUNIFORM3IARBPROC)(COIN_GLhandle, const GLint, GLint, GLint);
+typedef void (APIENTRY * COIN_PFNGLUNIFORM4IARBPROC)(COIN_GLhandle, const GLint, GLint, GLint, GLint);
+typedef void (APIENTRY * COIN_PFNGLUNIFORM1IVARBPROC)(COIN_GLhandle, GLsizei, const GLint *);
+typedef void (APIENTRY * COIN_PFNGLUNIFORM2IVARBPROC)(COIN_GLhandle, GLsizei, const GLint *);
+typedef void (APIENTRY * COIN_PFNGLUNIFORM3IVARBPROC)(COIN_GLhandle, GLsizei, const GLint *);
+typedef void (APIENTRY * COIN_PFNGLUNIFORM4IVARBPROC)(COIN_GLhandle, GLsizei, const GLint *);
+
+typedef void (APIENTRY * COIN_PFNGLUNIFORMMATRIX2FVARBPROC)(COIN_GLhandle, GLsizei, GLboolean, const GLfloat *);
+typedef void (APIENTRY * COIN_PFNGLUNIFORMMATRIX3FVARBPROC)(COIN_GLhandle, GLsizei, GLboolean, const GLfloat *);
+typedef void (APIENTRY * COIN_PFNGLUNIFORMMATRIX4FVARBPROC)(COIN_GLhandle, GLsizei, GLboolean, const GLfloat *);
+
 
 /* Typedefs for GLX functions. */
 typedef void *(APIENTRY * COIN_PFNGLXGETCURRENTDISPLAYPROC)(void);
 
+/* ********************************************************************** */
+
 /* Type specification for GLX info storage structure, embedded within
    the main GL info structure below. */
 struct cc_glxglue {
@@ -473,6 +530,8 @@ struct cc_glxglue {
   COIN_PFNGLXGETCURRENTDISPLAYPROC glXGetCurrentDisplay;
 };
 
+/* ********************************************************************** */
+
 /* GL info storage structure. An instance will be allocated and
    initialized for each new GL context id. */
 struct cc_glglue {
@@ -634,20 +693,59 @@ struct cc_glglue {
   COIN_PFNGLGETVERTEXATTRIBFVARBPROC glGetVertexAttribfvARB;
   COIN_PFNGLGETVERTEXATTRIBIVARBPROC glGetVertexAttribivARB;
   COIN_PFNGLGETVERTEXATTRIBPOINTERVARBPROC glGetVertexAttribPointervARB;
-  
+
+  /* shader objects */
+  COIN_PFNGLGETUNIFORMLOCATIONARBPROC glGetUniformLocationARB;
+  COIN_PFNGLGETACTIVEUNIFORMARBPROC glGetActiveUniformARB;
+  COIN_PFNGLUNIFORM1FARBPROC glUniform1fARB;
+  COIN_PFNGLUNIFORM2FARBPROC glUniform2fARB;
+  COIN_PFNGLUNIFORM3FARBPROC glUniform3fARB;
+  COIN_PFNGLUNIFORM4FARBPROC glUniform4fARB;
+  COIN_PFNGLCREATESHADEROBJECTARBPROC glCreateShaderObjectARB;
+  COIN_PFNGLSHADERSOURCEARBPROC glShaderSourceARB;
+  COIN_PFNGLCOMPILESHADERARBPROC glCompileShaderARB;
+  COIN_PFNGLGETOBJECTPARAMETERIVARBPROC glGetObjectParameterivARB;
+  COIN_PFNGLDELETEOBJECTARBPROC glDeleteObjectARB;
+  COIN_PFNGLATTACHOBJECTARBPROC glAttachObjectARB;
+  COIN_PFNGLDETACHOBJECTARBPROC glDetachObjectARB;
+  COIN_PFNGLGETINFOLOGARBPROC glGetInfoLogARB;
+  COIN_PFNGLLINKPROGRAMARBPROC glLinkProgramARB;
+  COIN_PFNGLUSEPROGRAMOBJECTARBPROC glUseProgramObjectARB;
+  COIN_PFNGLCREATEPROGRAMOBJECTARBPROC glCreateProgramObjectARB;
+  COIN_PFNGLUNIFORM1FVARBPROC glUniform1fvARB;
+  COIN_PFNGLUNIFORM2FVARBPROC glUniform2fvARB;
+  COIN_PFNGLUNIFORM3FVARBPROC glUniform3fvARB;  
+  COIN_PFNGLUNIFORM4FVARBPROC glUniform4fvARB;
+  COIN_PFNGLUNIFORM1IARBPROC glUniform1iARB;
+  COIN_PFNGLUNIFORM2IARBPROC glUniform2iARB;
+  COIN_PFNGLUNIFORM3IARBPROC glUniform3iARB;  
+  COIN_PFNGLUNIFORM4IARBPROC glUniform4iARB;
+  COIN_PFNGLUNIFORM1IVARBPROC glUniform1ivARB;
+  COIN_PFNGLUNIFORM2IVARBPROC glUniform2ivARB;
+  COIN_PFNGLUNIFORM3IVARBPROC glUniform3ivARB;  
+  COIN_PFNGLUNIFORM4IVARBPROC glUniform4ivARB;
+  COIN_PFNGLUNIFORMMATRIX2FVARBPROC glUniformMatrix2fvARB;
+  COIN_PFNGLUNIFORMMATRIX3FVARBPROC glUniformMatrix3fvARB;
+  COIN_PFNGLUNIFORMMATRIX4FVARBPROC glUniformMatrix4fvARB;
+ 
   const char * versionstr;
   const char * vendorstr;
   SbBool vendor_is_SGI;
+  SbBool vendor_is_nvidia;
+  SbBool nvidia_color_per_face_bug;
   const char * rendererstr;
   const char * extensionsstr;
   int maxtextureunits;
   struct cc_glxglue glx;
+  float max_anisotropy;
 
   /* normalization cube map */
   GLuint normalizationcubemap;
+  GLuint specularlookup;
 
   SbBool can_do_bumpmapping;
   SbBool can_do_sortedlayersblend;
+  SbBool can_do_anisotropic_filtering;
 
   SbBool has_nv_register_combiners;
   SbBool has_ext_texture_rectangle;
@@ -656,24 +754,63 @@ struct cc_glglue {
   SbBool has_shadow;
   SbBool has_arb_fragment_program;
   SbBool has_arb_vertex_program;
+  SbBool has_arb_shader_objects;
   SbBool has_texture_env_combine;
 
   int max_lights;
   float line_width_range[2];
   float point_size_range[2];
   int max_texture_size;
-  cc_hash * glextdict;
+  cc_dict * glextdict;
 };
 
+/* ********************************************************************** */
+
+/* Primarily used internally from functions that are badly designed,
+   lacking a cc_glglue* argument in the function signature.
+
+   Note: you should try to avoid using this function if possible! */
+
+void * coin_gl_current_context(void);
+
+/* ********************************************************************** */
+
+/* Scanning for and printing info about current set of glGetError()s. */
+unsigned int coin_catch_gl_errors(cc_string *);
+/* Convert OpenGL glGetError() error code to string. */
+const char * coin_glerror_string(GLenum errorcode);
+
+/* ********************************************************************** */
+
 /* Exported internally to gl_glx.c and gl_wgl.c. */
 int coin_glglue_debug(void);
 int coin_glglue_extension_available(const char * extensions, const char * ext);
 
+/* ********************************************************************** */
+
 /* needed for bumpmap rendering */
 void coin_apply_normalization_cube_map(const cc_glglue * glue);
 
+/* ********************************************************************** */
+
+/* ARB_shader_objects */
+SbBool cc_glglue_has_arb_shader_objects(const cc_glglue * glue);
+
+/* Moved from gl.h and added compressed parameter.
+   Original function is deprecated for internal use.
+*/
+SbBool coin_glglue_is_texture_size_legal(const cc_glglue * glw, 
+                                       int xsize, int ysize, int zsize, 
+                                       int bytespertexel, 
+                                       SbBool mipmap,
+                                       SbBool compressed);
+GLint coin_glglue_get_internal_texture_format(int numcomponents,
+                                            SbBool compress);
+
+GLenum coin_glglue_get_texture_format(int numcomponents);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
 
-#endif /* COIN_GLUE_GLP_H */
+#endif /* !COIN_GLUE_GLP_H */
