@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -19,21 +19,29 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
 
+#include <stddef.h> // for NULL definition
 #include <Inventor/lists/SbList.h>
-#include <Inventor/lists/SbIntList.h>
 #include <Inventor/SbVec3f.h>
 #include <Inventor/SbBox3f.h>
-#include <stddef.h> // for NULL definition
+
+#ifdef COIN_INTERNAL
+ #define COIN_ALLOW_SBINTLIST
+ #include <Inventor/lists/SbIntList.h>
+ #undef COIN_ALLOW_SBINTLIST
+#else
+ #include <Inventor/lists/SbIntList.h>
+#endif // COIN_INTERNAL
 
 class SbSphere;
-
 class coin_bspnode;
 
+// *************************************************************************
+
 class COIN_DLL_API SbBSPTree {
 public:
   SbBSPTree(const int maxnodepts = 64, const int initsize = 4);
