@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -19,7 +19,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -28,12 +28,15 @@
 #error "Do not include SbBox.h internally."
 #endif // COIN_INTERNAL
 
-#define COIN_INCLUDED_FROM_SBBOX_H
+// These are here for compile-time compatibility with SGI / TGS
+// Inventor.
+#include <Inventor/SbBasic.h>
+#include <Inventor/SbLinear.h>
+
 #include <Inventor/SbBox2f.h>
 #include <Inventor/SbBox2s.h>
 #include <Inventor/SbBox3s.h>
 #include <Inventor/SbBox3f.h>
 #include <Inventor/SbXfBox3f.h>
-#undef  COIN_INCLUDED_FROM_SBBOX_H
 
 #endif // !COIN_SBBOX_H
