@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -19,17 +19,31 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
 
-#include <Inventor/SbBasic.h>
-#include <Inventor/C/base/hash.h>
+// *************************************************************************
+
+// Internally, we should use the SbHash template class, since it
+// provides a better interface than SbDict, and since the interface of
+// SbDict has issues on 64-bit platforms when keeping strict
+// compatibility with the original SbDict of SGI Inventor.
+
+#if defined(COIN_INTERNAL) && !defined(COIN_ALLOW_SBDICT)
+#error prefer SbHash over SbDict for internal code
+#endif // COIN_INTERNAL
+
+// *************************************************************************
+
 #include <stddef.h>
+#include <Inventor/SbBasic.h>
 
 class SbPList;
 
+// *************************************************************************
+
 class COIN_DLL_API SbDict {
 public:
   SbDict(const int entries = 251);
@@ -51,8 +65,10 @@ public:
   void setHashingFunction(unsigned long (*func)(const unsigned long key));
 
 private:
-  cc_hash * hashtable;
+  struct cc_hash * hashtable;
   static void copyval(unsigned long key, void * value, void * data);
 };
 
+// *************************************************************************
+
 #endif // !COIN_SBDICT_H
