@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -19,7 +19,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -46,6 +46,7 @@ typedef void SoDBHeaderCB(void * data, SoInput * in);
 class COIN_DLL_API SoDB {
 public:
   static void init(void);
+  static void finish(void);
   static void cleanup(void);
 
   static const char * getVersion(void);
@@ -104,6 +105,11 @@ public:
   static void writelock(void);
   static void writeunlock(void);
 
+  static void createRoute(SoNode * from, const char * eventout,
+                          SoNode * to, const char * eventin);
+  static void removeRoute(SoNode * from, const char * eventout,
+                          SoNode * to, const char * eventin);
+
 private:
   static SoGroup * readAllWrapper(SoInput * in, const SoType & grouptype);
 };
