@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -19,21 +19,27 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
 
 #include <Inventor/system/inttypes.h>
 #include <Inventor/SbBasic.h>
-#include <Inventor/SbDict.h>
 #include <Inventor/lists/SbList.h>
 #include <stdio.h> // FILE
 #ifndef COIN_INTERNAL
  // For Open Inventor compatibility.
  #include <Inventor/SoDB.h>
+ #include <Inventor/SbDict.h>
+#else
+ #define COIN_ALLOW_SBDICT // Coin 2.x hack, for ABI compatibility
+ #include <Inventor/SbDict.h>
+ #undef COIN_ALLOW_SBDICT
 #endif // COIN_INTERNAL
 
+// *************************************************************************
+
 class SoBase;
 class SbString;
 class SbTime;
@@ -44,6 +50,8 @@ class SoProto;
 class SoField;
 class SoFieldContainer;
 
+// *************************************************************************
+
 class COIN_DLL_API SoInput {
 public:
   SoInput(void);
@@ -166,6 +174,7 @@ protected:
 
 private:
   friend class SoDB;
+  friend class SoInputP;
 
   static void clean(void);
   void constructorsCommon(void);
@@ -178,6 +187,7 @@ private:
   SbList<SoInput_FileInfo *> filestack;
   SoInput_FileInfo * getTopOfStack(void) const;
 
+  // OBSOLETE, only kept for Coin 2.x ABI compatibility.
   SbDict refdict;
 };
 
