@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -19,22 +19,34 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
 
 #include <Inventor/SbBasic.h>
-#include <Inventor/SbDict.h>
 #include <stdlib.h> // For NULL definition.
 
+#ifndef COIN_INTERNAL
+// The next include for Open Inventor compatibility.
+//
+// FIXME: I haven't checked that this is actually required -- test vs
+// SGI Inventor. 20050524 mortene.
+#include <Inventor/SbDict.h>
+#endif // COIN_INTERNAL
+
+// *************************************************************************
+
 class SbName;
 class SoTypedObject;
 class SoTypeList;
 class SoFieldData;
+class SbDict;
 struct SoTypeData;
 template <class Type> class SbList;
 
+// *************************************************************************
+
 class COIN_DLL_API SoType {
 public:
   typedef void * (*instantiationMethod)(void);
@@ -86,6 +98,8 @@ private:
   int16_t index;
 
   static SbList<SoTypeData *> * typedatalist;
+
+  // OBSOLETE, only kept for Coin 2.x ABI compatibility.
   static SbDict * typedict;
   static SbDict * moduledict;
 };
