@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -19,7 +19,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -119,9 +119,11 @@ public:
                    HLHSR_PAINTER_SURFACE_REMOVAL, HIDDEN_LINES_REMOVAL };
 
   virtual void setDrawingDimensions(const SbVec2f & d, DimensionUnit u = MM);
+  virtual void setDrawingDimensions(float w, float h, DimensionUnit u = MM) { this->setDrawingDimensions(SbVec2f(w, h), u); }
   virtual SbVec2f getDrawingDimensions(DimensionUnit u = MM) const;
 
   virtual void setStartPosition(const SbVec2f & p, DimensionUnit u = MM);
+  virtual void setStartPosition(float x, float y, DimensionUnit u = MM) { this->setStartPosition(SbVec2f(x, y), u); }
   virtual SbVec2f getStartPosition(DimensionUnit u = MM) const;
 
   virtual void setColorTranslationMethod(ColorTranslationMethod method);
@@ -179,4 +181,11 @@ private:
 
 // *************************************************************************
 
-#endif //!COIN_SOVECTORIZEACTION_H
+#ifndef COIN_INTERNAL
+// For SGI / TGS Open Inventor compile-time compatibility.
+#include <Inventor/nodes/SoImage.h>
+#endif // COIN_INTERNAL
+
+// *************************************************************************
+
+#endif // !COIN_SOVECTORIZEACTION_H
