@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -19,7 +19,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -48,11 +48,11 @@ public:
     COLOR = 0x04,
     ALL = (NORMAL|TEXCOORD|COLOR)
   };
-  
+
   void renderTriangles(SoState * state, const int arrays = ALL) const;
   void renderLines(SoState * state, const int arrays = ALL) const;
   void renderPoints(SoState * state, const int array = ALL) const;
-  
+
   void addTriangle(const SoPrimitiveVertex * v0,
                    const SoPrimitiveVertex * v1,
                    const SoPrimitiveVertex * v2,
@@ -77,7 +77,7 @@ public:
 
   int getNumLineIndices(void) const;
   int getNumPointIndices(void) const;
-  
+
   const int32_t * getLineIndices(void) const;
   const int32_t * getPointIndices(void) const;
 
@@ -96,7 +96,10 @@ private:
 
     // needed for SbHash
     operator unsigned long(void) const;
-    int operator==(const Vertex & v) const;
+    
+    // needed, since if we don't add this the unsigned long operator
+    // will be used when comparing two vertices.
+    int operator==(const Vertex & v);
   };
 
   SoPrimitiveVertexCacheP * pimpl;
