@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -19,7 +19,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -28,23 +28,6 @@
 #include <Inventor/lists/SbList.h>
 #include <Inventor/SbPlane.h>
 
-class so_plane_data // internal
-{
-public:
-  so_plane_data() {}
-  so_plane_data(const SbPlane &plane, const SbMatrix &matrix);
-public:
-  SbPlane plane;
-  SbPlane wcPlane;
-  // Needed by AIX compiler
-  SbBool operator==(const so_plane_data &spd) const {
-    return spd.plane == plane && spd.wcPlane == wcPlane;
-  }
-  SbBool operator!=(const so_plane_data &spd) const {
-    return spd.plane != plane || spd.wcPlane != wcPlane;
-  }
-};
-
 class COIN_DLL_API SoClipPlaneElement : public SoAccumulatedElement {
   typedef SoAccumulatedElement inherited;
 
@@ -64,6 +47,24 @@ public:
   int getNum(void) const;
   const SbPlane & get(const int index, const SbBool inworldspace = TRUE) const;
 
+private:
+  class so_plane_data // internal
+  {
+  public:
+    so_plane_data() {}
+    so_plane_data(const SbPlane &plane, const SbMatrix &matrix);
+  public:
+    SbPlane plane;
+    SbPlane wcPlane;
+    // Needed by AIX compiler
+    SbBool operator==(const so_plane_data &spd) const {
+      return spd.plane == plane && spd.wcPlane == wcPlane;
+    }
+    SbBool operator!=(const so_plane_data &spd) const {
+      return spd.plane != plane || spd.wcPlane != wcPlane;
+    }
+  };
+
 protected:
   SbList <so_plane_data> planes;
   int startIndex;
