@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -19,7 +19,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -36,6 +36,25 @@ protected:
   virtual ~SoShapeStyleElement();
 
 public:
+
+  // flags used for optimized testing of features
+  enum Flags {
+    LIGHTING                = 0x000100,
+    TEXENABLED              = 0x000200,
+    TEXFUNC                 = 0x000400,
+    BBOXCMPLX               = 0x000800,
+    INVISIBLE               = 0x001000,
+    ABORTCB                 = 0x002000,
+    OVERRIDE                = 0x004000,
+    TEX3ENABLED             = 0x008000,
+    BIGIMAGE                = 0x010000,
+    BUMPMAP                 = 0x020000,
+    VERTEXARRAY             = 0x040000,
+    TRANSP_TEXTURE          = 0x080000,
+    TRANSP_MATERIAL         = 0x100000,
+    TRANSP_SORTED_TRIANGLES = 0x200000
+  };
+  
   virtual void init(SoState * state);
 
   virtual void push(SoState * state);
@@ -46,6 +65,7 @@ public:
 
   static const SoShapeStyleElement * get(SoState * const state);
 
+  unsigned int getFlags(void) const;
   SbBool mightNotRender(void) const;
   SbBool needNormals(void) const;
   SbBool needTexCoords(void) const;
@@ -67,6 +87,13 @@ public:
   static int getTransparencyType(SoState * const state);
   SbBool isTextureFunction(void) const;
 
+  static void setBumpmapEnabled(SoState * state, const SbBool value);
+  static void setBigImageEnabled(SoState * state, const SbBool value);
+  static void setVertexArrayRendering(SoState * state, const SbBool value);
+
+  static void setTransparentMaterial(SoState * state, const SbBool value);
+  static void setTransparentTexture(SoState * state, const SbBool value);
+
 private:
 
   static SoShapeStyleElement * getElement(SoState * const state);
