@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -19,13 +19,15 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
 
 #include <Inventor/events/SoEvent.h>
 
+// *************************************************************************
+
 #define SO_EVENT_HEADER() \
 private: \
   static SoType classTypeId; \
@@ -33,14 +35,14 @@ public: \
   static SoType getClassTypeId(void); \
   virtual SoType getTypeId(void) const
 
+// *************************************************************************
 
 #define SO_EVENT_SOURCE(_class_) \
 SoType _class_::getClassTypeId(void) { return _class_::classTypeId; } \
 SoType _class_::getTypeId(void) const { return _class_::classTypeId; } \
-/* Don't set value explicitly to SoType::badType(), to avoid a bug in */ \
-/* Sun CC v4.0. (Bitpattern 0x0000 equals SoType::badType()). */ \
-SoType _class_::classTypeId
+SoType _class_::classTypeId STATIC_SOTYPE_INIT
 
+// *************************************************************************
 
 #define SO_EVENT_INIT_CLASS(_class_, _parentclass_) \
   do { \
@@ -53,4 +55,6 @@ SoType _class_::classTypeId
       SoType::createType(_parentclass_::getClassTypeId(), SO__QUOTE(_class_)); \
   } while (0)
 
+// *************************************************************************
+
 #endif // !COIN_SOSUBEVENT_H
