@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -19,15 +19,15 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
 
 #if defined(IV_STRICT)
-#error "SoSFLong has been obsoleted. Use SoSFInt32 instead."
+#error SoSFLong has been obsoleted. Use SoSFInt32 instead.
 #else // !IV_STRICT
-#warning "SoSFLong has been obsoleted. Use SoSFInt32 instead."
+#warning SoSFLong has been obsoleted. Use SoSFInt32 instead.
 #endif // !IV_STRICT
 
 #include <Inventor/fields/SoSFInt32.h>
