@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -19,7 +19,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -28,11 +28,16 @@
 #include <Inventor/lists/SoAuditorList.h>
 #include <Inventor/C/base/rbptree.h>
 
+// *************************************************************************
+
+class SbDict;
 class SbString;
 class SoBaseList;
 class SoInput;
 class SoOutput;
 
+// *************************************************************************
+
 class COIN_DLL_API SoBase {
 
 public:
@@ -115,6 +120,8 @@ private:
   static void flushInput(SoInput * in);
 
   static void cleanClass(void);
+
+// OBSOLETE! Just kept around for Coin 2.x ABI compatibility.
   static void freeLists(unsigned long, void * value);
 
   static SoType classTypeId;
@@ -131,6 +138,7 @@ private:
   static void rbptree_notify_cb(void * auditor, void * type, void * closure);
   cc_rbptree auditortree;
 
+  // OBSOLETE! Just kept around for Coin 2.x ABI compatibility.
   static SbDict * name2obj;
   static SbDict * obj2name;
 
