@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -19,7 +19,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -69,7 +69,9 @@ public:
     COIN_2_0     = 0x0100,
     EXTENSION    = 0x0200,
     COIN_2_2     = 0x0400,
-    COIN_2_3     = 0x0800
+    COIN_2_3     = 0x0800,
+    COIN_2_4     = 0x1000,
+    INVENTOR_5_0 = 0x2000
   };
 
   static uint32_t getCompatibilityTypes(const SoType & nodetype);
@@ -161,8 +163,15 @@ private:
   void clearStateFlags(const unsigned int bits);
   void setStateFlags(const unsigned int bits);
   SbBool getState(const unsigned int bits) const;
+  // OBSOLETE, only kept for Coin 2.x ABI compatibility.
   static SbDict * compatibilitydict;
+
   static void cleanupClass(void);
 };
 
+#ifndef COIN_INTERNAL
+// For SGI / TGS Open Inventor compile-time compatibility.
+#include <Inventor/SoLists.h>
+#endif // COIN_INTERNAL
+
 #endif // !COIN_SONODE_H
