@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -19,7 +19,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -130,5 +130,16 @@
 #include <Inventor/nodes/SoWWWInline.h>
 #include <Inventor/nodes/SoTransparencyType.h>
 #include <Inventor/nodes/SoListener.h>
+// extensions beyond Open Inventor 2.1
+#include <Inventor/nodes/SoTextureScalePolicy.h>
+#include <Inventor/nodes/SoTextureUnit.h>
+#include <Inventor/nodes/SoBumpMap.h>
+#include <Inventor/nodes/SoBumpMapCoordinate.h>
+#include <Inventor/nodes/SoBumpMapTransform.h>
+#include <Inventor/nodes/SoSceneTexture2.h>
+#include <Inventor/nodes/SoTextureCoordinateCube.h>
+#include <Inventor/nodes/SoTextureCoordinateSphere.h>
+#include <Inventor/nodes/SoTextureCoordinateCylinder.h>
+#include <Inventor/nodes/SoTextureCombine.h>
 
 #endif // !COIN_SONODES_H
