@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -19,16 +19,11 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
 
-#include <Inventor/SbName.h>
-#include <Inventor/SoType.h>
-#include <Inventor/fields/SoFieldData.h>
-#include <Inventor/nodes/SoNode.h>
-
 /*
   The most used macros and their arguments:
 
@@ -48,6 +43,16 @@
       SO_NODE_INIT_ABSTRACT_CLASS(classname, parentclassname, parentclassname)
 */
 
+// *************************************************************************
+
+#include <string.h> /* strcmp used in assert() */
+#include <Inventor/SbName.h>
+#include <Inventor/SoType.h>
+#include <Inventor/fields/SoFieldData.h>
+#include <Inventor/nodes/SoNode.h>
+
+// *************************************************************************
+
 #define PRIVATE_NODE_TYPESYSTEM_HEADER( ) \
 public: \
   static SoType getClassTypeId(void); \
@@ -55,10 +60,6 @@ public: \
 private: \
   static SoType classTypeId
 
-#ifndef NDEBUG
-#include <string.h> /* strcmp used in assert() */
-#endif
-
 // FIXME: document. 20000103 mortene.
 #define SO_NODE_ABSTRACT_HEADER(_class_) \
   PRIVATE_NODE_TYPESYSTEM_HEADER(); \
@@ -80,13 +81,12 @@ private: \
 private: \
   static void * createInstance(void)
 
+// *************************************************************************
 
 #define PRIVATE_NODE_TYPESYSTEM_SOURCE(_class_) \
 SoType _class_::getClassTypeId(void) { return _class_::classTypeId; } \
 SoType _class_::getTypeId(void) const { return _class_::classTypeId; } \
-/* Don't set value explicitly to SoType::badType(), to avoid a bug in */ \
-/* Sun CC v4.0. (Bitpattern 0x0000 equals SoType::badType()). */ \
-SoType _class_::classTypeId
+SoType _class_::classTypeId STATIC_SOTYPE_INIT
 
 
 // FIXME: document. 20000103 mortene.
@@ -119,27 +119,32 @@ _class_::atexit_cleanup(void) \
 // FIXME: document. 20000103 mortene.
 #define SO_NODE_SOURCE(_class_) \
 SO_NODE_ABSTRACT_SOURCE(_class_); \
- \
+\
+/*! \
+  Creates a new instance of the class type corresponding to the SoType object. \
+*/ \
 void * \
 _class_::createInstance(void) \
 { \
   return new _class_; \
 }
 
+// *************************************************************************
+
 // FIXME: document. 20000103 mortene.
 #define SO_NODE_IS_FIRST_INSTANCE() \
    (classinstances == 1)
 
 // FIXME: document. 20000103 mortene.
-#define SO_NODE_CONSTRUCTOR(_class_) \
+#define SO_NODE_CONSTRUCTOR_NOLOCK(_class_) \
   do { \
-    SoBase::staticDataLock(); \
     _class_::classinstances++; \
     /* Catch attempts to use a node class which has not been initialized. */ \
     assert(_class_::classTypeId != SoType::badType() && "you forgot init()!"); \
     /* Initialize a fielddata container for the class only once. */ \
     if (!_class_::fieldData) { \
       /* FIXME: this is a "static" memory leak. 20030131 mortene. */ \
+      /* Internal Coin nodes are handled properly though. pederb, 20041122 */ \
       _class_::fieldData = \
         new SoFieldData(_class_::parentFieldData ? \
                         *_class_::parentFieldData : NULL); \
@@ -148,9 +153,17 @@ _class_::createInstance(void) \
        considered native. This is important to get the export code to do \
        the Right Thing. */ \
     this->isBuiltIn = FALSE; \
+  } while (0)
+
+// FIXME: document. 20000103 mortene.
+#define SO_NODE_CONSTRUCTOR(_class_) \
+  do { \
+    SoBase::staticDataLock(); \
+    SO_NODE_CONSTRUCTOR_NOLOCK(_class_); \
     SoBase::staticDataUnlock(); \
   } while (0)
 
+// *************************************************************************
 
 // FIXME: create-type with get-next and inc-next must be an atomic step in an MT
 // environment  20020216 larsa
@@ -198,6 +211,8 @@ _class_::createInstance(void) \
   } while (0)
 
 
+// *************************************************************************
+
 // FIXME: document. 20000103 mortene.
 #define SO_NODE_ADD_FIELD(_field_, _defaultval_) \
   do { \
@@ -214,4 +229,6 @@ _class_::createInstance(void) \
                             SO__QUOTE(_enumval_), _enumval_); \
   } while (0)
 
+// *************************************************************************
+
 #endif // !COIN_SOSUBNODE_H
