@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -19,7 +19,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -83,6 +83,11 @@ private:
   static void filenameSensorCB(void *, SoSensor *);
 
   SoTexture2P * pimpl;
+
+  // This is a hack for Coin-2 to let SoFieldData handle the
+  // enableCompressedTexture field being part of SoTexture2P, and not
+  // SoTexture2:
+  friend class SoFieldData;
 };
 
 #endif // !COIN_SOTEXTURE2_H
