@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -19,7 +19,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -38,7 +38,7 @@
 /* #include <windows.h> - not needed on system */
 #include <GL/gl.h>
 #include <GL/glu.h>
-/* #include <GL/glext.h> - not found on system */
+#include <GL/glext.h>
 
 /**********************************************************************/
 
@@ -1327,6 +1327,85 @@
 #define GL_MAX_PROGRAM_MATRIX_STACK_DEPTH_ARB 0x862E
 #endif /* GL_MAX_PROGRAM_MATRIX_STACK_DEPTH_ARB */
 
+/* misc shaders */
+#ifndef GL_VERTEX_SHADER_ARB
+#define GL_VERTEX_SHADER_ARB 0x8B31
+#endif /* GL_VERTEX_SHADER_ARB */
+#ifndef GL_FRAGMENT_SHADER_ARB
+#define GL_FRAGMENT_SHADER_ARB 0x8B30
+#endif /* GL_FRAGMENT_SHADER_ARB */
+
+/* GL_ARB_shader_objects */
+#ifndef GL_INT_VEC2_ARB
+#define GL_INT_VEC2_ARB 0x8B53
+#endif /* GL_INT_VEC2_ARB */
+#ifndef GL_INT_VEC3_ARB
+#define GL_INT_VEC3_ARB 0x8B54
+#endif /* GL_INT_VEC3_ARB */
+#ifndef GL_INT_VEC4_ARB
+#define GL_INT_VEC4_ARB 0x8B55
+#endif /* GL_INT_VEC4_ARB */
+#ifndef GL_FLOAT_VEC2_ARB
+#define GL_FLOAT_VEC2_ARB 0x8B50
+#endif /* GL_FLOAT_VEC2_ARB */
+#ifndef GL_FLOAT_VEC3_ARB
+#define GL_FLOAT_VEC3_ARB 0x8B51
+#endif /* GL_FLOAT_VEC3_ARB */
+#ifndef GL_FLOAT_VEC4_ARB
+#define GL_FLOAT_VEC4_ARB 0x8B52
+#endif /* GL_FLOAT_VEC4_ARB */
+#ifndef GL_FLOAT_MAT2_ARB
+#define GL_FLOAT_MAT2_ARB 0x8B5A
+#endif /* GL_FLOAT_MAT2_ARB */
+#ifndef GL_FLOAT_MAT3_ARB
+#define GL_FLOAT_MAT3_ARB 0x8B5B
+#endif /* GL_FLOAT_MAT3_ARB */
+#ifndef GL_FLOAT_MAT4_ARB
+#define GL_FLOAT_MAT4_ARB 0x8B5C
+#endif /* GL_FLOAT_MAT4_ARB */
+#ifndef GL_SAMPLER_1D_ARB
+#define GL_SAMPLER_1D_ARB 0x8B5D
+#endif /* GL_SAMPLER_1D_ARB */
+#ifndef GL_SAMPLER_2D_ARB
+#define GL_SAMPLER_2D_ARB 0x8B5E
+#endif /* GL_SAMPLER_2D_ARB */
+#ifndef GL_SAMPLER_3D_ARB
+#define GL_SAMPLER_3D_ARB 0x8B5F
+#endif /* GL_SAMPLER_3D_ARB */
+#ifndef GL_SAMPLER_CUBE_ARB
+#define GL_SAMPLER_CUBE_ARB 0x8B60
+#endif /* GL_SAMPLER_CUBE_ARB */
+#ifndef GL_SAMPLER_1D_SHADOW_ARB
+#define GL_SAMPLER_1D_SHADOW_ARB 0x8B61
+#endif /* GL_SAMPLER_1D_SHADOW_ARB */
+#ifndef GL_SAMPLER_2D_SHADOW_ARB
+#define GL_SAMPLER_2D_SHADOW_ARB 0x8B62
+#endif /* GL_SAMPLER_2D_SHADOW_ARB */
+#ifndef GL_SAMPLER_2D_RECT_ARB
+#define GL_SAMPLER_2D_RECT_ARB 0x8B63
+#endif /* GL_SAMPLER_2D_RECT_ARB */
+#ifndef GL_SAMPLER_2D_RECT_SHADOW_ARB
+#define GL_SAMPLER_2D_RECT_SHADOW_ARB 0x8B64
+#endif /* GL_SAMPLER_2D_RECT_SHADOW_ARB */
+
+#ifndef GL_OBJECT_COMPILE_STATUS_ARB
+#define GL_OBJECT_COMPILE_STATUS_ARB 0x8B81
+#endif /* GL_OBJECT_COMPILE_STATUS_ARB */
+#ifndef GL_OBJECT_INFO_LOG_LENGTH_ARB
+#define GL_OBJECT_INFO_LOG_LENGTH_ARB 0x8B84
+#endif /* GL_OBJECT_INFO_LOG_LENGTH_ARB */
+#ifndef GL_OBJECT_LINK_STATUS_ARB
+#define GL_OBJECT_LINK_STATUS_ARB 0x8B82
+#endif /* GL_OBJECT_LINK_STATUS_ARB */
+
+/* GL_EXT_texture_filter_anisotropic */
+#ifndef GL_TEXTURE_MAX_ANISOTROPY_EXT
+#define GL_TEXTURE_MAX_ANISOTROPY_EXT 0x84FE
+#endif /* GL_TEXTURE_MAX_ANISOTROPY_EXT */
+
+#ifndef GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT
+#define GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT 0x84FF
+#endif /* GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT */
 
 /*** GL enums, end ****************************************************/
 /**********************************************************************/
@@ -1363,6 +1442,21 @@
 #define GLX_PBUFFER_BIT			0x00000004
 #endif /* GLX_PBUFFER_BIT */
 
+/* These are also from GLX 1.3, and they have the same values in the
+   GLX_SGIX_pbuffer extension (with name suffix _SGIX): */
+
+#ifndef GLX_MAX_PBUFFER_WIDTH
+#define GLX_MAX_PBUFFER_WIDTH 0x8016
+#endif /* GLX_MAX_PBUFFER_WIDTH */
+
+#ifndef GLX_MAX_PBUFFER_HEIGHT
+#define GLX_MAX_PBUFFER_HEIGHT 0x8017
+#endif /* GLX_MAX_PBUFFER_HEIGHT */
+
+#ifndef GLX_MAX_PBUFFER_PIXELS
+#define GLX_MAX_PBUFFER_PIXELS 0x8018
+#endif /* GLX_MAX_PBUFFER_PIXELS */
+
 /*** GLX enums, end ***************************************************/
 /**********************************************************************/
 
@@ -1612,6 +1706,18 @@
 #define WGL_BACK_RIGHT_ARB                 0x2086
 #endif /* WGL_BACK_RIGHT_ARB */
 
+#ifndef WGL_MAX_PBUFFER_PIXELS_ARB
+#define WGL_MAX_PBUFFER_PIXELS_ARB 0x202E
+#endif /* WGL_MAX_PBUFFER_PIXELS_ARB */
+
+#ifndef WGL_MAX_PBUFFER_WIDTH_ARB
+#define WGL_MAX_PBUFFER_WIDTH_ARB 0x202F
+#endif /* WGL_MAX_PBUFFER_WIDTH_ARB */
+
+#ifndef WGL_MAX_PBUFFER_HEIGHT_ARB
+#define WGL_MAX_PBUFFER_HEIGHT_ARB 0x2030
+#endif /* WGL_MAX_PBUFFER_HEIGHT_ARB */
+
 /* Note: the above also exists in *_EXT variants. They have the same
    values. So we only bother with the *_ARB names, as they can be used
    in place of the *_EXT names. */
