@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -19,7 +19,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -51,15 +51,20 @@
 #define COIN_STUB_LINE 0
 #endif
 
-#if defined(HAVE_VAR___PRETTY_FUNCTION__)
-#define COIN_STUB_FUNC __PRETTY_FUNCTION__
-#elif defined(HAVE_VAR___FUNCTION__)
-#define COIN_STUB_FUNC __FUNCTION__
-#elif defined(HAVE_VAR___func__)
-#define COIN_STUB_FUNC __func__
+#ifdef __cplusplus
+#ifdef HAVE_CPP_COMPILER_FUNCTION_NAME_VAR
+#define COIN_STUB_FUNC HAVE_CPP_COMPILER_FUNCTION_NAME_VAR
 #else
 #define COIN_STUB_FUNC ((const char *)0L)
 #endif
+#else /* !__cplusplus */
+#ifdef HAVE_C_COMPILER_FUNCTION_NAME_VAR
+#define COIN_STUB_FUNC HAVE_C_COMPILER_FUNCTION_NAME_VAR
+#else
+#define COIN_STUB_FUNC ((const char *)0L)
+#endif
+#endif /* !__cplusplus */
+
 
 /*
   COIN_STUB(): this is the method which prints out stub
@@ -116,6 +121,7 @@
 
 #define COIN_STUB()       do { } while (0)
 #define COIN_OBSOLETED()  do { } while (0)
+#define COIN_STUB_ONCE()  do { } while (0)
 
 #endif /* !COIN_DEBUG */
 
