@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,11 +16,17 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif // HAVE_CONFIG_H
+
+#ifdef HAVE_3DS_IMPORT_CAPABILITIES
+
 //
 //  3DS File Loader for Open Inventor
 //
@@ -81,6 +87,8 @@
 //
 
 
+#include <string.h>
+
 #include <Inventor/SbPlane.h>
 #include <Inventor/SbRotation.h>
 #include <Inventor/nodes/SoCoordinate3.h>
@@ -100,7 +108,6 @@
 #include <Inventor/errors/SoDebugError.h>
 #include <Inventor/SoInput.h>
 #include <Inventor/C/tidbits.h>
-#include <string.h>
 #include "SoStream.h"
 
 
@@ -2019,3 +2026,5 @@ coin_3ds_read_file(SoInput *in, SoSeparator *&root,
 }
 
 // *************************************************************************
+
+#endif // HAVE_3DS_IMPORT_CAPABILITIES
