@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,24 +16,32 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif // HAVE_CONFIG_H
+
+#ifdef HAVE_3DS_IMPORT_CAPABILITIES
+
 //  Part of the 3DS File Loader for Open Inventor
 //
 //  developed originally by PC John (peciva@fit.vutbr.cz)
 
+#include <stdlib.h>
+#include <string.h>
+#include <assert.h>
+
 #include <Inventor/SbString.h>
 #include <Inventor/errors/SoDebugError.h>
 #include <Inventor/SoInput.h>
 #include <Inventor/C/tidbits.h>
 
 #include "SoStream.h"
-#include <stdlib.h>
-#include <string.h>
-#include <assert.h>
+
 #define STUB assert(FALSE && "Unimplemented SoStream behaviour.")
 
 
@@ -777,3 +785,5 @@ void SoStream::wrapSoInput(SoInput *input)
   streamType = SO_INPUT_WRAP;
   soinput = input;
 }
+
+#endif // HAVE_3DS_IMPORT_CAPABILITIES
