@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -31,6 +31,13 @@
   the current OpenGL context.
  */
 
+// *************************************************************************
+
+// FIXME: this doesn't seem to end up in the Doxygen-generated
+// documentation anywhere, with Doxygen version 1.2.18, at least.
+// Find out why. There are also many other typedefs like this in Coin,
+// which are not within the scope of a class declaration. 20040707 mortene.
+
 /*!
   \typedef void SoGLRenderPassCB(void * userdata)
 
@@ -43,6 +50,17 @@
 
   \sa setPassCallback()
  */
+
+// *************************************************************************
+
+#include <string.h>
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif // HAVE_CONFIG_H
+
+#include <Inventor/actions/SoGLRenderAction.h>
+
 #include <Inventor/C/glue/gl.h>
 #include <Inventor/C/glue/glp.h>
 #include <Inventor/C/glue/simage_wrapper.h>
@@ -50,45 +68,37 @@
 #include <Inventor/C/tidbitsp.h>
 #include <Inventor/SbColor.h>
 #include <Inventor/SbPlane.h>
-#include <Inventor/actions/SoGLRenderAction.h>
 #include <Inventor/actions/SoGetBoundingBoxAction.h>
 #include <Inventor/actions/SoSubActionP.h>
+#include <Inventor/caches/SoBoundingBoxCache.h>
+#include <Inventor/elements/SoCacheElement.h>
 #include <Inventor/elements/SoDecimationPercentageElement.h>
 #include <Inventor/elements/SoDecimationTypeElement.h>
+#include <Inventor/elements/SoGLCacheContextElement.h>
+#include <Inventor/elements/SoGLLazyElement.h>
 #include <Inventor/elements/SoGLLightIdElement.h>
 #include <Inventor/elements/SoGLRenderPassElement.h>
 #include <Inventor/elements/SoGLUpdateAreaElement.h>
 #include <Inventor/elements/SoGLViewportRegionElement.h>
-#include <Inventor/elements/SoGLLazyElement.h>
+#include <Inventor/elements/SoLazyElement.h>
 #include <Inventor/elements/SoModelMatrixElement.h>
 #include <Inventor/elements/SoOverrideElement.h>
-#include <Inventor/elements/SoShapeStyleElement.h>
-#include <Inventor/elements/SoTextureOverrideElement.h>
-#include <Inventor/elements/SoViewVolumeElement.h>
-#include <Inventor/elements/SoGLCacheContextElement.h>
-#include <Inventor/elements/SoWindowElement.h>
-#include <Inventor/elements/SoLazyElement.h>
-#include <Inventor/elements/SoCacheElement.h>
-#include <Inventor/elements/SoViewingMatrixElement.h>
 #include <Inventor/elements/SoProjectionMatrixElement.h>
 #include <Inventor/elements/SoShapeHintsElement.h>
+#include <Inventor/elements/SoShapeStyleElement.h>
 #include <Inventor/elements/SoTextureEnabledElement.h>
+#include <Inventor/elements/SoTextureOverrideElement.h>
+#include <Inventor/elements/SoViewVolumeElement.h>
+#include <Inventor/elements/SoViewingMatrixElement.h>
+#include <Inventor/elements/SoWindowElement.h>
 #include <Inventor/errors/SoDebugError.h>
+#include <Inventor/lists/SoCallbackList.h>
+#include <Inventor/lists/SoEnabledElementsList.h>
 #include <Inventor/misc/SoGL.h>
 #include <Inventor/misc/SoState.h>
 #include <Inventor/nodes/SoNode.h>
 #include <Inventor/nodes/SoShape.h>
 #include <Inventor/nodes/SoShapeHints.h>
-#include <Inventor/lists/SoCallbackList.h>
-#include <Inventor/lists/SoEnabledElementsList.h>
-#include <Inventor/caches/SoBoundingBoxCache.h>
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif // HAVE_CONFIG_H
-#include <Inventor/system/gl.h>
-
-#include <string.h>
 
 // *************************************************************************
 
@@ -131,6 +141,10 @@
   for instance SoGLRenderAction::SORTED_OBJECT_BLEND for all other
   transparent objects.
 
+  The highest quality transparency mode in Coin is
+  SoGLRenderAction::SORTED_LAYERS_BLEND. It is also the only mode that
+  overrides all other modes in the scenegraph.
+
   \sa SoTransparencyType
 */
 
@@ -144,6 +158,12 @@
   it always renders the transparent parts of the scene correct with
   regard to internal depth ordering of objects / polygons, something
   which is not the case for any other transparency mode.
+
+  Polygons rendered with only transparent textures are not shown as
+  being transparent when using this mode. The reason being that the
+  SCREEN_DOOR mode is working on polygons, not pixels. To render
+  polygons with dither pattern, a material node has to be inserted
+  into the scenegraph with it's transparency field set.
 */
 
 /*!
@@ -274,6 +294,8 @@
 
   Turns off transparency for objects, even if transparency is set using
   an SoMaterial node.
+
+  \since Coin 1.0
 */
 
 /*!
@@ -287,6 +309,10 @@
   rendering order. It is the only transparency type rendering mode
   which is guaranteed to do so.
 
+  This mode is different from all other modes in that it overrides the
+  SoTransparencyType nodes in the scenegraph; all objects are drawn using
+  SORTED_LAYERS_BLEND.
+
   There are currently two separate code paths for this mode. Both
   paths are heavily based on OpenGL extensions. The first method is
   based on extensions which are only available on NVIDIA chipsets
@@ -315,7 +341,13 @@
 
   A rendering context with >= 24 bits depth buffer and 8 bits alpha
   channel must be the current rendering context for this blending mode
-  to be active.
+  to actually become activated. If the current rendering canvas does
+  not have these properties, Coin will fall back on a simpler
+  transparency handling mode. If you are using one of the
+  window-system binding libraries provided by Systems in Motion,
+  e.g. SoXt, SoQt or SoWin, you will need to explicitly enable this in
+  your viewer. See the API documentation of the \c setAlphaChannel()
+  method of either SoXtGLWidget, SoQtGLWidget or SoWinGLWidget.
 
   The detection of whether or not the SORTED_LAYERS_BLEND mode can be
   used will be done automatically by the Coin internals. If one or
@@ -390,8 +422,7 @@
   \sa setAbortCallback()
 */
 
-
-#ifndef DOXYGEN_SKIP_THIS
+// *************************************************************************
 
 class SoGLRenderActionP {
 public:
@@ -414,7 +445,8 @@ public:
   SbBool delayedpathrender;
   SbBool transparencyrender;
   SoPathList transpobjpaths;
-  SbList<float> transpobjdistances;
+  SoPathList sorttranspobjpaths;
+  SbList<float> sorttranspobjdistances;
 
   SoGetBoundingBoxAction * bboxaction;
   SbVec2f updateorigin, updatesize;
@@ -459,12 +491,16 @@ public:
   void renderMulti(SoNode * node);
   void renderSingle(SoNode * node);
 
+  // For transparent paths that need to be sorted
+  void addSortTransPath(SoPath * path);
 };
 
-#endif // DOXYGEN_SKIP_THIS
+// *************************************************************************
 
 SO_ACTION_SOURCE(SoGLRenderAction);
 
+// *************************************************************************
+
 // Override from parent class.
 void
 SoGLRenderAction::initClass(void)
@@ -680,7 +716,15 @@ SoGLRenderAction::getTransparencyType(void) const
 /*!
   Sets (or unsets) smoothing. If the smoothing flag is \c on, Coin
   will try to use built-in features from the OpenGL implementation to
-  smooth the appearance of otherwise jagged borders.
+  smooth the appearance of otherwise jagged line and point primitives,
+  calling
+
+  \verbatim
+      glEnable(GL_POINT_SMOOTH);
+      glEnable(GL_LINE_SMOOTH);
+  \endverbatim
+
+  ...before rendering the scene.
 
   This is a simple (and computationally non-intensive) way of doing
   anti-aliasing.
@@ -707,7 +751,8 @@ SoGLRenderAction::isSmoothing(void) const
 
 /*!
   Sets the number of rendering passes.  Default is 1, anything greater
-  will enable antialiasing.
+  will enable antialiasing through the use of an OpenGL accumulation
+  buffer.
 */
 void
 SoGLRenderAction::setNumPasses(const int num)
@@ -848,7 +893,7 @@ SoGLRenderAction::beginTraversal(SoNode * node)
     SoDebugError::postWarning("SoGLRenderAction::beginTraversal",
                               "GL error %s initialization: %s",
                               (err_before_init != GL_NO_ERROR) ? "before" : "after",
-                              sogl_glerror_string(err).getString());
+                              coin_glerror_string(err));
   }
 
   THIS->render(node);
@@ -948,7 +993,7 @@ SoGLRenderAction::handleTransparency(SbBool istransparent)
   case SoGLRenderAction::SORTED_OBJECT_BLEND:
   case SoGLRenderAction::SORTED_OBJECT_SORTED_TRIANGLE_ADD:
   case SoGLRenderAction::SORTED_OBJECT_SORTED_TRIANGLE_BLEND:
-    this->addTransPath(this->getCurPath()->copy());
+    THIS->addSortTransPath(this->getCurPath()->copy());
     SoCacheElement::setInvalid(TRUE);
     if (thestate->isCacheOpen()) {
       SoCacheElement::invalidate(thestate);
@@ -981,6 +1026,26 @@ SbBool
 SoGLRenderAction::abortNow(void)
 {
   if (this->hasTerminated()) return TRUE;
+
+#if COIN_DEBUG && 0 // for dumping the scene graph during GLRender traversals
+  static int debug = -1;
+  if (debug == -1) {
+    const char * env = coin_getenv("COIN_DEBUG_GLRENDER_TRAVERSAL");
+    debug = env && (atoi(env) > 0);
+  }
+  if (debug) {
+    const SoFullPath * p = (const SoFullPath *)this->getCurPath();
+    assert(p);
+    const int len = p->getLength();
+    for (int i=1; i < len; i++) { printf("  "); }
+    const SoNode * n = p->getTail();
+    assert(n);
+    printf("%p %s (\"%s\")\n",
+           n, n->getTypeId().getName().getString(),
+           n->getName().getString());
+  }
+#endif // debug
+
   SbBool abort = FALSE;
   if (THIS->abortcallback) {
     switch (THIS->abortcallback(THIS->abortcallbackdata)) {
@@ -1071,49 +1136,13 @@ SoGLRenderAction::isRenderingDelayedPaths(void) const
 
 // Remember a path containing a transparent object for later
 // rendering. We know path == this->getCurPath() when we get here.
+// This method is only used to add paths that are to be rendered after
+// all transparent paths that need sorting have been rendered, so no
+// need to calculate distances. Just add to list.
 void
 SoGLRenderAction::addTransPath(SoPath * path)
 {
   THIS->transpobjpaths.append(path);
-
-  // if we're not going to sort the paths we don't need to calculate
-  // distance
-  if (THIS->transparencytype == DELAYED_BLEND ||
-      THIS->transparencytype == DELAYED_ADD) return;
-
-  SoNode * tail = ((SoFullPath*)path)->getTail();
-  float dist;
-
-  // test if we can find the bbox using SoShape::getBoundingBoxCache()
-  // or SoShape::computeBBox. This is the common case, and quite a lot
-  // faster than using an SoGetBoundingBoxAction.
-  if (tail->isOfType(SoShape::getClassTypeId())) { // common case
-    SoShape * tailshape = (SoShape*) tail;
-    const SoBoundingBoxCache * bboxcache = tailshape->getBoundingBoxCache();
-    SbVec3f center;
-    
-    if (bboxcache && bboxcache->isValid(this->state)) {
-      if (bboxcache->isCenterSet()) center = bboxcache->getCenter();
-      center = bboxcache->getProjectedBox().getCenter();
-    }
-    else {
-      SbBox3f dummy;
-      tailshape->computeBBox(this, dummy, center);
-    }
-    SoModelMatrixElement::get(this->state).multVecMatrix(center, center);
-    dist = SoViewVolumeElement::get(this->state).getPlane(0.0f).getDistance(center);
-  }
-  else {
-    if (THIS->bboxaction == NULL) {
-      THIS->bboxaction =
-        new SoGetBoundingBoxAction(SoViewportRegionElement::get(this->state));
-    }
-    THIS->bboxaction->setViewportRegion(SoViewportRegionElement::get(this->state));
-    THIS->bboxaction->apply(path);
-    SbVec3f center = THIS->bboxaction->getBoundingBox().getCenter();
-    dist = SoViewVolumeElement::get(this->state).getPlane(0.0f).getDistance(center);
-  }
-  THIS->transpobjdistances.append(dist);
 }
 
 // Documented in superclass. Overridden to reinitialize GL state on
@@ -1130,10 +1159,10 @@ void
 SoGLRenderAction::doPathSort(void)
 {
   // need to cast to SbPList to avoid ref/unref problems
-  SbPList * plist = (SbPList *)&THIS->transpobjpaths;
-  float * darray = (float *)THIS->transpobjdistances.getArrayPtr();
+  SbPList * plist = (SbPList *)&THIS->sorttranspobjpaths;
+  float * darray = (float *)THIS->sorttranspobjdistances.getArrayPtr();
 
-  int i, j, distance, n = THIS->transpobjdistances.getLength();
+  int i, j, distance, n = THIS->sorttranspobjdistances.getLength();
   void * ptmp;
   float dtmp;
 
@@ -1198,9 +1227,51 @@ SoGLRenderAction::removePreRenderCallback(SoGLPreRenderCB * func, void * userdat
 
 #undef THIS
 
+// *************************************************************************
 // methods in SoGLRenderActionP
-#ifndef DOXYGEN_SKIP_THIS
 
+// Private function to save transparent paths that need to be sorted.
+// The transparent paths that don't need to be sorted are rendered 
+// after the sorted ones.
+void
+SoGLRenderActionP::addSortTransPath(SoPath * path) 
+{
+  this->sorttranspobjpaths.append(path);
+
+  SoNode * tail = ((SoFullPath*)path)->getTail();
+  float dist;
+
+  // test if we can find the bbox using SoShape::getBoundingBoxCache()
+  // or SoShape::computeBBox. This is the common case, and quite a lot
+  // faster than using an SoGetBoundingBoxAction.
+  if (tail->isOfType(SoShape::getClassTypeId())) { // common case
+    SoShape * tailshape = (SoShape*) tail;
+    const SoBoundingBoxCache * bboxcache = tailshape->getBoundingBoxCache();
+    SbVec3f center;
+    
+    if (bboxcache && bboxcache->isValid(action->state)) {
+      if (bboxcache->isCenterSet()) center = bboxcache->getCenter();
+      center = bboxcache->getProjectedBox().getCenter();
+    }
+    else {
+      SbBox3f dummy;
+      tailshape->computeBBox(action, dummy, center);
+    }
+    SoModelMatrixElement::get(action->state).multVecMatrix(center, center);
+    dist = SoViewVolumeElement::get(action->state).getPlane(0.0f).getDistance(center);
+  }
+  else {
+    if (this->bboxaction == NULL) {
+      this->bboxaction =
+        new SoGetBoundingBoxAction(SoViewportRegionElement::get(action->state));
+    }
+    this->bboxaction->setViewportRegion(SoViewportRegionElement::get(action->state));
+    this->bboxaction->apply(path);
+    SbVec3f center = this->bboxaction->getBoundingBox().getCenter();
+    dist = SoViewVolumeElement::get(action->state).getPlane(0.0f).getDistance(center);
+  }
+  this->sorttranspobjdistances.append(dist);
+}
 
 // Private function which "unwinds" the real value of the "rendering"
 // variable.
@@ -1244,6 +1315,11 @@ SoGLRenderActionP::render(SoNode * node)
   SoViewportRegionElement::set(state, this->viewport);
   SoLazyElement::setTransparencyType(state,
                                      (int32_t)this->transparencytype);
+
+  if (this->transparencytype == SoGLRenderAction::SORTED_LAYERS_BLEND) {
+    SoOverrideElement::setTransparencyTypeOverride(state, node, TRUE);
+  }
+
   SoLazyElement::setColorMaterial(state, TRUE);
 
   SoGLUpdateAreaElement::set(state,
@@ -1255,10 +1331,22 @@ SoGLRenderActionP::render(SoNode * node)
 
   this->precblist.invokeCallbacks((void*) this->action);
 
+  // Check if the current OpenGL context has an accumulation buffer
+  // (rendering multiple passes doesn't make much sense otherwise).
+  GLint accumbits;
+  glGetIntegerv(GL_ACCUM_RED_BITS, &accumbits);
+
   if (this->action->getNumPasses() > 1) {
-    this->renderMulti(node);
-  }
-  else {
+    if (accumbits == 0) {
+      SoDebugError::postWarning("SoGLRenderActionP::render", 
+                                "Multipass rendering requested,\nbut current "
+                                "GL context has no accumulation buffer - "
+                                "falling back to single pass\nrendering.");
+      this->renderSingle(node);
+    } else {
+      this->renderMulti(node);
+    }
+  } else {
     this->renderSingle(node);
   }
 
@@ -1313,13 +1401,13 @@ SoGLRenderActionP::renderSingle(SoNode * node)
   assert(this->transparencyrender == FALSE);
 
   // Truncate just in case
+  this->sorttranspobjpaths.truncate(0);
   this->transpobjpaths.truncate(0);
-  this->transpobjdistances.truncate(0);
+  this->sorttranspobjdistances.truncate(0);
   this->delayedpaths.truncate(0);
 
   // Do order independent transparency rendering
   if (this->transparencytype == SoGLRenderAction::SORTED_LAYERS_BLEND) {
-
     GLint depthbits, alphabits;
     glGetIntegerv(GL_DEPTH_BITS, &depthbits);
     glGetIntegerv(GL_ALPHA_BITS, &alphabits);
@@ -1351,28 +1439,30 @@ SoGLRenderActionP::renderSingle(SoNode * node)
   
   this->action->beginTraversal(node);
 
-  if (this->transpobjpaths.getLength() && !this->action->hasTerminated()) {
+  if ((this->transpobjpaths.getLength() || this->sorttranspobjpaths.getLength()) && 
+      !this->action->hasTerminated()) {
+
     this->transparencyrender = TRUE;
     // disable writing into the z-buffer when rendering transparent
     // objects
     glDepthMask(GL_FALSE);
     SoGLCacheContextElement::set(state, this->cachecontext,
                                  TRUE, !this->isDirectRendering(state));
-       
-    // test if paths should be rendered back-to-front
-    if (this->transparencytype == SoGLRenderAction::SORTED_OBJECT_BLEND ||
-        this->transparencytype == SoGLRenderAction::SORTED_OBJECT_ADD ||
-        this->transparencytype == SoGLRenderAction::SORTED_OBJECT_SORTED_TRIANGLE_BLEND ||
-        this->transparencytype == SoGLRenderAction::SORTED_OBJECT_SORTED_TRIANGLE_ADD) {
-      this->action->doPathSort();
-      int n = this->transpobjpaths.getLength();
-      for (int i  = 0; i < n; i++) {
-        this->action->apply(this->transpobjpaths[i]);
-      }
+
+
+    // All paths in the sorttranspobjpaths should be sorted
+    // back-to-front and rendered
+    this->action->doPathSort();
+    int i;
+    for (i = 0; i < this->sorttranspobjpaths.getLength(); i++) {
+      this->action->apply(this->sorttranspobjpaths[i]);
     }
-    else {
-      this->action->apply(this->transpobjpaths, TRUE);
+
+    // Render all transparent paths that should not be sorted
+    for (i = 0; i < this->transpobjpaths.getLength(); i++) {
+      this->action->apply(this->transpobjpaths[i]);
     }
+
     // enable depth buffer writes again
     glDepthMask(GL_TRUE);
     this->transparencyrender = FALSE;
@@ -1385,8 +1475,9 @@ SoGLRenderActionP::renderSingle(SoNode * node)
   }
 
   // truncate lists to unref paths.
+  this->sorttranspobjpaths.truncate(0);
   this->transpobjpaths.truncate(0);
-  this->transpobjdistances.truncate(0);
+  this->sorttranspobjdistances.truncate(0);
   this->delayedpaths.truncate(0);
 
 }
@@ -1602,10 +1693,14 @@ SoGLRenderActionP::initSortedLayersBlendRendering(const SoState * state)
   if (glue->has_arb_fragment_program && !this->usenvidiaregistercombiners) {
   
     // Initialize fragment program
-    glue->glGenProgramsARB(1, &sortedlayersblendprogramid);
-    glue->glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, sortedlayersblendprogramid);
+    //
+    // FIXME: the program id must be bound to the current rendering
+    // context, and deallocated when it is destructed. 20040718 mortene.
+    glue->glGenProgramsARB(1, &this->sortedlayersblendprogramid);
+    glue->glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, this->sortedlayersblendprogramid);
     glue->glProgramStringARB(GL_FRAGMENT_PROGRAM_ARB, GL_PROGRAM_FORMAT_ASCII_ARB,
-                             strlen(sortedlayersblendprogram), sortedlayersblendprogram);
+                             (GLsizei)strlen(sortedlayersblendprogram),
+                             sortedlayersblendprogram);
    
     // FIXME: Maybe a wrapper for catching fragment program errors
     // should be a part of GLUE... (20031204 handegar)
@@ -1836,6 +1931,8 @@ SoGLRenderActionP::setupSortedLayersBlendTextures(const SoState * state)
     // Depth texture setup
     glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
   
+    // FIXME: the texture id must be bound to the current rendering
+    // context, and deallocated when it is destructed. 20040718 mortene.
     glGenTextures(1, &this->depthtextureid);
     glBindTexture(GL_TEXTURE_RECTANGLE_EXT, this->depthtextureid);      
     glTexImage2D(GL_TEXTURE_RECTANGLE_EXT, 0, GL_DEPTH_COMPONENT24, canvassize[0], canvassize[1],
@@ -1858,6 +1955,8 @@ SoGLRenderActionP::setupSortedLayersBlendTextures(const SoState * state)
     if(this->usenvidiaregistercombiners) { 
       // HILO texture setup
       GLushort HILOtexture[] = {0, 0};
+      // FIXME: the texture id must be bound to the current rendering
+      // context, and deallocated when it is destructed. 20040718 mortene.
       glGenTextures(1, &this->hilotextureid);
       glBindTexture(GL_TEXTURE_2D, this->hilotextureid);  
       glTexImage2D(GL_TEXTURE_2D, 0, GL_HILO_NV, 1, 1, 0, GL_HILO_NV, 
@@ -1873,6 +1972,9 @@ SoGLRenderActionP::setupSortedLayersBlendTextures(const SoState * state)
     // FIXME: What if channels are > 8 bits? This must be examined
     // closer... [Only highend ATI cards supports these resolutions if
     // I'm not mistaken.] (20031126 handegar)
+    //
+    // FIXME: the texture ids must be bound to the current rendering
+    // context, and deallocated when it is destructed. 20040718 mortene.
     glGenTextures(this->sortedlayersblendpasses, this->rgbatextureids);
     for (int i=0;i<sortedlayersblendpasses;++i) {
       glBindTexture(GL_TEXTURE_RECTANGLE_EXT, this->rgbatextureids[i]);  
@@ -2044,5 +2146,4 @@ SoGLRenderActionP::renderSortedLayersNV(const SoState * state)
   
 }
 
-
-#endif // DOXYGEN_SKIP_THIS
+// *************************************************************************
