@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -41,7 +41,7 @@
   \code
 
   // First get hold of an SoPath through the scenegraph down to the
-  // node ("mynode") you want to query about it's current world space
+  // node ("mynode") you want to query about its current world space
   // transformation(s).
 
   SoSearchAction * searchaction = new SoSearchAction;
@@ -187,6 +187,7 @@ SoGetMatrixAction::beginTraversal(SoNode * node)
 {
   assert(this->traversalMethods);
 
+  SoViewportRegionElement::set(state, this->viewportregion);
   this->matrix.makeIdentity();
   this->invmatrix.makeIdentity();
   this->texmatrix.makeIdentity();
