@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -49,6 +49,7 @@
 #include <Inventor/actions/SoSearchAction.h>
 #include <Inventor/SoDB.h>
 #include <Inventor/nodes/SoNodes.h>
+#include <Inventor/nodekits/SoBaseKit.h>
 #include <Inventor/SoInput.h>
 #include <Inventor/SoOutput.h>
 #include <Inventor/SbBSPTree.h>
@@ -466,6 +467,10 @@ SoToVRMLActionP::search_for_node(SoNode * root, const SbName & name, const SoTyp
   this->searchaction.setType(type);
   this->searchaction.setInterest(SoSearchAction::LAST);
   this->searchaction.setFind(SoSearchAction::TYPE|SoSearchAction::NAME);
+
+  SbBool old = SoBaseKit::isSearchingChildren();
+  SoBaseKit::setSearchingChildren(TRUE);
+
   this->searchaction.apply(root);
   SoNode * tail = NULL;
   SoFullPath * path = (SoFullPath*) this->searchaction.getPath();
@@ -473,6 +478,7 @@ SoToVRMLActionP::search_for_node(SoNode * root, const SbName & name, const SoTyp
     tail = path->getTail();
   }
   this->searchaction.reset();
+  SoBaseKit::setSearchingChildren(old);
   return tail;
 }
 
