@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -48,10 +48,10 @@
   #include <Inventor/actions/SoWriteAction.h>
   #include <Inventor/nodes/SoCone.h>
   #include <Inventor/nodes/SoSeparator.h>
-  
+
   static char * buffer;
   static size_t buffer_size = 0;
-  
+
   static void *
   buffer_realloc(void * bufptr, size_t size)
   {
@@ -59,7 +59,7 @@
     buffer_size = size;
     return buffer;
   }
-  
+
   static SbString
   buffer_writeaction(SoNode * root)
   {
@@ -67,28 +67,28 @@
     buffer = (char *)malloc(1024);
     buffer_size = 1024;
     out.setBuffer(buffer, buffer_size, buffer_realloc);
-  
+
     SoWriteAction wa(&out);
     wa.apply(root);
-  
+
     SbString s(buffer);
     free(buffer);
     return s;
   }
-  
+
   int
   main(int argc, char ** argv)
   {
     SoDB::init();
-  
+
     SoSeparator * root = new SoSeparator;
     root->ref();
-  
+
     root->addChild(new SoCone);
-  
+
     SbString s = buffer_writeaction(root);
     (void)fprintf(stdout, "%s\n", s.getString());
-  
+
     root->unref();
     return 0;
   }
@@ -102,7 +102,7 @@
 
 #include <Inventor/SoOutput.h>
 #include <Inventor/nodes/SoNode.h>
-
+#include "../io/SoWriterefCounter.h"
 
 SO_ACTION_SOURCE(SoWriteAction);
 
@@ -204,6 +204,11 @@ void
 SoWriteAction::beginTraversal(SoNode * node)
 {
   if (this->continuing == FALSE) { // Run through both stages.
+    // call SoWriterefCounter::instance() before traversing to set the
+    // "current" pointer in SoWriterefCounter. This is needed to be
+    // backwards compatible with old code that uses the writeref
+    // system in SoBase.
+    (void) SoWriterefCounter::instance(this->getOutput());
     this->outobj->setStage(SoOutput::COUNT_REFS);
     this->traverse(node);
     this->outobj->setStage(SoOutput::WRITE);
@@ -213,6 +218,9 @@ SoWriteAction::beginTraversal(SoNode * node)
     outobj->write('\n');
     outobj->resolveRoutes();
   }
+  if (!this->continuing) {
+    SoWriterefCounter::instance(this->getOutput())->debugCleanup();
+  }
 }
 
 /*!
