@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -42,7 +42,60 @@
 
   If you have a matrix in row-order from some other source, it can be
   "converted" to column-order by transposing it with
-  SbMatrix::transpose().
+  SbMatrix::transpose(). A simple example will help to explain
+  this.
+
+  With row-order matrices, a transformation matrix with position,
+  rotation and scale looks like this:
+
+  \code
+  M = T * R * S
+  \endcode
+
+  Where T is translation, R is rotation and S is the scale. What this
+  means is that scale is applied first. The scaled matrix is then
+  rotated, and finally the scaled and rotated matrix is
+  translated. When using column-order matrices, as done in Coin,
+  matrices are represented slightly differently; the order of
+  multiplication is reversed:
+
+  \code
+  M = S * R * T
+  \endcode
+
+  The transformation is just the same as the row-order matrix. The
+  only difference being the order of multiplication. To understand why
+  this is so, consider the sample transformation:
+
+  \code
+  M = T * R * S
+  \endcode
+
+  Converting M from a row-order matrix to a column-order matrix is
+  done as follows:
+
+  \code
+  M^t = (T * R * S)^t
+  M^t = ((T * R) * S)^t
+  M^t = S^t * (T * R)^t
+  M^t = S^t * R^t * T^t
+  \endcode
+
+  All left to be done is to remove the transpose symbols, and the
+  matrices have been converted to column-order matrices:
+
+  \code
+  M = S * R * T
+  \endcode
+
+  This was done using the fact that:
+
+  \code
+  A^t = (B * C)^t = C^t * B^t
+  \endcode
+
+  Converting from column-order to row-order is done using the same
+  principle.
 */
 
 // FIXME:
@@ -61,6 +114,7 @@
 #include <coindefs.h> // COIN_STUB()
 #include <assert.h>
 #include <string.h>
+#include <float.h>
 #if COIN_DEBUG
 #include <Inventor/errors/SoDebugError.h>
 #endif // COIN_DEBUG
@@ -298,7 +352,7 @@ float
 SbMatrix::det3(int r1, int r2, int r3,
                int c1, int c2, int c3) const
 {
-#if COIN_EXTRA_DEBUG
+#ifdef COIN_EXTRA_DEBUG
   // Check indices.
   if (r1<0 || r1>3 || r2<0 || r2>3 || r3<0 || r3>3 ||
       c1<0 || c1>3 || c2<0 || c2>3 || c3<0 || c3>3) {
@@ -363,40 +417,166 @@ SbMatrix::det4(void) const
 SbMatrix
 SbMatrix::inverse(void) const
 {
-  float det = this->det4();
-#if COIN_DEBUG
-  if (det == 0.0f) {
-    SoDebugError::postWarning("SbMatrix::inverse",
-                              "Determinant of matrix is zero.");
-    return *this;
-  }
-#endif // COIN_DEBUG
+  // check for identity matrix
+  if (SbMatrixP::isIdentity(this->matrix)) { return SbMatrix::identity(); }
 
   SbMatrix result;
 
-  // FIXME: we should be using an optimized way of calculating the
-  // inverse matrix. 20010114 mortene.
-  // UPDATE 20020513 mortene: pederb has written one which is now
-  // available in Coin/patches/.
+  // use local pointers for speed
+  float (*dst)[4];
+  dst = (float (*)[4]) result.matrix[0];
+  float (*src)[4];
+  src = (float (*)[4]) this->matrix[0];
 
-  result.matrix[0][0] = this->det3(1, 2, 3, 1, 2, 3);
-  result.matrix[1][0] = -this->det3(1, 2, 3, 0, 2, 3);
-  result.matrix[2][0] = this->det3(1, 2, 3, 0, 1, 3);
-  result.matrix[3][0] = -this->det3(1, 2, 3, 0, 1, 2);
-  result.matrix[0][1] = -this->det3(0, 2, 3, 1, 2, 3);
-  result.matrix[1][1] = this->det3(0, 2, 3, 0, 2, 3);
-  result.matrix[2][1] = -this->det3(0, 2, 3, 0, 1, 3);
-  result.matrix[3][1] = this->det3(0, 2, 3, 0, 1, 2);
-  result.matrix[0][2] = this->det3(0, 1, 3, 1, 2, 3);
-  result.matrix[1][2] = -this->det3(0, 1, 3, 0, 2, 3);
-  result.matrix[2][2] = this->det3(0, 1, 3, 0, 1, 3);
-  result.matrix[3][2] = -this->det3(0, 1, 3, 0, 1, 2);
-  result.matrix[0][3] = -this->det3(0, 1, 2, 1, 2, 3);
-  result.matrix[1][3] = this->det3(0, 1, 2, 0, 2, 3);
-  result.matrix[2][3] = -this->det3(0, 1, 2, 0, 1, 3);
-  result.matrix[3][3] = this->det3(0, 1, 2, 0, 1, 2);
+  // check for affine matrix (common case)
+  if (src[0][3] == 0.0f && src[1][3] == 0.0f &&
+      src[2][3] == 0.0f && src[3][3] == 1.0f) {
+    
+    // More or less directly from:
+    // Kevin Wu, "Fast Matrix Inversion",  Graphics Gems II
+    float det_1;
+    float pos, neg, temp;
 
-  result /= det;
+#define ACCUMULATE     \
+    if (temp >= 0.0f)  \
+      pos += temp;     \
+    else               \
+      neg += temp
+
+    /*
+     * Calculate the determinant of submatrix A and determine if the
+     * the matrix is singular as limited by floating-point data 
+     * representation.
+     */
+    pos = neg = 0.0f;
+    temp =  src[0][0] * src[1][1] * src[2][2];
+    ACCUMULATE;
+    temp =  src[0][1] * src[1][2] * src[2][0];
+    ACCUMULATE;
+    temp =  src[0][2] * src[1][0] * src[2][1];
+    ACCUMULATE;
+    temp = -src[0][2] * src[1][1] * src[2][0];
+    ACCUMULATE;
+    temp = -src[0][1] * src[1][0] * src[2][2];
+    ACCUMULATE;
+    temp = -src[0][0] * src[1][2] * src[2][1];
+    ACCUMULATE;
+    det_1 = pos + neg;
+
+#undef ACCUMULATE
+
+    /* Is the submatrix A singular? */
+    if ((det_1 == 0.0f) || (SbAbs(det_1 / (pos - neg)) < FLT_EPSILON)) {
+      /* Matrix M has no inverse */
+#if COIN_DEBUG
+      SoDebugError::postWarning("SbMatrix::inverse",
+                                "Matrix is singular.");
+#endif // COIN_DEBUG
+      return *this;
+    }
+    else {
+      /* Calculate inverse(A) = adj(A) / det(A) */
+      det_1 = 1.0f / det_1;
+      dst[0][0] = (src[1][1] * src[2][2] -
+                   src[1][2] * src[2][1]) * det_1;
+      dst[1][0] = - (src[1][0] * src[2][2] -
+                     src[1][2] * src[2][0]) * det_1;
+      dst[2][0] = (src[1][0] * src[2][1] -
+                   src[1][1] * src[2][0]) * det_1;
+      dst[0][1] = - (src[0][1] * src[2][2] -
+                     src[0][2] * src[2][1]) * det_1;
+      dst[1][1] = (src[0][0] * src[2][2] -
+                   src[0][2] * src[2][0]) * det_1;
+      dst[2][1] = - (src[0][0] * src[2][1] -
+                     src[0][1] * src[2][0]) * det_1;
+      dst[0][2] =  (src[0][1] * src[1][2] -
+                    src[0][2] * src[1][1]) * det_1;
+      dst[1][2] = - (src[0][0] * src[1][2] -
+                     src[0][2] * src[1][0]) * det_1;
+      dst[2][2] =  (src[0][0] * src[1][1] -
+                    src[0][1] * src[1][0]) * det_1;
+      
+      /* Calculate -C * inverse(A) */
+      dst[3][0] = - (src[3][0] * dst[0][0] +
+                     src[3][1] * dst[1][0] +
+                     src[3][2] * dst[2][0]);
+      dst[3][1] = - (src[3][0] * dst[0][1] +
+                     src[3][1] * dst[1][1] +
+                     src[3][2] * dst[2][1]);
+      dst[3][2] = - (src[3][0] * dst[0][2] +
+                     src[3][1] * dst[1][2] +
+                     src[3][2] * dst[2][2]);
+      
+      /* Fill in last column */
+      dst[0][3] = dst[1][3] = dst[2][3] = 0.0f;
+      dst[3][3] = 1.0f;
+    }
+  }
+  else { // non-affine matrix
+    float max, sum, tmp, inv_pivot;
+    int p[4];
+    int i, j, k;
+
+    // algorithm from: Schwarz, "Numerische Mathematik"
+    result = *this;
+    
+    for (k = 0; k < 4; k++) { 
+      max = 0.0f;
+      p[k] = 0;
+      
+      for (i = k; i < 4; i++) { 
+        sum = 0.0f;
+        for (j = k; j < 4; j++)
+          sum += SbAbs(dst[i][j]);
+        if (sum > 0.0f) {
+          tmp = SbAbs(dst[i][k]) / sum;
+          if (tmp > max) { 
+            max = tmp;  
+            p[k] = i;
+          }
+        }
+      }
+      
+      if (max == 0.0f) {
+#if COIN_DEBUG
+        SoDebugError::postWarning("SbMatrix::inverse",
+                                  "Matrix is singular.");
+#endif // COIN_DEBUG
+        return *this;
+      }
+      
+      if (p[k] != k) {
+        for (j = 0; j < 4; j++) {
+          tmp = dst[k][j];  
+          dst[k][j] = dst[p[k]][j];  
+          dst[p[k]][j] = tmp;
+        }
+      }
+      
+      inv_pivot = 1.0f / dst[k][k];
+      for (j = 0; j < 4; j++) {
+        if (j != k) {
+          dst[k][j] = - dst[k][j] * inv_pivot;
+          for (i = 0; i < 4; i++) {
+            if (i != k) dst[i][j] += dst[i][k] * dst[k][j];
+          }
+        }
+      }
+      
+      for (i = 0; i < 4; i++) dst[i][k] *= inv_pivot;
+      dst[k][k] = inv_pivot;
+    }
+    
+    for (k = 2; k >= 0; k--) {
+      if (p[k] != k) {
+        for (i = 0; i < 4; i++) {
+          tmp = dst[i][k];
+          dst[i][k] = dst[i][p[k]]; 
+          dst[i][p[k]] = tmp;
+        }
+      }
+    }
+  }
   return result;
 }
 
@@ -450,7 +630,7 @@ SbMatrix::operator SbMat&(void)
 float *
 SbMatrix::operator [](int i)
 {
-#if COIN_EXTRA_DEBUG
+#ifdef COIN_EXTRA_DEBUG
   if (i<0 || i>3) {
     SoDebugError::post("SbMatrix::operator[]", "Index out of bounds. ");
   }
@@ -468,7 +648,7 @@ SbMatrix::operator [](int i)
 const float *
 SbMatrix::operator [](int i) const
 {
-#if COIN_EXTRA_DEBUG
+#ifdef COIN_EXTRA_DEBUG
   if (i<0 || i>3) {
     SoDebugError::postWarning("SbMatrix::operator[]", "Index out of bounds. ");
   }
