@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,11 +16,21 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
 
+/*!
+  \class SbOctTree SbOctTree.h Inventor/SbOctTree.h
+  \brief The SbOctTree class defines a generic oct tree for fast geometry searches.
+  \ingroup base
+
+  \COIN_CLASS_EXTENSION
+*/
+
+// *************************************************************************
+
 #include <assert.h>
 #include <float.h>
 #include <stddef.h>
@@ -28,31 +38,25 @@
 #include <Inventor/SbOctTree.h>
 #include <Inventor/SbSphere.h>
 #include <Inventor/SbPlane.h>
+#include <Inventor/errors/SoDebugError.h>
 
-/*!
-  \class SbOctTree SbOctTree.h Inventor/SbOctTree.h
-  \brief The SbOctTree class defines a generic oct tree for fast geometry searches.
-  \ingroup base
-
-  Note: SbOctTree is an extension versus the Open Inventor API.
-*/
+// *************************************************************************
 
 /*!
   \struct SbOctTreeFuncs SbOctTree.h Inventor/SbOctTree.h
+
   The SbOctTreeFuncs struct is used to specify callback functions for
-  items SbOcTree.
+  working with items in an SbOctTree.
 
-  The only function that must be set is the \e bboxfunc. The other functions
-  must be set if you intend to use the corresponding find methods in
-  SbOctTree.
+  The only function pointer that \e must be set up is \c
+  insideboxfunc. The other functions must be set if you intend to use
+  the corresponding find methods in SbOctTree.
 */
 
-// FIXME: the doc above is faulty -- it's the insideboxfunc that's
-// needed. The bboxfunc is simply ignored. 20030328 mortene.
-
 /*!
   \var SbOctTreeFuncs::bboxfunc
-  Should return the bounding box of the item.
+
+  \c OBSOLETED -- no need to set this function.
  */
 /*!
   \var SbOctTreeFuncs::ptinsidefunc
@@ -73,6 +77,8 @@
   planes.
 */
 
+// *************************************************************************
+
 //
 // got to have unique intersection funcs, therefore the standard
 // Inventor intersection functions won't do. E.g. SbBox3f::pointInside()
@@ -135,43 +141,40 @@ box_inside_planes(const SbBox3f & box, const SbPlane * const planes,
   return TRUE;
 }
 
+// *************************************************************************
+
 class SbOctTreeNode
 {
 public:
-
-  SbOctTreeNode(void);
+  SbOctTreeNode(const SbBox3f & b);
   ~SbOctTreeNode();
 
   void addItem(void * const item,
                const SbOctTreeFuncs & itemfuncs,
-               const int maxitems,
-               const SbBox3f &nodesize);
+               const int maxitems);
   void removeItem(void * const item,
-                  const SbOctTreeFuncs & itemfuncs,
-                  const SbBox3f &nodesize);
+                  const SbOctTreeFuncs & itemfuncs);
   void findItems(const SbVec3f &pos,
                  SbList <void*> &destarray,
                  const SbOctTreeFuncs &itemfuncs,
-                 const SbBox3f &nodesize,
                  const SbBool removeduplicates) const;
   void findItems(const SbBox3f &box,
                  SbList <void*> &destarray,
                  const SbOctTreeFuncs &itemfuncs,
-                 const SbBox3f &nodesize,
                  const SbBool removeduplicates) const;
   void findItems(const SbSphere &sphere,
                  SbList <void*> &destarray,
                  const SbOctTreeFuncs &itemfuncs,
-                 const SbBox3f &nodesize,
                  const SbBool removeduplicates) const;
   void findItems(const SbPlane * const planes,
                  const int numPlanes,
                  SbList <void*> &destarray,
                  const SbOctTreeFuncs &itemfuncs,
-                 const SbBox3f &nodesize,
                  const SbBool removeduplicates) const;
 
-  void debugTree(FILE *fp, const int indent, const SbBox3f & nodesize);
+  const SbBox3f & getBBox(void) const { return this->nodesize; }
+
+  void debugTree(FILE *fp, const int indent) const;
 
 private:
   SbBool isLeaf(void) const { return this->children[0] == NULL; }
@@ -179,11 +182,12 @@ private:
 
   unsigned int totalNumberOfItems(void) const;
 
-  void splitBox(const SbBox3f & box, SbBox3f * destarray) const;
-  SbBool splitNode(const SbBox3f & nodesize, const SbOctTreeFuncs & funcs);
+  static void split3Way(const SbBox3f & box, SbBox3f * destarray);
+  SbBool splitNode(const SbOctTreeFuncs & funcs);
 
   SbOctTreeNode * children[8];
   SbList <void*> items;
+  SbBox3f nodesize;
 };
 
 // Returns all items of the node, including all items in child nodes
@@ -202,15 +206,15 @@ SbOctTreeNode::totalNumberOfItems(void) const
 }
 
 void 
-SbOctTreeNode::debugTree(FILE *fp, const int indent, const SbBox3f & nodesize)
+SbOctTreeNode::debugTree(FILE *fp, const int indent) const
 {
   (void)fprintf(fp, "%02d", indent - 1);
 
   int i;
   for (i = 0; i < indent; i++) { (void)fprintf(fp, "  "); }
 
-  const SbVec3f & vmin = nodesize.getMin();
-  const SbVec3f & vmax = nodesize.getMax();
+  const SbVec3f & vmin = this->nodesize.getMin();
+  const SbVec3f & vmax = this->nodesize.getMax();
 
   (void)fprintf(fp, "%s, %d items, ",
                 this->isLeaf() ? "Leaf" : "Group", this->totalNumberOfItems());
@@ -219,29 +223,26 @@ SbOctTreeNode::debugTree(FILE *fp, const int indent, const SbBox3f & nodesize)
   (void)fprintf(fp, "\n");
 
   if (this->isGroup()) {
-    SbBox3f childboxes[8];
-    this->splitBox(nodesize, childboxes);
-
     for (i = 0; i < 8; i++) {
-      this->children[i]->debugTree(fp, indent+1, childboxes[i]);
+      this->children[i]->debugTree(fp, indent+1);
     }
   }
 }
 
 static void
-add_to_array(SbList <void*> & array, void * ptr)
+add_to_array(SbList<void *> & array, void * ptr)
 {
-  int i, n= array.getLength();
-  for (i = 0; i < n; i++) {
-    if (array[i] == ptr) return;
-  }
-  array.append(ptr);
+  // FIXME: this is rather awful, resulting in n^2 algorithm time.
+  // Should change to using the array as a sorted set. 20050512 mortene.
+  if (array.find(ptr) == -1) { array.append(ptr); }
 }
 
-SbOctTreeNode::SbOctTreeNode(void)
+SbOctTreeNode::SbOctTreeNode(const SbBox3f & b)
 {
-  for (int i = 0; i < 8; i++)
+  for (int i = 0; i < 8; i++) {
     this->children[i] = NULL;
+  }
+  this->nodesize = b;
 }
 
 SbOctTreeNode::~SbOctTreeNode()
@@ -254,24 +255,20 @@ SbOctTreeNode::~SbOctTreeNode()
 void
 SbOctTreeNode::addItem(void * const item,
                        const SbOctTreeFuncs & itemfuncs,
-                       const int maxitems,
-                       const SbBox3f & nodesize)
+                       const int maxitems)
 {
   if (this->isGroup()) { // node has been split
-    SbBox3f childbox[8];
-    this->splitBox(nodesize, childbox);
     for (int i = 0; i < 8; i++) {
-      if (itemfuncs.insideboxfunc(item, childbox[i])) {
-        this->children[i]->addItem(item, itemfuncs,
-                                   maxitems, childbox[i]);
+      if (itemfuncs.insideboxfunc(item, this->children[i]->nodesize)) {
+        this->children[i]->addItem(item, itemfuncs, maxitems);
       }
     }
   }
   else if (this->items.getLength() >= maxitems) {
     // avoid trying a split too often by using a modulo
     if ((this->items.getLength() % (maxitems+1) == maxitems) &&
-        this->splitNode(nodesize, itemfuncs)) {
-      this->addItem(item, itemfuncs, maxitems, nodesize);
+        this->splitNode(itemfuncs)) {
+      this->addItem(item, itemfuncs, maxitems);
     }
     else {
       this->items.append(item);
@@ -283,17 +280,12 @@ SbOctTreeNode::addItem(void * const item,
 }
 
 void
-SbOctTreeNode::removeItem(void *const item,
-                          const SbOctTreeFuncs & itemfuncs,
-                          const SbBox3f & nodesize)
+SbOctTreeNode::removeItem(void * const item, const SbOctTreeFuncs & itemfuncs)
 {
   if (children[0]) {
-    SbBox3f childbox[8];
-    this->splitBox(nodesize, childbox);
     for (int i = 0; i < 8; i++) {
-      if (itemfuncs.insideboxfunc(item, childbox[i])) {
-        this->children[i]->removeItem(item, itemfuncs,
-                                      childbox[i]);
+      if (itemfuncs.insideboxfunc(item, this->children[i]->nodesize)) {
+        this->children[i]->removeItem(item, itemfuncs);
       }
     }
   }
@@ -312,17 +304,13 @@ void
 SbOctTreeNode::findItems(const SbVec3f & pos,
                          SbList <void*> & destarray,
                          const SbOctTreeFuncs & itemfuncs,
-                         const SbBox3f & nodesize,
                          const SbBool removeduplicates) const
 {
   if (this->isGroup()) {
-    SbBox3f childbox[8];
-    splitBox(nodesize, childbox);
     for (int i = 0; i < 8; i++) {
-      if (point_inside_box(pos, childbox[i])) {
-        children[i]->findItems(pos, destarray,
-                               itemfuncs, childbox[i],
-                               removeduplicates);
+      if (point_inside_box(pos, this->children[i]->nodesize)) {
+        this->children[i]->findItems(pos, destarray,
+                                     itemfuncs, removeduplicates);
       }
     }
   }
@@ -344,17 +332,13 @@ void
 SbOctTreeNode::findItems(const SbBox3f & box,
                          SbList <void*> & destarray,
                          const SbOctTreeFuncs & itemfuncs,
-                         const SbBox3f & nodesize,
                          const SbBool removeduplicates) const
 {
   if (this->isGroup()) {
-    SbBox3f childbox[8];
-    splitBox(nodesize, childbox);
     for (int i = 0; i < 8; i++) {
-      if (intersect_box_box(box, childbox[i]))
+      if (intersect_box_box(box, this->children[i]->nodesize))
         this->children[i]->findItems(box, destarray,
-                                     itemfuncs, childbox[i],
-                                     removeduplicates);
+                                     itemfuncs, removeduplicates);
     }
   }
   else {
@@ -375,17 +359,13 @@ void
 SbOctTreeNode::findItems(const SbSphere & sphere,
                          SbList <void*> & destarray,
                          const SbOctTreeFuncs & itemfuncs,
-                         const SbBox3f & nodesize,
                          const SbBool removeduplicates) const
 {
   if (this->isGroup()) {
-    SbBox3f childbox[8];
-    splitBox(nodesize, childbox);
     for (int i = 0; i < 8; i++) {
-      if (intersect_box_sphere(childbox[i],sphere))
+      if (intersect_box_sphere(this->children[i]->nodesize, sphere))
         this->children[i]->findItems(sphere, destarray,
-                                     itemfuncs, childbox[i],
-                                     removeduplicates);
+                                     itemfuncs, removeduplicates);
     }
   }
   else {
@@ -407,20 +387,13 @@ SbOctTreeNode::findItems(const SbPlane * const planes,
                          const int numplanes,
                          SbList <void*> & destarray,
                          const SbOctTreeFuncs & itemfuncs,
-                         const SbBox3f & nodesize,
                          const SbBool removeduplicates) const
 {
   if (this->isGroup()) {
-    SbBox3f childbox[8];
-    splitBox(nodesize, childbox);
     for (int i = 0; i < 8; i++) {
-      if (box_inside_planes(childbox[i], planes, numplanes)) {
-        this->children[i]->findItems(planes,
-                                     numplanes,
-                                     destarray,
-                                     itemfuncs,
-                                     childbox[i],
-                                     removeduplicates);
+      if (box_inside_planes(this->children[i]->nodesize, planes, numplanes)) {
+        this->children[i]->findItems(planes, numplanes,
+                                     destarray, itemfuncs, removeduplicates);
       }
     }
   }
@@ -439,7 +412,7 @@ SbOctTreeNode::findItems(const SbPlane * const planes,
 }
 
 void
-SbOctTreeNode::splitBox(const SbBox3f & box, SbBox3f * dest) const
+SbOctTreeNode::split3Way(const SbBox3f & box, SbBox3f * dest)
 {
   SbVec3f mid = (box.getMin() + box.getMax()) * 0.5f;
 
@@ -454,22 +427,21 @@ SbOctTreeNode::splitBox(const SbBox3f & box, SbBox3f * dest) const
 }
 
 SbBool
-SbOctTreeNode::splitNode(const SbBox3f & nodesize,
-                         const SbOctTreeFuncs & itemfuncs)
+SbOctTreeNode::splitNode(const SbOctTreeFuncs & itemfuncs)
 {
-  int i;
-  for (i = 0; i < 8; i++)
-    this->children[i] = new SbOctTreeNode();
-
   SbBox3f childbox[8];
-  this->splitBox(nodesize, childbox);
+  SbOctTreeNode::split3Way(this->nodesize, childbox);
+  int i;
+  for (i = 0; i < 8; i++) {
+    this->children[i] = new SbOctTreeNode(childbox[i]);
+  }
 
   const int n = this->items.getLength();
   for (i = 0; i < n; i++) {
     void *item = this->items[i];
     for (int j = 0; j < 8; j++) {
       if (itemfuncs.insideboxfunc(item, childbox[j])) {
-        children[j]->items.append(item);
+        this->children[j]->items.append(item);
       }
     }
   }
@@ -478,23 +450,25 @@ SbOctTreeNode::splitNode(const SbBox3f & nodesize,
   // from the parent node (i.e. this node). If so, the split won't
   // gain us any in processing time (it will likely be hurtful), so
   // decide against splitting.
-  int cntnodes = 0;
+
   for (i = 0; i < 8; i++) {
-    const int len = children[i]->items.getLength();
-    if (len == n) cntnodes++;
-  }
-  if (cntnodes > 0) {
-    for (i = 0; i < 8; i++) {
-      delete children[i];
-      children[i] = NULL;
+    if (this->children[i]->items.getLength() == n) {
+      for (int j = 0; j < 8; j++) {
+        delete this->children[j];
+        this->children[j] = NULL;
+      }
+      return FALSE;
     }
-    return FALSE;
   }
 
+  // Box was indeed split, we're now a group node, so truncate our
+  // list of items and carry on with new tree structure.
+
   this->items.truncate(0, TRUE);
   return TRUE;
 }
 
+// *************************************************************************
 
 /*!
   Constructor.
@@ -502,8 +476,7 @@ SbOctTreeNode::splitNode(const SbBox3f & nodesize,
 SbOctTree::SbOctTree(const SbBox3f & bbox,
                      const SbOctTreeFuncs & itemfuncs,
                      const int maxitems)
-  : boundingbox(bbox),
-    topnode(new SbOctTreeNode()),
+  : topnode(new SbOctTreeNode(bbox)),
     itemfuncs(itemfuncs),
     maxitemspernode(maxitems)
 {
@@ -524,8 +497,9 @@ SbOctTree::~SbOctTree()
 void
 SbOctTree::clear(void)
 {
+  SbBox3f bbox = this->topnode->getBBox();
   delete this->topnode;
-  this->topnode = new SbOctTreeNode();
+  this->topnode = new SbOctTreeNode(bbox);
 }
 
 /*!
@@ -534,12 +508,37 @@ SbOctTree::clear(void)
 void
 SbOctTree::addItem(void * const item)
 {
-  assert(this->itemfuncs.insideboxfunc(item, this->boundingbox) &&
+  // Note that the assert() below can hit due to floating point
+  // inaccuracies.
+  //
+  // When that happens, an easy and fairly decent fix is usually to
+  // add a bit of slack on the caller side to the input bbox argument
+  // to SbOctTree::SbOctTree().
+
+  // FIXME: a better solution would be to not force an static bbox
+  // upon the SbOctTree through its constructor, but let it
+  // dynamically expand / re-structure itself as items are added.
+  //
+  // An easy, but a bit inefficient, way to do that would be to simply
+  // store a copy of all items in the octtree structure, destruct it,
+  // restore a new top-level node, and then re-add all items to let a
+  // new octtree structure build itself.
+  //
+  // 20050512 mortene.
+#if COIN_DEBUG && 0 // debug
+  const SbBox3f & b = this->topnode->getBBox();
+  if (!this->itemfuncs.insideboxfunc(item, b)) {
+    const SbVec3f & bmin = b.getMin();
+    const SbVec3f & bmax = b.getMax();
+    SoDebugError::post("SbOctTree::addItem",
+                       "tree bbox==<%f, %f, %f>, <%f, %f, %f>",
+                       bmin[0], bmin[1], bmin[2], bmax[0], bmax[1], bmax[2]);
+  }
+#endif // debug
+  assert(this->itemfuncs.insideboxfunc(item, this->topnode->getBBox()) &&
          "bbox of item outside the octtree top-level bbox");
 
-  this->topnode->addItem(item, this->itemfuncs,
-                         this->maxitemspernode,
-                         this->boundingbox);
+  this->topnode->addItem(item, this->itemfuncs, this->maxitemspernode);
 }
 
 /*!
@@ -549,8 +548,7 @@ SbOctTree::addItem(void * const item)
 void
 SbOctTree::removeItem(void * const item)
 {
-  this->topnode->removeItem(item, this->itemfuncs,
-                            this->boundingbox);
+  this->topnode->removeItem(item, this->itemfuncs);
 }
 
 /*!
@@ -561,17 +559,30 @@ SbOctTree::removeItem(void * const item)
   contain duplicate items. This is not an optimized process, so if
   you're looking for speed you should set this to FALSE and do
   your own postprocessing of the array of returned items.
+
+  \DANGEROUS_ALLOC_RETURN
 */
 void
 SbOctTree::findItems(const SbVec3f & pos,
                      SbList <void*> & destarray,
                      const SbBool removeduplicates) const
 {
+  // FIXME: passing in an SbList is dangerous under MS Windows, as
+  // allocation and deallocation can then happen on different
+  // C-library's heaps. The other findItems() functions below have the
+  // same problem. 20050512 mortene.
+
+  // FIXME: should be straightforward to drop the removeduplicates
+  // argument -- it's a hack. We just need to optimize the
+  // add_to_array() function above to keep a _sorted_ array.
+  //
+  // This also goes for the other findItems() functions below, of
+  // course.
+  //
+  // 20050512 mortene.
+
   assert(this->itemfuncs.ptinsidefunc);
-  topnode->findItems(pos, destarray,
-                     this->itemfuncs,
-                     this->boundingbox,
-                     removeduplicates);
+  topnode->findItems(pos, destarray, this->itemfuncs, removeduplicates);
 }
 
 /*!
@@ -581,16 +592,15 @@ SbOctTree::findItems(const SbVec3f & pos,
   contain duplicate items. This is not an optimized process, so if
   you're looking for speed you should set this to FALSE and do
   your own postprocessing of the array of returned items.
+
+  \DANGEROUS_ALLOC_RETURN
 */
 void
 SbOctTree::findItems(const SbBox3f & box, SbList <void*> & destarray,
                      const SbBool removeduplicates) const
 {
   assert(this->itemfuncs.insideboxfunc);
-  this->topnode->findItems(box, destarray,
-                           this->itemfuncs,
-                           this->boundingbox,
-                           removeduplicates);
+  this->topnode->findItems(box, destarray, this->itemfuncs, removeduplicates);
 }
 
 /*!
@@ -600,6 +610,8 @@ SbOctTree::findItems(const SbBox3f & box, SbList <void*> & destarray,
   contain duplicate items. This is not an optimized process, so if
   you're looking for speed you should set this to FALSE and do
   your own postprocessing of the array of returned items.
+
+  \DANGEROUS_ALLOC_RETURN
 */
 void
 SbOctTree::findItems(const SbSphere & sphere,
@@ -607,10 +619,7 @@ SbOctTree::findItems(const SbSphere & sphere,
                      const SbBool removeduplicates) const
 {
   assert(this->itemfuncs.insidespherefunc);
-  this->topnode->findItems(sphere, destarray,
-                           this->itemfuncs,
-                           this->boundingbox,
-                           removeduplicates);
+  this->topnode->findItems(sphere, destarray, this->itemfuncs, removeduplicates);
 }
 
 /*!
@@ -623,6 +632,8 @@ SbOctTree::findItems(const SbSphere & sphere,
   contain duplicate items. This is not an optimized process, so if
   you're looking for speed you should set this to FALSE and do
   your own postprocessing of the array of returned items.
+
+  \DANGEROUS_ALLOC_RETURN
 */
 void
 SbOctTree::findItems(const SbPlane * const planes,
@@ -631,8 +642,7 @@ SbOctTree::findItems(const SbPlane * const planes,
                      const SbBool removeduplicates) const
 {
   assert(this->itemfuncs.insideplanesfunc);
-  this->topnode->findItems(planes, numplanes, destarray,
-                           this->itemfuncs, this->boundingbox,
+  this->topnode->findItems(planes, numplanes, destarray, this->itemfuncs,
                            removeduplicates);
 }
 
@@ -644,12 +654,12 @@ SbOctTree::findItems(const SbPlane * const planes,
 const SbBox3f &
 SbOctTree::getBoundingBox(void) const
 {
-  return this->boundingbox;
+  return this->topnode->getBBox();
 }
 
 void 
 SbOctTree::debugTree(FILE * fp)
 {
   fprintf(fp, "Oct Tree:\n");
-  if (this->topnode) this->topnode->debugTree(fp, 1, this->boundingbox);
+  if (this->topnode) { this->topnode->debugTree(fp, 1); }
 }
