@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -30,9 +30,7 @@
   representation and calculations on time values of high resolution.
 */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif // HAVE_CONFIG_H
+// *************************************************************************
 
 #include <assert.h>
 #include <stdlib.h>
@@ -41,6 +39,10 @@
 #include <errno.h>
 #include <math.h>
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif // HAVE_CONFIG_H
+
 #ifdef HAVE_TIME_H
 #include <time.h> // struct timeval (Linux)
 #endif // HAVE_TIME_H
@@ -59,9 +61,27 @@
 #include <Inventor/C/base/time.h>
 #include <Inventor/C/tidbits.h>
 
+// *************************************************************************
+
+// FIXME: I don't agree with this willy-nilly, "gut feeling"
+// constant. As far as I can tell from a quick look, the exceptions in
+// the code using this should be handled in other ways.
+//
+// E.g. this:
+//
+//   if (s==0.0) { this->dtime /= s + SMALLEST_DOUBLE_TIMEUNIT; }
+//
+// ..could better be written as
+//
+//   if (s==0.0) { this->dtime = DBL_MAX; }
+//
+// If there are no protests, I'll take care of removing this constant.
+//
+// 20050526 mortene.
 
 static const double SMALLEST_DOUBLE_TIMEUNIT  = 1.0/1000000.0;
 
+// *************************************************************************
 
 /*!
   The default constructor sets up a time instance of 0 seconds.
@@ -102,7 +122,8 @@ SbTime::SbTime(const struct timeval * const tv)
 /*!
   Returns an SbTime instance with the current clock time. The current
   time will be given as a particular number of seconds and
-  microseconds since 00:00:00.00 January 1st 1970.
+  microseconds since 00:00:00 January 1, 1970, in Coordinated
+  Universal Time (UTC).
 
   \sa setToTimeOfDay().
  */
@@ -255,6 +276,12 @@ SbTime::getValue(time_t & sec, long & usec) const
 void
 SbTime::getValue(struct timeval * tv) const
 {
+  // FIXME: the below gives a warning with MSVC 7 on 64-bit Windows,
+  // as the struct timeval::tv_sec value seems to be of type long
+  // there. Ditto for the tv_usec value further below.
+  //
+  // I guess we need a configure check to find the correct type to
+  // cast to here, but investigate. 20050525 mortene.
   tv->tv_sec = (time_t)(this->dtime);
   double us = fmod(this->dtime, 1.0) * 1000000.0;
   tv->tv_usec = (time_t)(us + (us < 0.0 ? -0.5 : 0.5));
@@ -337,6 +364,9 @@ SbTime::getMsecValue(void) const
   %U - number of microseconds.<BR>
   %u - remaining microseconds after subtracting the total number of mseconds.<BR>
 
+  The result shows UTC time, not corrected for local time zone
+  nor daylight savings time.
+  
   \sa formatDate().
  */
 SbString
@@ -464,6 +494,9 @@ SbTime::format(const char * const fmt) const
 
   The value of SbTime will be interpreted as seconds since 00:00:00
   1970-01-01.
+  
+  The result shows local time, according to local time zone and
+  daylight savings time (if and when applicable).
 
   \sa format().
 */
@@ -481,15 +514,15 @@ SbTime::formatDate(const char * const fmt) const
 
   if (strlen(format) == 0) return SbString("");
 
-  const int buffersize = 256;
+  const size_t buffersize = 256;
   char buffer[buffersize];
   char * bufferpt = buffer;
   time_t secs = (time_t)(this->dtime);
-  int currentsize = buffersize;
+  size_t currentsize = buffersize;
 
   struct tm * ts = localtime(&secs);
 
-  int ret = strftime(bufferpt, currentsize, format, ts);
+  size_t ret = strftime(bufferpt, currentsize, format, ts);
   if ((ret == 0) || (ret == currentsize)) {
     bufferpt = NULL;
     // The resulting string was too large, so we will allocate
