@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -42,26 +42,26 @@
   be visible outside it. Click 'Esc' again to use 'U' and 'D'.
 
   \code
-  // Copyright (C) 2000-2004 by Systems in Motion. All rights reserved.
-  
+  // Copyright (C) 2000-2005 by Systems in Motion. All rights reserved.
+
   #include <Inventor/Qt/SoQt.h>
   #include <Inventor/Qt/viewers/SoQtExaminerViewer.h>
   #include <Inventor/nodes/SoEventCallback.h>
   #include <Inventor/nodes/SoSeparator.h>
   #include <Inventor/nodes/SoCone.h>
   #include <Inventor/events/SoKeyboardEvent.h>
-  
+
   // ************************************************************
-  
+
   static void
   keypresscbfunc(void * userdata, SoEventCallback * keyboardcb)
   {
     SoQtExaminerViewer * viewer = (SoQtExaminerViewer *)userdata;
-  
+
     const SoEvent * event = keyboardcb->getEvent();
-    
+
     int shift = 0;
-  
+
     if (SO_KEY_PRESS_EVENT(event, U)) {
       shift = 40;
       keyboardcb->setHandled();
@@ -70,7 +70,7 @@
       shift = -40;
       keyboardcb->setHandled();
     }
-  
+
     if (keyboardcb->isHandled()) {
       SbViewportRegion vpr = viewer->getViewportRegion();
       SbVec2s size = vpr.getViewportSizePixels();
@@ -80,34 +80,34 @@
       viewer->setViewportRegion(vpr);
     }
   }
-  
+
   // ************************************************************
-  
+
   int
   main(int argc, char ** argv)
   {
     QWidget * window = SoQt::init(argv[0]);
-  
+
     SoSeparator * root = new SoSeparator;
     root->ref();
-  
+
     root->addChild(new SoCone);
-  
+
     SoQtExaminerViewer * viewer = new SoQtExaminerViewer(window);
-  
+
     SoEventCallback * eventcb = new SoEventCallback;
     eventcb->addEventCallback(SoKeyboardEvent::getClassTypeId(),
                               keypresscbfunc, viewer);
     root->insertChild(eventcb, 0);
-  
+
     viewer->setViewing(FALSE);
     viewer->setDecoration(FALSE);
     viewer->setSceneGraph(root);
     viewer->show();
     SoQt::show(window);
-  
+
     SoQt::mainLoop();
-  
+
     delete viewer;
     root->unref();
     return 0;
@@ -399,6 +399,7 @@ SbViewportRegion::getViewportOriginPixels(void) const
 {
   // Cast away constness. Ugly.
   SbViewportRegion * thisp = (SbViewportRegion *)this;
+  // FIXME: the tmp storage seems totally unnecessary? 20040213 mortene.
   thisp->vporigin_s.setValue(round2short(this->winsize[0] * this->vporigin[0]),
                              round2short(this->winsize[1] * this->vporigin[1]));
   return this->vporigin_s;
@@ -425,6 +426,7 @@ SbViewportRegion::getViewportSizePixels(void) const
 {
   // Cast away constness. Ugly.
   SbViewportRegion * thisp = (SbViewportRegion *)this;
+  // FIXME: the tmp storage seems totally unnecessary? 20040213 mortene.
   thisp->vpsize_s = SbVec2s(round2short(this->winsize[0] * this->vpsize[0]),
                             round2short(this->winsize[1] * this->vpsize[1]));
   return this->vpsize_s;
@@ -452,7 +454,7 @@ SbViewportRegion::getViewportAspectRatio(void) const
   if this causes the viewport origin to be moved below (0,0), the
   origin coordinates will be clamped.
 
-  \sa scaleHeight().  
+  \sa scaleHeight().
 */
 void
 SbViewportRegion::scaleWidth(float ratio)
@@ -550,11 +552,10 @@ SbViewportRegion::getPixelsPerPoint(void) const
 
 /*!
   \relates SbViewportRegion
-
   Compares two SbViewportRegion instances for equality.
- */
+*/
 int
-operator ==(const SbViewportRegion& reg1, const SbViewportRegion& reg2)
+operator==(const SbViewportRegion & reg1, const SbViewportRegion & reg2)
 {
   return
     reg1.winsize == reg2.winsize &&
@@ -563,6 +564,18 @@ operator ==(const SbViewportRegion& reg1, const SbViewportRegion& reg2)
     reg1.pixperinch == reg2.pixperinch;
 }
 
+/*!
+  \relates SbViewportRegion
+  Compares two SbViewportRegion instances for inequality.
+
+  \since Coin 2.4
+*/
+int
+operator!=(const SbViewportRegion & reg1, const SbViewportRegion & reg2)
+{
+  return !(reg1 == reg2);
+}
+
 /*!
   Dump the state of this object to the \a file stream. Only works in
   debug version of library, method does nothing in an optimized compile.
