@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -735,7 +735,13 @@ SbXfBox3f::getVolume(void) const
   // in the coordinate system specified by a 3x3 matrix, and that we
   // can find the volume of our box by multiplying the volume of the
   // orthogonal box with the determinant of the upper-left 3x3 matrix.
-  return SbBox3f::getVolume() * this->matrix.det3();
+
+  float volume = (SbBox3f::getVolume() * this->matrix.det3());
+
+  // The determinant might be negative if e.g. negative scaling has
+  // been performed on the matrix. To rectify this, we make sure the
+  // returned volume is positive.
+  return (volume > 0) ? volume : -volume;
 }
 
 /*!
