@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -38,10 +38,13 @@
 #include <Inventor/elements/SoCacheElement.h>
 #include <Inventor/elements/SoGLCacheContextElement.h>
 #include <Inventor/elements/SoGLLazyElement.h>
+#include <Inventor/elements/SoShapeStyleElement.h>
 #include <Inventor/errors/SoDebugError.h>
 #include <Inventor/misc/SoGL.h>
 #include <Inventor/misc/SoState.h>
+#include <Inventor/misc/SoContextHandler.h>
 #include <Inventor/system/gl.h>
+#include <Inventor/C/glue/glp.h>
 
 // *************************************************************************
 
@@ -71,10 +74,27 @@ public:
   SbBool needclose;
   SoElement * invalidelement;
   int numframesok;
+
+  //
+  // Callback from SoContextHandler
+  //
+  static void contextCleanup(uint32_t context, void * closure) {
+    SoGLCacheListP * thisp = (SoGLCacheListP*) closure;
+
+    int i = 0;
+    int n = thisp->itemlist.getLength();
+    while (i < n) {
+      if (thisp->itemlist[i]->getCacheContext() == (int) context) {
+        thisp->itemlist[i]->unref();
+        thisp->itemlist.remove(i);
+        n--;
+      }
+      else i++;
+    }
+  }
 };
 
-#undef THIS
-#define THIS this->pimpl
+#define PRIVATE(obj) ((obj)->pimpl)
 
 // *************************************************************************
 
@@ -83,15 +103,15 @@ public:
 */
 SoGLCacheList::SoGLCacheList(int numcaches)
 {
-  THIS = new SoGLCacheListP;
-  THIS->numcaches = numcaches;
-  THIS->opencache = NULL;
-  THIS->autocachebits = 0;
-  THIS->numused = 0;
-  THIS->numdiscarded = 0;
-  THIS->needclose = FALSE;
-  THIS->invalidelement = NULL;
-  THIS->numframesok = 0;
+  PRIVATE(this) = new SoGLCacheListP;
+  PRIVATE(this)->numcaches = numcaches;
+  PRIVATE(this)->opencache = NULL;
+  PRIVATE(this)->autocachebits = 0;
+  PRIVATE(this)->numused = 0;
+  PRIVATE(this)->numdiscarded = 0;
+  PRIVATE(this)->needclose = FALSE;
+  PRIVATE(this)->invalidelement = NULL;
+  PRIVATE(this)->numframesok = 0;
 
   // auto caching must be enabled using an environment variable
   if (COIN_AUTO_CACHING < 0) {
@@ -106,6 +126,8 @@ SoGLCacheList::SoGLCacheList(int numcaches)
     else COIN_DEBUG_CACHING = 0;
   }
 #endif // debug
+  
+  SoContextHandler::addContextDestructionCallback(SoGLCacheListP::contextCleanup, PRIVATE(this));
 }
 
 /*!
@@ -113,11 +135,12 @@ SoGLCacheList::SoGLCacheList(int numcaches)
 */
 SoGLCacheList::~SoGLCacheList()
 {
-  const int n = THIS->itemlist.getLength();
+  SoContextHandler::removeContextDestructionCallback(SoGLCacheListP::contextCleanup, PRIVATE(this));
+  const int n = PRIVATE(this)->itemlist.getLength();
   for (int i = 0; i < n; i++) {
-    THIS->itemlist[i]->unref();
+    PRIVATE(this)->itemlist[i]->unref();
   }
-  delete THIS;
+  delete PRIVATE(this);
 }
 
 /*!
@@ -129,7 +152,7 @@ SbBool
 SoGLCacheList::call(SoGLRenderAction * action)
 {
   // do a quick return if there are no caches in the list
-  int n = THIS->itemlist.getLength();
+  int n = PRIVATE(this)->itemlist.getLength();
   if (n == 0) return FALSE;
 
   int i;
@@ -137,7 +160,7 @@ SoGLCacheList::call(SoGLRenderAction * action)
   int context = SoGLCacheContextElement::get(state);
 
   for (i = 0; i < n; i++) {
-    SoGLRenderCache * cache = THIS->itemlist[i];
+    SoGLRenderCache * cache = PRIVATE(this)->itemlist[i];
     if (cache->getCacheContext() == context) {
       if (cache->isValid(state) && 
           SoGLLazyElement::preCacheCall(state, cache->getPreLazyState())) {
@@ -146,14 +169,14 @@ SoGLCacheList::call(SoGLRenderAction * action)
         // the end of the list, and the LRU will be the first
         // item. This makes it easy to choose a cache to destroy when
         // the maximum number of caches is exceeded.
-        THIS->itemlist.remove(i);
-        THIS->itemlist.append(cache);
+        PRIVATE(this)->itemlist.remove(i);
+        PRIVATE(this)->itemlist.append(cache);
         // update lazy GL state before calling cache
         SoGLLazyElement::getInstance(state)->send(state, SoLazyElement::ALL_MASK);
         cache->call(state);
         SoGLLazyElement::postCacheCall(state, cache->getPostLazyState());
         cache->unref(state);
-        THIS->numused++;
+        PRIVATE(this)->numused++;
 
 #if COIN_DEBUG
         // The GL error test is default disabled for this optimized
@@ -179,7 +202,7 @@ SoGLCacheList::call(SoGLRenderAction * action)
                                "An OpenGL error (%s) was detected after a "
                                "renderlist invocation. This shouldn't happen, "
                                "low-level debugging is needed.",
-                               sogl_glerror_string(err).getString());
+                               coin_glerror_string(err));
           }
         }
 
@@ -193,7 +216,7 @@ SoGLCacheList::call(SoGLRenderAction * action)
         // #Inventor V2.1 ascii
         //
         //  Separator {
-        //   renderCaching OFF
+        //   # renderCaching OFF
         //
         //   Coordinate3 {
         //      point [ 903.23315 -292.8786 -261.79764,
@@ -362,7 +385,12 @@ SoGLCacheList::call(SoGLRenderAction * action)
         //      vKnotVector [ 0, 0, 0, 0,
         //       871.44604, 1419.9783, 1419.9783, 1419.9783,
         //       1419.9783 ]
+        //   }
+        // }
         // -----8<---- [snip] ------------------8<---- [snip] -------------
+        //
+        // UPDATE: the above now works without errors on my Linux dev
+        // machine with NVidia v 61.11 drivers. 20050309 mortene.
 
 #endif // COIN_DEBUG
 
@@ -376,7 +404,7 @@ SoGLCacheList::call(SoGLRenderAction * action)
                            "no valid cache found for %p. Node has %d caches",
                            this, n);
     for (i = 0; i < n; i++) {
-      SoGLRenderCache * cache = THIS->itemlist[i];
+      SoGLRenderCache * cache = PRIVATE(this)->itemlist[i];
       if (cache->getCacheContext() == context) {
         SoDebugError::postInfo("SoGLCacheList::call",
                                "cache %d isValid()? %s", i, cache->isValid(state) ? "TRUE" : "FALSE");        
@@ -405,65 +433,71 @@ void
 SoGLCacheList::open(SoGLRenderAction * action, SbBool autocache)
 {
   // needclose is used to quickly return in close()
-  if (THIS->numcaches == 0 || (autocache && COIN_AUTO_CACHING == 0)) {
-    THIS->needclose = FALSE;
+  if (PRIVATE(this)->numcaches == 0 || (autocache && COIN_AUTO_CACHING == 0)) {
+    PRIVATE(this)->needclose = FALSE;
     return;
   }
 
-  THIS->needclose = TRUE;
+  PRIVATE(this)->needclose = TRUE;
 
-  assert(THIS->opencache == NULL);
+  assert(PRIVATE(this)->opencache == NULL);
   SoState * state = action->getState();
 
   // will be restored in close()
-  THIS->savedinvalid = SoCacheElement::setInvalid(FALSE);
+  PRIVATE(this)->savedinvalid = SoCacheElement::setInvalid(FALSE);
 
   if (SoCacheElement::anyOpen(state)) return;
 
   SbBool shouldcreate = FALSE;
   if (!autocache) {
-    if (THIS->numframesok >= 1) shouldcreate = TRUE;
+    if (PRIVATE(this)->numframesok >= 1) shouldcreate = TRUE;
   }
   else {
-    if (THIS->numframesok >= 2 && 
-        (THIS->autocachebits == SoGLCacheContextElement::DO_AUTO_CACHE)) {
+    if (PRIVATE(this)->numframesok >= 2 && 
+        (PRIVATE(this)->autocachebits == SoGLCacheContextElement::DO_AUTO_CACHE)) {
       shouldcreate = TRUE;
 
 #if COIN_DEBUG
-      if (COIN_DEBUG_CACHING && THIS->numframesok >= 2) {
+      if (COIN_DEBUG_CACHING && PRIVATE(this)->numframesok >= 2) {
         SoDebugError::postInfo("SoGLCacheList::open",
                                "consider cache create: %p. numframesok: %d, numused: %d, numdiscarded: %d",
-                               this, THIS->numframesok, THIS->numused, THIS->numdiscarded);
+                               this, PRIVATE(this)->numframesok, PRIVATE(this)->numused, PRIVATE(this)->numdiscarded);
       }
 #endif // debug
 
     }
   }
 
-  if (shouldcreate) {
+  if (shouldcreate && autocache) {
     // determine if we really should create a new cache, based on numused and numdiscarded
-    double docreate = (double) (THIS->numframesok + THIS->numused);
-    double dontcreate = (THIS->numdiscarded);
+    double docreate = (double) (PRIVATE(this)->numframesok + PRIVATE(this)->numused);
+    double dontcreate = (PRIVATE(this)->numdiscarded);
     dontcreate *= dontcreate;
     dontcreate *= dontcreate;
-
+    
     if (dontcreate >= docreate) shouldcreate = FALSE;
   }
 
   if (shouldcreate) {
-    if (THIS->itemlist.getLength() >= THIS->numcaches) {
+    if (PRIVATE(this)->itemlist.getLength() >= PRIVATE(this)->numcaches) {
       // the cache at position 0 will be the LRU cache. Remove it.
-      SoGLRenderCache * cache = THIS->itemlist[0];
+      SoGLRenderCache * cache = PRIVATE(this)->itemlist[0];
       cache->unref(state);
-      THIS->itemlist.remove(0);
-      THIS->numdiscarded++;
+      PRIVATE(this)->itemlist.remove(0);
+      PRIVATE(this)->numdiscarded++;
     }
-    THIS->opencache = new SoGLRenderCache(state);
-    THIS->opencache->ref();
-    SoCacheElement::set(state, THIS->opencache);
-    SoGLLazyElement::beginCaching(state, THIS->opencache->getPreLazyState(),
-                                  THIS->opencache->getPostLazyState());
-    THIS->opencache->open(state);
+    PRIVATE(this)->opencache = new SoGLRenderCache(state);
+    PRIVATE(this)->opencache->ref();
+    SoCacheElement::set(state, PRIVATE(this)->opencache);
+    SoGLLazyElement::beginCaching(state, PRIVATE(this)->opencache->getPreLazyState(),
+                                  PRIVATE(this)->opencache->getPostLazyState());
+    PRIVATE(this)->opencache->open(state);
+
+    // force a dependency on the transparecy type 
+    // FIXME: consider adding a new element for storing the
+    // transparency type.  The dependency tracking on the transparency
+    // type would then work automatically. pederb, 2005-02-18
+    (void) SoShapeStyleElement::get(state);
 
 #if COIN_DEBUG // debug
     if (COIN_DEBUG_CACHING) {
@@ -472,7 +506,7 @@ SoGLCacheList::open(SoGLRenderAction * action, SbBool autocache)
     }
 #endif // debug
   }
-  THIS->autocachebits = SoGLCacheContextElement::resetAutoCacheBits(state);
+  PRIVATE(this)->autocachebits = SoGLCacheContextElement::resetAutoCacheBits(state);
 }
 
 /*!
@@ -482,24 +516,24 @@ SoGLCacheList::open(SoGLRenderAction * action, SbBool autocache)
 void
 SoGLCacheList::close(SoGLRenderAction * action)
 {
-  if (!THIS->needclose) return;
+  if (!PRIVATE(this)->needclose) return;
 
   SoState * state = action->getState();
 
   // close open cache before accepting it or throwing it away
-  if (THIS->opencache) {
-    THIS->opencache->close();
+  if (PRIVATE(this)->opencache) {
+    PRIVATE(this)->opencache->close();
     SoGLLazyElement::endCaching(state);            
   }
-  if (SoCacheElement::setInvalid(THIS->savedinvalid)) {
+  if (SoCacheElement::setInvalid(PRIVATE(this)->savedinvalid)) {
     // notify parent caches
     SoCacheElement::setInvalid(TRUE);
-    THIS->numframesok = 0;
+    PRIVATE(this)->numframesok = 0;
     // just throw away the open cache, it's invalid
-    if (THIS->opencache) {
-      THIS->opencache->unref();
-      THIS->opencache = NULL;
-      THIS->numdiscarded += 1;
+    if (PRIVATE(this)->opencache) {
+      PRIVATE(this)->opencache->unref();
+      PRIVATE(this)->opencache = NULL;
+      PRIVATE(this)->numdiscarded += 1;
 
 #if COIN_DEBUG
       if (COIN_DEBUG_CACHING) {
@@ -510,24 +544,24 @@ SoGLCacheList::close(SoGLRenderAction * action)
     }
   }
   else {
-    THIS->numframesok++;
+    PRIVATE(this)->numframesok++;
   }
 
   // open cache is ok, add it to the cache list
-  if (THIS->opencache) {
+  if (PRIVATE(this)->opencache) {
 #if COIN_DEBUG
     if (COIN_DEBUG_CACHING) {
       SoDebugError::postInfo("SoGLCacheList::close",
                              "new cache created: %p", this);
     }
 #endif // debug
-    THIS->itemlist.append(THIS->opencache);
-    THIS->opencache = NULL;
+    PRIVATE(this)->itemlist.append(PRIVATE(this)->opencache);
+    PRIVATE(this)->opencache = NULL;
   }
 
   int bits = SoGLCacheContextElement::resetAutoCacheBits(state);
-  SoGLCacheContextElement::setAutoCacheBits(state, bits|THIS->autocachebits);
-  THIS->autocachebits = bits;
+  SoGLCacheContextElement::setAutoCacheBits(state, bits|PRIVATE(this)->autocachebits);
+  PRIVATE(this)->autocachebits = bits;
 }
 
 /*!
@@ -537,7 +571,7 @@ SoGLCacheList::close(SoGLRenderAction * action)
 void
 SoGLCacheList::invalidateAll(void)
 {
-  int n = THIS->itemlist.getLength();
+  int n = PRIVATE(this)->itemlist.getLength();
 #if COIN_DEBUG
   if (n && COIN_DEBUG_CACHING && 0) {
     SoDebugError::postInfo("SoGLCacheList::invalidateAll",
@@ -546,9 +580,11 @@ SoGLCacheList::invalidateAll(void)
 #endif // debug
 
   for (int i = 0; i < n; i++) {
-    THIS->itemlist[i]->unref();
+    PRIVATE(this)->itemlist[i]->unref();
   }
-  THIS->itemlist.truncate(0);
-  THIS->numdiscarded += n;
-  THIS->numframesok = 0;
+  PRIVATE(this)->itemlist.truncate(0);
+  PRIVATE(this)->numdiscarded += n;
+  PRIVATE(this)->numframesok = 0;
 }
+
+#undef PRIVATE
