@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -27,13 +27,18 @@
   \ingroup caches
 */
 
-#include <Inventor/caches/SoGLRenderCache.h>
-#include <Inventor/elements/SoGLCacheContextElement.h>
-#include <Inventor/elements/SoCacheElement.h>
-#include <Inventor/lists/SbList.h>
+// *************************************************************************
+
 #include <assert.h>
 
-#ifndef DOXYGEN_SKIP_THIS
+#include <Inventor/caches/SoGLRenderCache.h>
+#include <Inventor/elements/SoGLCacheContextElement.h>
+#include <Inventor/elements/SoGLDisplayList.h>
+#include <Inventor/elements/SoCacheElement.h>
+#include <Inventor/lists/SbList.h>
+
+// *************************************************************************
+
 class SoGLRenderCacheP {
 public:
   SoGLDisplayList * displaylist;
@@ -42,10 +47,10 @@ public:
   SoGLLazyElement::GLState prestate;
   SoGLLazyElement::GLState poststate;
 };
-#endif // DOXYGEN_SKIP_THIS
 
-#undef THIS
-#define THIS this->pimpl
+#define PRIVATE(obj) ((obj)->pimpl)
+
+// *************************************************************************
 
 /*!
   Constructor with \a state being the current state.
@@ -53,9 +58,9 @@ public:
 SoGLRenderCache::SoGLRenderCache(SoState * state)
   : SoCache(state)
 {
-  THIS = new SoGLRenderCacheP;
-  THIS->displaylist = NULL;
-  THIS->openstate = NULL;
+  PRIVATE(this) = new SoGLRenderCacheP;
+  PRIVATE(this)->displaylist = NULL;
+  PRIVATE(this)->openstate = NULL;
 }
 
 /*!
@@ -64,10 +69,10 @@ SoGLRenderCache::SoGLRenderCache(SoState * state)
 SoGLRenderCache::~SoGLRenderCache()
 {
   // stuff should have been deleted in destroy()
-  assert(THIS->displaylist == NULL);
-  assert(THIS->nestedcachelist.getLength() == 0);
+  assert(PRIVATE(this)->displaylist == NULL);
+  assert(PRIVATE(this)->nestedcachelist.getLength() == 0);
   
-  delete THIS;
+  delete PRIVATE(this);
 }
 
 /*!
@@ -79,13 +84,13 @@ SoGLRenderCache::~SoGLRenderCache()
 void
 SoGLRenderCache::open(SoState * state)
 {
-  assert(THIS->displaylist == NULL);
-  assert(THIS->openstate == NULL); // cache should not be open
-  THIS->openstate = state;
-  THIS->displaylist =
+  assert(PRIVATE(this)->displaylist == NULL);
+  assert(PRIVATE(this)->openstate == NULL); // cache should not be open
+  PRIVATE(this)->openstate = state;
+  PRIVATE(this)->displaylist =
     new SoGLDisplayList(state, SoGLDisplayList::DISPLAY_LIST);
-  THIS->displaylist->ref();
-  THIS->displaylist->open(state);
+  PRIVATE(this)->displaylist->ref();
+  PRIVATE(this)->displaylist->open(state);
 }
 
 /*!
@@ -96,10 +101,10 @@ SoGLRenderCache::open(SoState * state)
 void
 SoGLRenderCache::close(void)
 {
-  assert(THIS->openstate != NULL);
-  assert(THIS->displaylist != NULL);
-  THIS->displaylist->close(THIS->openstate);
-  THIS->openstate = NULL;
+  assert(PRIVATE(this)->openstate != NULL);
+  assert(PRIVATE(this)->displaylist != NULL);
+  PRIVATE(this)->displaylist->close(PRIVATE(this)->openstate);
+  PRIVATE(this)->openstate = NULL;
 }
 
 /*!
@@ -110,7 +115,7 @@ SoGLRenderCache::close(void)
 void
 SoGLRenderCache::call(SoState * state)
 {
-  assert(THIS->displaylist != NULL);
+  assert(PRIVATE(this)->displaylist != NULL);
 
   // FIXME: needed for nested caching, pederb 2002-01-24
   //  SoCacheElement::addCacheDependency(state, this);
@@ -118,7 +123,7 @@ SoGLRenderCache::call(SoState * state)
   // FIXME: this is just a temporary workaround until nested caching is
   // properly supported. pederb, 2002-01-24
   SoCacheElement::invalidate(state);
-  THIS->displaylist->call(state);
+  PRIVATE(this)->displaylist->call(state);
 
   if (state->isCacheOpen()) {
     // FIXME: support nested caching properly, pederb, 2002-01-24
@@ -134,7 +139,7 @@ SoGLRenderCache::call(SoState * state)
 int
 SoGLRenderCache::getCacheContext(void) const
 {
-  if (THIS->displaylist) return THIS->displaylist->getContext();
+  if (PRIVATE(this)->displaylist) return PRIVATE(this)->displaylist->getContext();
   return -1;
 }
 
@@ -158,33 +163,34 @@ void
 SoGLRenderCache::addNestedCache(SoGLDisplayList * child)
 {
   child->ref();
-  THIS->nestedcachelist.append(child);
+  PRIVATE(this)->nestedcachelist.append(child);
 }
 
 // Documented in superclass. Overridden to unref display lists.
 void
 SoGLRenderCache::destroy(SoState * state)
 {
-  int n = THIS->nestedcachelist.getLength();
+  int n = PRIVATE(this)->nestedcachelist.getLength();
   for (int i = 0; i < n; i++) {
-    THIS->nestedcachelist[i]->unref(state);
+    PRIVATE(this)->nestedcachelist[i]->unref(state);
   }
-  THIS->nestedcachelist.truncate(0);
-  if (THIS->displaylist) {
-    THIS->displaylist->unref(state);
-    THIS->displaylist = NULL;
+  PRIVATE(this)->nestedcachelist.truncate(0);
+  if (PRIVATE(this)->displaylist) {
+    PRIVATE(this)->displaylist->unref(state);
+    PRIVATE(this)->displaylist = NULL;
   }
 }
 
 SoGLLazyElement::GLState * 
 SoGLRenderCache::getPreLazyState(void)
 {
-  return &THIS->prestate;
+  return &PRIVATE(this)->prestate;
 }
 
 SoGLLazyElement::GLState * 
 SoGLRenderCache::getPostLazyState(void)
 {
-  return &THIS->poststate;
+  return &PRIVATE(this)->poststate;
 }
 
+#undef PRIVATE
