@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -26,27 +26,33 @@
   The SoPrimitiveVertexClass is used to cache generated triangles.
 */
 
+// *************************************************************************
+
 #include <Inventor/caches/SoPrimitiveVertexCache.h>
-#include <Inventor/lists/SbList.h>
-#include <Inventor/SbVec3f.h>
-#include <Inventor/misc/SbHash.h>
-#include <Inventor/elements/SoMultiTextureCoordinateElement.h>
-#include <Inventor/elements/SoMultiTextureEnabledElement.h>
-#include <Inventor/elements/SoBumpMapCoordinateElement.h>
-#include <Inventor/elements/SoLazyElement.h>
-#include <Inventor/elements/SoGLCacheContextElement.h>
-#include <Inventor/SoPrimitiveVertex.h>
-#include <Inventor/details/SoPointDetail.h>
-#include <Inventor/details/SoLineDetail.h>
-#include <Inventor/details/SoFaceDetail.h>
+
 #include <stddef.h>
 #include <stdlib.h>
-#include <Inventor/SbDict.h>
+#include <string.h>
+
 #include <Inventor/C/glue/gl.h>
-#include <Inventor/misc/SoGL.h>
-#include <string.h> // memcmp()
-#include <Inventor/system/gl.h>
 #include <Inventor/C/tidbits.h>
+#include <Inventor/SbVec3f.h>
+#include <Inventor/SoPrimitiveVertex.h>
+#include <Inventor/details/SoFaceDetail.h>
+#include <Inventor/details/SoLineDetail.h>
+#include <Inventor/details/SoPointDetail.h>
+#include <Inventor/elements/SoBumpMapCoordinateElement.h>
+#include <Inventor/elements/SoGLCacheContextElement.h>
+#include <Inventor/elements/SoLazyElement.h>
+#include <Inventor/elements/SoMultiTextureCoordinateElement.h>
+#include <Inventor/elements/SoMultiTextureEnabledElement.h>
+#include <Inventor/lists/SbList.h>
+#include <Inventor/misc/SbHash.h>
+#include <Inventor/misc/SoGL.h>
+#include <Inventor/system/gl.h>
+#include <Inventor/misc/SoContextHandler.h>
+
+// *************************************************************************
 
 #define MAX_UNITS 16
 
@@ -65,13 +71,13 @@ static int COIN_ENABLE_VBO = -1;
 
 class SoPrimitiveVertexCacheP {
 public:
-  SoPrimitiveVertexCacheP(void) 
-    : vertexlist(256), 
+  SoPrimitiveVertexCacheP(void)
+    : vertexlist(256),
       normallist(256),
       texcoordlist(256),
       bumpcoordlist(256),
       rgbalist(256),
-      indices(1024), 
+      indices(1024),
       vhash(1024),
       vbodict(4) { }
 
@@ -87,7 +93,7 @@ public:
   SbList <int32_t> lineindices;
   SbList <int32_t> pointindices;
   SbHash <int32_t, SoPrimitiveVertexCache::Vertex> vhash;
-
+  
   const SbVec2f * bumpcoords;
   int numbumpcoords;
 
@@ -107,42 +113,44 @@ public:
   const SoMultiTextureCoordinateElement * multielem;
   SbList <SbVec4f> * multitexcoords;
   SoState * state;
-  SbDict vbodict;
+  SbHash<SoPrimitiveVertexCache_vboidx *, uint32_t> vbodict;
 
   void addVertex(const SoPrimitiveVertexCache::Vertex & v);
   void enableArrays(const cc_glglue * glue,
                     const SbBool color, const SbBool normal,
                     const SbBool texture, const SbBool * enabled,
                     const int lastenabled);
-  
+
   void disableArrays(const cc_glglue * glue,
                      const SbBool color, const SbBool normal,
                      const SbBool texture, const SbBool * enabled,
                      const int lastenabled);
 
   void enableVBOs(const cc_glglue * glue,
-                  const unsigned long contextid,
+                  const uint32_t contextid,
                   const SbBool color, const SbBool normal,
                   const SbBool texture, const SbBool * enabled,
                   const int lastenabled);
-  
+
   void disableVBOs(const cc_glglue * glue,
                    const SbBool color, const SbBool normal,
                    const SbBool texture, const SbBool * enabled,
                    const int lastenabled);
 
   unsigned long countVBOSize(const cc_glglue * glue,
-                             const unsigned long contextid,
+                             const uint32_t contextid,
                              const SbBool color, const SbBool normal,
                              const SbBool texture, const SbBool * enabled,
                              const int lastenabled);
 
-  static void vbo_schedule(unsigned long key, void * value);
+  static void vbo_schedule(const uint32_t & key, SoPrimitiveVertexCache_vboidx * const & value, void * closure);
   static void vbo_delete(void * closure, uint32_t contextid);
+  static void contextCleanup(uint32_t context, void * closure);
 };
 
-#undef PRIVATE
-#define PRIVATE(obj) obj->pimpl
+#define PRIVATE(obj) ((obj)->pimpl)
+
+// *************************************************************************
 
 /*!
   Constructor.
@@ -152,7 +160,8 @@ SoPrimitiveVertexCache::SoPrimitiveVertexCache(SoState * state)
 {
   PRIVATE(this) = new SoPrimitiveVertexCacheP;
   PRIVATE(this)->state = state;
-
+  SoContextHandler::addContextDestructionCallback(SoPrimitiveVertexCacheP::contextCleanup, 
+                                                  PRIVATE(this));
   const SoBumpMapCoordinateElement * belem =
     SoBumpMapCoordinateElement::getInstance(state);
 
@@ -176,7 +185,7 @@ SoPrimitiveVertexCache::SoPrimitiveVertexCache(SoState * state)
 
   // set up variables to test if we need to supply color per vertex
   PRIVATE(this)->colorpervertex = FALSE;
-  
+
   // just store diffuse color with index 0
   uint32_t col;
   if (PRIVATE(this)->packedptr) {
@@ -187,22 +196,22 @@ SoPrimitiveVertexCache::SoPrimitiveVertexCache(SoState * state)
     float tmpt = PRIVATE(this)->transpptr[0];
     col = tmpc.getPackedValue(tmpt);
   }
-  PRIVATE(this)->firstcolor = col;   
-  
+  PRIVATE(this)->firstcolor = col;
+
   // set up for multi texturing
   PRIVATE(this)->lastenabled = -1;
-  PRIVATE(this)->enabledunits = 
+  PRIVATE(this)->enabledunits =
     SoMultiTextureEnabledElement::getEnabledUnits(state, PRIVATE(this)->lastenabled);
   PRIVATE(this)->multielem = NULL;
   PRIVATE(this)->multitexcoords = NULL;
   if (PRIVATE(this)->lastenabled >= 1) {
-    PRIVATE(this)->multitexcoords = new SbList<SbVec4f>[PRIVATE(this)->lastenabled+1];    
+    PRIVATE(this)->multitexcoords = new SbList<SbVec4f>[PRIVATE(this)->lastenabled+1];
     // delay fetching SoMultiTextureCoordinateElement until the first
     // triangle callback. SoTextureCoordinateBundle might push a new
     // element.
   }
 
-  if (COIN_MAX_VBO_MEMORY == 0) { 
+  if (COIN_MAX_VBO_MEMORY == 0) {
     COIN_MAX_VBO_MEMORY = 0xffffffff;
     const char * env = coin_getenv("COIN_MAX_VBO_MEMORY");
     if (env) {
@@ -223,14 +232,17 @@ SoPrimitiveVertexCache::SoPrimitiveVertexCache(SoState * state)
 */
 SoPrimitiveVertexCache::~SoPrimitiveVertexCache()
 {
-  PRIVATE(this)->vbodict.applyToAll(SoPrimitiveVertexCacheP::vbo_schedule);
+  SoContextHandler::removeContextDestructionCallback(SoPrimitiveVertexCacheP::contextCleanup, 
+                                                     PRIVATE(this));
+
+  PRIVATE(this)->vbodict.apply(SoPrimitiveVertexCacheP::vbo_schedule, NULL);
   if (PRIVATE(this)->lastenabled >= 1) {
     delete[] PRIVATE(this)->multitexcoords;
   }
   delete PRIVATE(this);
 }
 
-void 
+void
 SoPrimitiveVertexCache::renderTriangles(SoState * state, const int arrays) const
 {
   int lastenabled = -1;
@@ -240,17 +252,17 @@ SoPrimitiveVertexCache::renderTriangles(SoState * state, const int arrays) const
   const SbBool * enabled = NULL;
   const SbBool normal = (arrays & NORMAL) != 0;
   const SbBool texture = (arrays & TEXCOORD) != 0;
-  const SbBool color = this->colorPerVertex() && ((arrays & COLOR) != 0);  
+  const SbBool color = this->colorPerVertex() && ((arrays & COLOR) != 0);
   if (texture) {
     enabled = SoMultiTextureEnabledElement::getEnabledUnits(state, lastenabled);
   }
 
-  unsigned long contextid = (unsigned long) SoGLCacheContextElement::get(state);    
+  const uint32_t contextid = SoGLCacheContextElement::get(state);
   const cc_glglue * glue = cc_glglue_instance((int) contextid);
   SbBool renderasvbo = COIN_ENABLE_VBO && cc_glglue_has_vertex_buffer_object(glue);
-  
+
   if (renderasvbo) {
-    unsigned long size = PRIVATE(this)->countVBOSize(glue, contextid, color, 
+    unsigned long size = PRIVATE(this)->countVBOSize(glue, contextid, color,
                                                      normal, texture, enabled, lastenabled);
     if (total_vbo_memory + size > COIN_MAX_VBO_MEMORY) {
       renderasvbo = FALSE;
@@ -259,7 +271,7 @@ SoPrimitiveVertexCache::renderTriangles(SoState * state, const int arrays) const
       total_vbo_memory += size;
     }
   }
-  
+
   if (renderasvbo) {
     PRIVATE(this)->enableVBOs(glue, contextid, color, normal, texture, enabled, lastenabled);
     cc_glglue_glDrawElements(glue, GL_TRIANGLES, n, GL_UNSIGNED_INT, NULL);
@@ -273,7 +285,7 @@ SoPrimitiveVertexCache::renderTriangles(SoState * state, const int arrays) const
   }
 }
 
-void 
+void
 SoPrimitiveVertexCache::renderLines(SoState * state, const int arrays) const
 {
   // FIXME: VBO support for lines, pederb 2004-02-24
@@ -283,7 +295,7 @@ SoPrimitiveVertexCache::renderLines(SoState * state, const int arrays) const
   const SbBool * enabled = NULL;
   const SbBool normal = (arrays & NORMAL) != 0;
   const SbBool texture = (arrays & TEXCOORD) != 0;
-  SbBool color = this->colorPerVertex() && ((arrays & COLOR) != 0);  
+  SbBool color = this->colorPerVertex() && ((arrays & COLOR) != 0);
   if (texture) {
     enabled = SoMultiTextureEnabledElement::getEnabledUnits(state, lastenabled);
   }
@@ -295,7 +307,7 @@ SoPrimitiveVertexCache::renderLines(SoState * state, const int arrays) const
   PRIVATE(this)->disableArrays(glue, color, normal, texture, enabled, lastenabled);
 }
 
-void 
+void
 SoPrimitiveVertexCache::renderPoints(SoState * state, const int arrays) const
 {
   // FIXME: VBO support for points, pederb 2004-02-24
@@ -305,7 +317,7 @@ SoPrimitiveVertexCache::renderPoints(SoState * state, const int arrays) const
   const SbBool * enabled = NULL;
   const SbBool normal = (arrays & NORMAL) != 0;
   const SbBool texture = (arrays & TEXCOORD) != 0;
-  SbBool color = this->colorPerVertex() && ((arrays & COLOR) != 0);  
+  SbBool color = this->colorPerVertex() && ((arrays & COLOR) != 0);
   if (texture) {
     enabled = SoMultiTextureEnabledElement::getEnabledUnits(state, lastenabled);
   }
@@ -338,7 +350,7 @@ SoPrimitiveVertexCache::addTriangle(const SoPrimitiveVertex * v0,
     v.bumpcoord = SbVec2f(tmp[0], tmp[1]);
     v.texcoord0 = tmp;
     v.texcoordidx = -1;
-    
+
     int midx = vp[i]->getMaterialIndex();
     uint32_t col;
     if (PRIVATE(this)->packedptr) {
@@ -350,23 +362,22 @@ SoPrimitiveVertexCache::addTriangle(const SoPrimitiveVertex * v0,
       col = tmpc.getPackedValue(tmpt);
     }
     if (col != PRIVATE(this)->firstcolor) PRIVATE(this)->colorpervertex = TRUE;
-    
+
     v.rgba[0] = col>>24;
     v.rgba[1] = (col>>16)&0xff;
     v.rgba[2] = (col>>8)&0xff;
     v.rgba[3] = col&0xff;
-    
-    SoDetail * d = (SoDetail*) vp[i]->getDetail();
 
-    if (d && d->isOfType(SoFaceDetail::getClassTypeId())) {
+    SoDetail * d = (SoDetail*) vp[i]->getDetail();
+    
+    if (d && d->isOfType(SoFaceDetail::getClassTypeId()) && pointdetailidx) {
       SoFaceDetail * fd = (SoFaceDetail*) d;
-      assert(pointdetailidx != NULL);
       assert(pointdetailidx[i] < fd->getNumPoints());
-      
-      SoPointDetail * pd = (SoPointDetail*) 
+
+      SoPointDetail * pd = (SoPointDetail*)
         fd->getPoint(pointdetailidx[i]);
-      
-      int tidx  = v.texcoordidx = pd->getTextureCoordIndex();            
+
+      int tidx  = v.texcoordidx = pd->getTextureCoordIndex();
       if (PRIVATE(this)->numbumpcoords) {
         v.bumpcoord = PRIVATE(this)->bumpcoords[SbClamp(tidx, 0, PRIVATE(this)->numbumpcoords)];
       }
@@ -375,9 +386,9 @@ SoPrimitiveVertexCache::addTriangle(const SoPrimitiveVertex * v0,
     if (!PRIVATE(this)->vhash.get(v, idx)) {
       idx = PRIVATE(this)->vertexlist.getLength();
       PRIVATE(this)->vhash.put(v, idx);
-      PRIVATE(this)->addVertex(v);      
+      PRIVATE(this)->addVertex(v);
       PRIVATE(this)->indices.append(idx);
-      
+
       // update texture coordinates for unit 1-n
       for (int j = 1; j <= PRIVATE(this)->lastenabled; j++) {
         if (v.texcoordidx >= 0 &&
@@ -398,7 +409,7 @@ SoPrimitiveVertexCache::addTriangle(const SoPrimitiveVertex * v0,
   }
 }
 
-void 
+void
 SoPrimitiveVertexCache::addLine(const SoPrimitiveVertex * v0,
                                 const SoPrimitiveVertex * v1)
 {
@@ -416,7 +427,7 @@ SoPrimitiveVertexCache::addLine(const SoPrimitiveVertex * v0,
     v.bumpcoord = SbVec2f(tmp[0], tmp[1]);
     v.texcoord0 = tmp;
     v.texcoordidx = -1;
-    
+
     int midx = vp[i]->getMaterialIndex();
     uint32_t col;
     if (PRIVATE(this)->packedptr) {
@@ -428,12 +439,12 @@ SoPrimitiveVertexCache::addLine(const SoPrimitiveVertex * v0,
       col = tmpc.getPackedValue(tmpt);
     }
     if (col != PRIVATE(this)->firstcolor) PRIVATE(this)->colorpervertex = TRUE;
-    
+
     v.rgba[0] = col>>24;
     v.rgba[1] = (col>>16)&0xff;
     v.rgba[2] = (col>>8)&0xff;
     v.rgba[3] = col&0xff;
-    
+
     SoDetail * d = (SoDetail*) vp[i]->getDetail();
 
     if (d && d->isOfType(SoLineDetail::getClassTypeId())) {
@@ -441,8 +452,8 @@ SoPrimitiveVertexCache::addLine(const SoPrimitiveVertex * v0,
       const SoPointDetail * pd;
       if (i == 0) pd = ld->getPoint0();
       else pd = ld->getPoint1();
-      
-      int tidx  = v.texcoordidx = ((SoPointDetail*)pd)->getTextureCoordIndex();            
+
+      int tidx  = v.texcoordidx = ((SoPointDetail*)pd)->getTextureCoordIndex();
       if (PRIVATE(this)->numbumpcoords) {
         v.bumpcoord = PRIVATE(this)->bumpcoords[SbClamp(tidx, 0, PRIVATE(this)->numbumpcoords)];
       }
@@ -453,7 +464,7 @@ SoPrimitiveVertexCache::addLine(const SoPrimitiveVertex * v0,
       PRIVATE(this)->vhash.put(v, idx);
       PRIVATE(this)->addVertex(v);
       PRIVATE(this)->lineindices.append(idx);
-      
+
       // update texture coordinates for unit 1-n
       for (int j = 1; j <= PRIVATE(this)->lastenabled; j++) {
         if (v.texcoordidx >= 0 &&
@@ -474,7 +485,7 @@ SoPrimitiveVertexCache::addLine(const SoPrimitiveVertex * v0,
   }
 }
 
-void 
+void
 SoPrimitiveVertexCache::addPoint(const SoPrimitiveVertex * v0)
 {
   if (PRIVATE(this)->lastenabled >= 1 && PRIVATE(this)->multielem == NULL) {
@@ -488,7 +499,7 @@ SoPrimitiveVertexCache::addPoint(const SoPrimitiveVertex * v0)
   v.bumpcoord = SbVec2f(tmp[0], tmp[1]);
   v.texcoord0 = tmp;
   v.texcoordidx = -1;
-    
+
   int midx = v0->getMaterialIndex();
   uint32_t col;
   if (PRIVATE(this)->packedptr) {
@@ -500,17 +511,17 @@ SoPrimitiveVertexCache::addPoint(const SoPrimitiveVertex * v0)
     col = tmpc.getPackedValue(tmpt);
   }
   if (col != PRIVATE(this)->firstcolor) PRIVATE(this)->colorpervertex = TRUE;
-    
+
   v.rgba[0] = col>>24;
   v.rgba[1] = (col>>16)&0xff;
   v.rgba[2] = (col>>8)&0xff;
   v.rgba[3] = col&0xff;
-    
+
   SoDetail * d = (SoDetail*) v0->getDetail();
 
   if (d && d->isOfType(SoPointDetail::getClassTypeId())) {
-    SoPointDetail * pd = (SoPointDetail*) d;      
-    int tidx  = v.texcoordidx = pd->getTextureCoordIndex();            
+    SoPointDetail * pd = (SoPointDetail*) d;
+    int tidx  = v.texcoordidx = pd->getTextureCoordIndex();
     if (PRIVATE(this)->numbumpcoords) {
       v.bumpcoord = PRIVATE(this)->bumpcoords[SbClamp(tidx, 0, PRIVATE(this)->numbumpcoords)];
     }
@@ -522,7 +533,7 @@ SoPrimitiveVertexCache::addPoint(const SoPrimitiveVertex * v0)
     PRIVATE(this)->vhash.put(v, idx);
     PRIVATE(this)->addVertex(v);
     PRIVATE(this)->pointindices.append(idx);
-      
+
     // update texture coordinates for unit 1-n
     for (int j = 1; j <= PRIVATE(this)->lastenabled; j++) {
       if (v.texcoordidx >= 0 &&
@@ -548,31 +559,31 @@ SoPrimitiveVertexCache::getNumVertices(void) const
   return PRIVATE(this)->vertexlist.getLength();
 }
 
-const SbVec3f * 
+const SbVec3f *
 SoPrimitiveVertexCache::getVertexArray(void) const
 {
   return PRIVATE(this)->vertexlist.getArrayPtr();
 }
 
-const SbVec3f * 
+const SbVec3f *
 SoPrimitiveVertexCache::getNormalArray(void) const
 {
   return PRIVATE(this)->normallist.getArrayPtr();
 }
 
-const SbVec4f * 
+const SbVec4f *
 SoPrimitiveVertexCache::getTexCoordArray(void) const
 {
   return PRIVATE(this)->texcoordlist.getArrayPtr();
 }
 
-const SbVec2f * 
+const SbVec2f *
 SoPrimitiveVertexCache::getBumpCoordArray(void) const
 {
   return PRIVATE(this)->bumpcoordlist.getArrayPtr();
 }
 
-const uint8_t * 
+const uint8_t *
 SoPrimitiveVertexCache::getColorArray(void) const
 {
   return PRIVATE(this)->rgbalist.getArrayPtr();
@@ -596,45 +607,118 @@ SoPrimitiveVertexCache::getIndex(const int idx) const
   return PRIVATE(this)->indices[idx];
 }
 
-SbBool 
+SbBool
 SoPrimitiveVertexCache::colorPerVertex(void) const
 {
   return PRIVATE(this)->colorpervertex;
 }
 
-const SbVec4f * 
+const SbVec4f *
 SoPrimitiveVertexCache::getMultiTextureCoordinateArray(const int unit) const
 {
   assert(unit <= PRIVATE(this)->lastenabled);
   return PRIVATE(this)->multitexcoords[unit].getArrayPtr();
 }
 
-int 
+int
 SoPrimitiveVertexCache::getNumLineIndices(void) const
 {
   return PRIVATE(this)->lineindices.getLength();
 }
 
-int 
+int
 SoPrimitiveVertexCache::getNumPointIndices(void) const
 {
   return PRIVATE(this)->pointindices.getLength();
 }
 
-  
-const int32_t * 
+
+const int32_t *
 SoPrimitiveVertexCache::getLineIndices(void) const
 {
   return PRIVATE(this)->lineindices.getArrayPtr();
 }
 
-const int32_t * 
+const int32_t *
 SoPrimitiveVertexCache::getPointIndices(void) const
 {
   return PRIVATE(this)->pointindices.getArrayPtr();
 }
 
-void 
+// sort an array of three integers
+static void sort3(int32_t * arr)
+{
+  // simple bubble-sort
+  int32_t tmp;
+  if (arr[1] < arr[0]) {
+    tmp = arr[0];
+    arr[0] = arr[1];
+    arr[1] = tmp;
+  }
+  if (arr[2] < arr[1]) {
+    tmp = arr[1];
+    arr[1] = arr[2];
+    arr[2] = tmp;
+  }
+  if (arr[1] < arr[0]) {
+    tmp = arr[0];
+    arr[0] = arr[1];
+    arr[1] = tmp;
+  }
+}
+
+// qsort callback used for sorting triangles based on vertex indices
+extern "C" {
+static int
+compare_triangle(const void * v0, const void * v1)
+{
+  int i;
+  int32_t * t0 = (int*) v0;
+  int32_t * t1 = (int*) v1;
+
+  int32_t ti0[3];
+  int32_t ti1[3];
+  for (i = 0; i < 3; i++) {
+    ti0[i] = t0[i];
+    ti1[i] = t1[i];
+  }
+  sort3(ti0);
+  sort3(ti1);
+
+  for (i = 0; i < 3; i++) {
+    int32_t diff = ti0[i] - ti1[i];
+    if (diff != 0) return diff;
+  }
+  return 0;
+}
+}
+
+// qsort callback used for sorting line segments based on vertex
+// indices
+extern "C" {
+static int
+compare_line_segment(const void * v0, const void * v1)
+{
+  int i;
+  int32_t * l0 = (int*) v0;
+  int32_t * l1 = (int*) v1;
+  int32_t li0[2];
+  int32_t li1[2];
+  
+  li0[0] = SbMin(l0[0], l0[1]);
+  li0[1] = SbMax(l0[0], l0[1]);
+  li1[0] = SbMin(l1[0], l1[1]);
+  li1[1] = SbMax(l1[0], l1[1]);
+
+  for (i = 0; i < 2; i++) {
+    int diff = li0[i] - li1[i];
+    if (diff) return diff;
+  }
+  return 0;
+}
+}
+
+void
 SoPrimitiveVertexCache::fit(void)
 {
   PRIVATE(this)->vertexlist.fit();
@@ -646,16 +730,39 @@ SoPrimitiveVertexCache::fit(void)
   PRIVATE(this)->lineindices.fit();
   PRIVATE(this)->pointindices.fit();
   PRIVATE(this)->vhash.clear();
-}
 
-#undef PRIVATE
+
+  // sort triangles based on vertex indices to get more hits in the
+  // GPU vertex cache. Not the optimal solution, but should work
+  // pretty well. Example: bunny.iv (~70000 triangles) went from 238
+  // fps with no sorting to 380 fps with sorting.
+  if (PRIVATE(this)->indices.getLength()) {
+    qsort((void*) PRIVATE(this)->indices.getArrayPtr(),
+          PRIVATE(this)->indices.getLength() / 3,
+          sizeof(int32_t) * 3,
+          compare_triangle);
+  }
+  
+  // sort lines as well
+  if (PRIVATE(this)->lineindices.getLength()) {
+    qsort((void*) PRIVATE(this)->lineindices.getArrayPtr(),
+          PRIVATE(this)->lineindices.getLength() / 2,
+          sizeof(int32_t) * 2,
+          compare_line_segment);
+  }
+}
 
 
 SoPrimitiveVertexCache::Vertex::operator unsigned long(void) const
 {
-  int size = sizeof(*this);
   unsigned long key = 0;
+  // create an xor key based on coordinates, normal and texcoords
   const unsigned char * ptr = (const unsigned char *) this;
+  
+  // a bit hackish. Stop xor'ing at bumpcoord
+  const unsigned char * stop = (const unsigned char*) &this->bumpcoord;
+  const ptrdiff_t size = stop-ptr;
+  
   for (int i = 0; i < size; i++) {
     int shift = (i%4) * 8;
     key ^= (ptr[i]<<shift);
@@ -663,10 +770,10 @@ SoPrimitiveVertexCache::Vertex::operator unsigned long(void) const
   return key;
 }
 
-int
-SoPrimitiveVertexCache::Vertex::operator==(const Vertex & v) const
+int 
+SoPrimitiveVertexCache::Vertex::operator==(const Vertex & v)
 {
-  return memcmp(this, &v, sizeof(Vertex)) == 0;
+  return memcmp(this, &v, sizeof(v)) == 0;
 }
 
 void 
@@ -681,7 +788,7 @@ SoPrimitiveVertexCacheP::addVertex(const SoPrimitiveVertexCache::Vertex & v)
   }
 }
 
-void 
+void
 SoPrimitiveVertexCacheP::enableArrays(const cc_glglue * glue,
                                       const SbBool color, const SbBool normal,
                                       const SbBool texture, const SbBool * enabled,
@@ -691,7 +798,7 @@ SoPrimitiveVertexCacheP::enableArrays(const cc_glglue * glue,
   cc_glglue_glVertexPointer(glue, 3, GL_FLOAT, 0,
                             (GLvoid*) this->vertexlist.getArrayPtr());
   cc_glglue_glEnableClientState(glue, GL_VERTEX_ARRAY);
-  
+
   if (color) {
     cc_glglue_glColorPointer(glue, 4, GL_UNSIGNED_BYTE, 0,
                              (GLvoid*) this->rgbalist.getArrayPtr());
@@ -702,7 +809,7 @@ SoPrimitiveVertexCacheP::enableArrays(const cc_glglue * glue,
     cc_glglue_glTexCoordPointer(glue, 4, GL_FLOAT, 0,
                                 (GLvoid*) this->texcoordlist.getArrayPtr());
     cc_glglue_glEnableClientState(glue, GL_TEXTURE_COORD_ARRAY);
-    
+
     for (i = 1; i <= lastenabled; i++) {
       if (enabled[i]) {
         cc_glglue_glClientActiveTexture(glue, GL_TEXTURE0 + i);
@@ -719,8 +826,8 @@ SoPrimitiveVertexCacheP::enableArrays(const cc_glglue * glue,
   }
 }
 
-  
-void 
+
+void
 SoPrimitiveVertexCacheP::disableArrays(const cc_glglue * glue,
                                        const SbBool color, const SbBool normal,
                                        const SbBool texture, const SbBool * enabled,
@@ -749,43 +856,40 @@ SoPrimitiveVertexCacheP::disableArrays(const cc_glglue * glue,
   cc_glglue_glDisableClientState(glue, GL_VERTEX_ARRAY);
 }
 
-void 
+void
 SoPrimitiveVertexCacheP::enableVBOs(const cc_glglue * glue,
-                                    unsigned long contextid,
+                                    uint32_t contextid,
                                     const SbBool color, const SbBool normal,
                                     const SbBool texture, const SbBool * enabled,
                                     const int lastenabled)
 {
-  void * tmp;
   SoPrimitiveVertexCache_vboidx * vbo;
-  if (!this->vbodict.find(contextid, tmp)) {
+  if (!this->vbodict.get(contextid, vbo)) {
     vbo = new SoPrimitiveVertexCache_vboidx;
     memset(vbo, 0, sizeof(SoPrimitiveVertexCache_vboidx));
-    (void) this->vbodict.enter(contextid, (void*) vbo);
-  } else {
-    vbo = (SoPrimitiveVertexCache_vboidx *) tmp;
+    (void) this->vbodict.put(contextid, vbo);
   }
 
   int i;
   if (vbo->vertex == 0) {
     cc_glglue_glGenBuffers(glue, 1, &vbo->vertex);
     cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, vbo->vertex);
-    cc_glglue_glBufferData(glue, GL_ARRAY_BUFFER, 
+    cc_glglue_glBufferData(glue, GL_ARRAY_BUFFER,
                            this->vertexlist.getLength()*3*sizeof(float),
                            this->vertexlist.getArrayPtr(),
                            GL_STATIC_DRAW);
   }
   else {
-    cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, vbo->vertex);    
+    cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, vbo->vertex);
   }
-  cc_glglue_glVertexPointer(glue, 3, GL_FLOAT, 0, NULL);  
+  cc_glglue_glVertexPointer(glue, 3, GL_FLOAT, 0, NULL);
   cc_glglue_glEnableClientState(glue, GL_VERTEX_ARRAY);
-  
+
   if (color) {
     if (vbo->rgba == 0) {
       cc_glglue_glGenBuffers(glue, 1, &vbo->rgba);
       cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, vbo->rgba);
-      cc_glglue_glBufferData(glue, GL_ARRAY_BUFFER, 
+      cc_glglue_glBufferData(glue, GL_ARRAY_BUFFER,
                              this->rgbalist.getLength() * sizeof(uint8_t),
                              this->rgbalist.getArrayPtr(),
                              GL_STATIC_DRAW);
@@ -806,12 +910,12 @@ SoPrimitiveVertexCacheP::enableVBOs(const cc_glglue * glue,
                              GL_STATIC_DRAW);
     }
     else {
-      cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, vbo->texcoord0);    
+      cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, vbo->texcoord0);
     }
-    
+
     cc_glglue_glTexCoordPointer(glue, 4, GL_FLOAT, 0, NULL);
     cc_glglue_glEnableClientState(glue, GL_TEXTURE_COORD_ARRAY);
-    
+
     for (i = 1; i <= lastenabled; i++) {
       if (enabled[i]) {
         if (vbo->multitex[i] == 0) {
@@ -823,7 +927,7 @@ SoPrimitiveVertexCacheP::enableVBOs(const cc_glglue * glue,
                                  GL_STATIC_DRAW);
         }
         else {
-          cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, vbo->multitex[i]);    
+          cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, vbo->multitex[i]);
         }
 
         cc_glglue_glClientActiveTexture(glue, GL_TEXTURE0 + i);
@@ -836,15 +940,15 @@ SoPrimitiveVertexCacheP::enableVBOs(const cc_glglue * glue,
     if (vbo->normal == 0) {
       cc_glglue_glGenBuffers(glue, 1, &vbo->normal);
       cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, vbo->normal);
-      cc_glglue_glBufferData(glue, GL_ARRAY_BUFFER, 
+      cc_glglue_glBufferData(glue, GL_ARRAY_BUFFER,
                              this->normallist.getLength()*3*sizeof(float),
                              this->normallist.getArrayPtr(),
                              GL_STATIC_DRAW);
     }
     else {
-      cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, vbo->normal);    
+      cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, vbo->normal);
     }
-    
+
     cc_glglue_glNormalPointer(glue, GL_FLOAT, 0, NULL);
     cc_glglue_glEnableClientState(glue, GL_NORMAL_ARRAY);
   }
@@ -852,7 +956,7 @@ SoPrimitiveVertexCacheP::enableVBOs(const cc_glglue * glue,
   if (vbo->triangleindex == 0) {
     cc_glglue_glGenBuffers(glue, 1, &vbo->triangleindex);
     cc_glglue_glBindBuffer(glue, GL_ELEMENT_ARRAY_BUFFER, vbo->triangleindex);
-    cc_glglue_glBufferData(glue, GL_ELEMENT_ARRAY_BUFFER, 
+    cc_glglue_glBufferData(glue, GL_ELEMENT_ARRAY_BUFFER,
                            this->indices.getLength()*sizeof(int32_t),
                            this->indices.getArrayPtr(),
                            GL_STATIC_DRAW);
@@ -862,35 +966,31 @@ SoPrimitiveVertexCacheP::enableVBOs(const cc_glglue * glue,
   }
 }
 
-  
-void 
+
+void
 SoPrimitiveVertexCacheP::disableVBOs(const cc_glglue * glue,
                                      const SbBool color, const SbBool normal,
                                      const SbBool texture, const SbBool * enabled,
                                      const int lastenabled)
 {
   this->disableArrays(glue, color, normal, texture, enabled, lastenabled);
-  cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, 0); // Reset VBO binding  
-  cc_glglue_glBindBuffer(glue, GL_ELEMENT_ARRAY_BUFFER, 0); // Reset VBO binding  
+  cc_glglue_glBindBuffer(glue, GL_ARRAY_BUFFER, 0); // Reset VBO binding
+  cc_glglue_glBindBuffer(glue, GL_ELEMENT_ARRAY_BUFFER, 0); // Reset VBO binding
 }
 
 unsigned long
 SoPrimitiveVertexCacheP::countVBOSize(const cc_glglue * glue,
-                                      unsigned long contextid,
+                                      const uint32_t contextid,
                                       const SbBool color, const SbBool normal,
                                       const SbBool texture, const SbBool * enabled,
                                       const int lastenabled)
 {
   unsigned long size = 0;
-  void * tmp;
   SoPrimitiveVertexCache_vboidx * vbo;
-  if (!this->vbodict.find(contextid, tmp)) {
+  if (!this->vbodict.get(contextid, vbo)) {
     vbo = new SoPrimitiveVertexCache_vboidx;
     memset(vbo, 0, sizeof(SoPrimitiveVertexCache_vboidx));
-    (void) this->vbodict.enter(contextid, (void*) vbo);
-  }
-  else {
-    vbo = (SoPrimitiveVertexCache_vboidx *) tmp;
+    (void) this->vbodict.put(contextid, vbo);
   }
 
   int i;
@@ -905,7 +1005,7 @@ SoPrimitiveVertexCacheP::countVBOSize(const cc_glglue * glue,
   if (texture) {
     if (vbo->texcoord0 == 0) {
       size += this->texcoordlist.getLength()*4*sizeof(float);
-    }    
+    }
     for (i = 1; i <= lastenabled; i++) {
       if (enabled[i]) {
         if (vbo->multitex[i] == 0) {
@@ -926,11 +1026,11 @@ SoPrimitiveVertexCacheP::countVBOSize(const cc_glglue * glue,
 }
 
 void
-SoPrimitiveVertexCacheP::vbo_schedule(unsigned long key,
-                                      void * value)
+SoPrimitiveVertexCacheP::vbo_schedule(const uint32_t & key,
+                                      SoPrimitiveVertexCache_vboidx * const & value,
+                                      void * closure)
 {
-  SoGLCacheContextElement::scheduleDeleteCallback((uint32_t) key,
-                                                  vbo_delete, value);
+  SoGLCacheContextElement::scheduleDeleteCallback(key, vbo_delete, value);
 }
 
 void
@@ -938,7 +1038,7 @@ SoPrimitiveVertexCacheP::vbo_delete(void * closure, uint32_t contextid)
 {
   SoPrimitiveVertexCache_vboidx * vbo =
     (SoPrimitiveVertexCache_vboidx *) closure;
-  
+
   const cc_glglue * glue = cc_glglue_instance((int)contextid);
   if (vbo->triangleindex) {
     cc_glglue_glDeleteBuffers(glue, 1, &vbo->triangleindex);
@@ -964,8 +1064,21 @@ SoPrimitiveVertexCacheP::vbo_delete(void * closure, uint32_t contextid)
   delete vbo;
 }
 
+//
+// Callback from SoContextHandler
+//
+void 
+SoPrimitiveVertexCacheP::contextCleanup(uint32_t context, void * closure)
+{
+  SoPrimitiveVertexCacheP * thisp = (SoPrimitiveVertexCacheP*) closure;
+
+  SoPrimitiveVertexCache_vboidx * tmp;
+  if (thisp->vbodict.get(context, tmp)) {
+    SoPrimitiveVertexCacheP::vbo_delete(tmp, context);
+    thisp->vbodict.remove(context);
+  }
+}
+
 
 #undef MAX_UNITS
-
-
-
+#undef PRIVATE
