@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -40,8 +40,7 @@ public:
 
 #endif // DOXYGEN_SKIP_THIS
 
-#undef THIS
-#define THIS this->pimpl
+#define PRIVATE(obj) ((obj)->pimpl)
 
 /*!
   Constructor.
@@ -49,7 +48,7 @@ public:
 SoTextureCoordinateCache::SoTextureCoordinateCache(SoState * const state)
   : SoCache(state)
 {
-  THIS = new SoTextureCoordinateCacheP;
+  PRIVATE(this) = new SoTextureCoordinateCacheP;
 }
 
 /*!
@@ -57,7 +56,7 @@ SoTextureCoordinateCache::SoTextureCoordinateCache(SoState * const state)
 */
 SoTextureCoordinateCache::~SoTextureCoordinateCache()
 {
-  delete THIS;
+  delete PRIVATE(this);
 }
 
 /*!
@@ -111,12 +110,12 @@ SoTextureCoordinateCache::generate(const SbBox3f & bbox,
     s /= sizes[0];
     t /= sizes[1];
     // expand list array as needed
-    if (i >= THIS->texCoords.getLength()) THIS->texCoords.append(SbVec2f());
-    THIS->texCoords[i].setValue(s, t);
+    if (i >= PRIVATE(this)->texCoords.getLength()) PRIVATE(this)->texCoords.append(SbVec2f());
+    PRIVATE(this)->texCoords[i].setValue(s, t);
   }
 
   // fit list array in case we used to have more items than now
-  THIS->texCoords.truncate(numvertices);
+  PRIVATE(this)->texCoords.truncate(numvertices);
 }
 
 /*!
@@ -125,7 +124,7 @@ SoTextureCoordinateCache::generate(const SbBox3f & bbox,
 const SbVec2f *
 SoTextureCoordinateCache::get(void) const
 {
-  return THIS->texCoords.getArrayPtr();
+  return PRIVATE(this)->texCoords.getArrayPtr();
 }
 
 /*!
@@ -134,5 +133,7 @@ SoTextureCoordinateCache::get(void) const
 int
 SoTextureCoordinateCache::getNum(void) const
 {
-  return THIS->texCoords.getLength();
+  return PRIVATE(this)->texCoords.getLength();
 }
+
+#undef PRIVATE
