@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -28,6 +28,11 @@
   \ingroup actions
   \ingroup collision
 
+  Note that only collisions between actual geometry in the scene is
+  detected, so the contents of some special nodes like e.g. SoText2
+  and SoImage (which projects to screen-plane bitmap graphics, and not
+  actual polygons) will not be considered for collision detection.
+
   Below is a simple usage example for this class. It was written as a
   stand-alone framework set up for profiling and optimization of the
   SoIntersectionDetectionAction. It tests intersection of all shapes
@@ -108,12 +113,21 @@
   \since TGS Inventor 2.4
 */
 
+// *************************************************************************
+
 // FIXME: intersection with lines not implemented yet. 20030507 mortene.
 //
 // UPDATE 20030815 mortene.: just noticed that there's line-line
 // intersection testing code in SoExtSelection. Check if that could be
 // used.
 
+// FIXME: I just noticed that there's an O(n^2) algorithm for removing
+// duplicates from SbOctTree::findItems() results in SbOctTree.cpp
+// (see the add_to_array() helper function). This may very well be a
+// serious bottleneck, should investigate. 20050512 mortene.
+
+// *************************************************************************
+
 #include <Inventor/collision/SoIntersectionDetectionAction.h>
 
 #include <Inventor/C/tidbits.h>
@@ -265,7 +279,7 @@ ida_debug(void)
 {
   static int dbg = -1;
   if (dbg == -1) {
-    const char * env = coin_getenv("COIN_IDA_DEBUG");
+    const char * env = coin_getenv("COIN_DEBUG_INTERSECTIONDETECTIONACTION");
     dbg = env && atoi(env) > 0;
   }
   return dbg == 0 ? FALSE : TRUE;
@@ -291,11 +305,12 @@ SoIntersectionDetectionAction::~SoIntersectionDetectionAction(void)
   delete PRIVATE(this);
 }
 
-// FIXME: is the "epsilon is not supported yet" statement below false?
-// 20030326 mortene.
-
 /*!
   Sets the global intersection detection distance epsilon value.
+
+  This makes primitives within the epsilon distance be considered to
+  intersect each other.
+
   This will affect all intersection detection action objects in use that
   don't have a locally set value.
 
@@ -306,10 +321,7 @@ SoIntersectionDetectionAction::~SoIntersectionDetectionAction(void)
   decide intersections. Increasing the epsilon value can therefore
   cause serious slow-downs in the running time of the intersections
   checks.
-
-  Epsilon is not supported yet.
 */
-
 void
 SoIntersectionDetectionAction::setIntersectionEpsilon(float epsilon) // static
 {
@@ -611,9 +623,8 @@ SoIntersectionDetectionAction::removeIntersectionCallback(SoIntersectionCB * cb,
 void
 SoIntersectionDetectionAction::apply(SoNode * node)
 {
-  // Keep this around for now, for getting a stand-alone scene to work
-  // with from an invocation from the ToyotaViewer
-  // application. mortene.
+  // Keep this around, as it's handy for dumping a stand-alone scene
+  // to work with from an invocation within an application framework.
 #if 0 // disabled
   SoOutput out;
   SbBool ok = out.openFile("/tmp/assembly.wrl");
@@ -702,20 +713,38 @@ public:
 
   ~PrimitiveData()
   {
-    if (this->octtree) { delete this->octtree; }
+    delete this->octtree;
     for (unsigned int i = 0; i < this->numTriangles(); i++) { delete this->getTriangle(i); }
   }
 
 
   const SbOctTree * getOctTree(void) {
     if (this->octtree == NULL) {
-      SbOctTreeFuncs funcs = {
-        NULL /* bboxfunc */, NULL /* ptinsidefunc */,
+      const SbOctTreeFuncs funcs = {
+        NULL /* bboxfunc */,
+        NULL /* ptinsidefunc */,
         PrimitiveData::insideboxfunc,
-        NULL /* insidespherefunc */, NULL /* insideplanesfunc */
+        NULL /* insidespherefunc */,
+        NULL /* insideplanesfunc */
       };
 
-      this->octtree = new SbOctTree(this->getBoundingBox(), funcs);
+      SbBox3f b = this->getBoundingBox();
+      // Add a 1% slack to the bounding box, to avoid problems in
+      // SbOctTree due to floating point inaccuracies (see assert() in
+      // SbOctTree::addItem()).
+      //
+      // This may be just a temporary hack -- see the FIXME at the
+      // same place as mentioned above.
+      SbMatrix m;
+      m.setTransform(SbVec3f(0, 0, 0), // translation
+                     SbRotation::identity(), // rotation
+                     SbVec3f(1.01f, 1.01f, 1.01f), // scalefactor
+                     SbRotation::identity(), // scaleorientation
+                     SbVec3f(b.getCenter())); // center
+      b.transform(m);
+
+      this->octtree = new SbOctTree(b, funcs);
+
       if (ida_debug()) {
         SoDebugError::postInfo("PrimitiveData::getOctTree",
                                "made new octtree for PrimitiveData %p", this);
@@ -1112,13 +1141,26 @@ SoIntersectionDetectionActionP::doIntersectionTesting(void)
     
   }
 
-  SbOctTreeFuncs funcs = {
-    NULL /* bboxfunc */, NULL /* ptinsidefunc */,
+  const SbOctTreeFuncs funcs = {
+    NULL /* bboxfunc */,
+    NULL /* ptinsidefunc */,
     shapeinsideboxfunc,
-    NULL /* insidespherefunc */, NULL /* insideplanesfunc */
+    NULL /* insidespherefunc */,
+    NULL /* insideplanesfunc */
   };
 
-  SbOctTree shapetree(this->fullxfbbox.project(), funcs);
+  SbBox3f b = this->fullxfbbox.project();
+  // Add a 1% slack to the bounding box, to avoid problems in
+  // SbOctTree due to floating point inaccuracies (see assert() in
+  // SbOctTree::addItem()).
+  //
+  // This may be just a temporary hack -- see the FIXME at the
+  // same place.
+  SbMatrix m;
+  m.setScale(1.01f);
+  b.transform(m);
+
+  SbOctTree shapetree(b, funcs);
   for (int k = 0; k < this->shapedata.getLength(); k++) {
     ShapeData * shape = this->shapedata[k];
     if (shape->xfbbox.isEmpty()) { continue; }
