@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -89,6 +89,8 @@
   \endverbatim
 */
 
+#include <string.h>
+
 #include <Inventor/draggers/SoCenterballDragger.h>
 #include <Inventor/nodekits/SoSubKitP.h>
 #include <Inventor/draggers/SoRotateCylindricalDragger.h>
@@ -235,7 +237,7 @@ SoCenterballDragger::SoCenterballDragger(void)
   if (SO_KIT_IS_FIRST_INSTANCE()) {
     SoInteractionKit::readDefaultParts("centerballDragger.iv",
                                        CENTERBALLDRAGGER_draggergeometry,
-                                       sizeof(CENTERBALLDRAGGER_draggergeometry));
+                                       (int)strlen(CENTERBALLDRAGGER_draggergeometry));
   }
 
   SO_KIT_ADD_FIELD(rotation, (SbRotation(SbVec3f(0.0f, 0.0f, 1.0f), 0.0f)));
@@ -308,7 +310,6 @@ SoCenterballDragger::saveStartParameters(void)
   this->savedtransl[0] = m[3][0];
   this->savedtransl[1] = m[3][1];
   this->savedtransl[2] = m[3][2];
-  this->savedcenter = this->center.getValue();
 }
 
 /*!
@@ -454,6 +455,9 @@ SoCenterballDragger::setUpConnections(SbBool onoff, SbBool doitalways)
       this->addChildDragger(child);
     }
 
+    // Update dragger in case fields have changed values before connection
+    SoCenterballDragger::fieldSensorCB(this, NULL);
+
     if (this->rotFieldSensor->getAttachedField() != &this->rotation) {
       this->rotFieldSensor->attach(&this->rotation);
     }
@@ -505,6 +509,15 @@ void
 SoCenterballDragger::fieldSensorCB(void * d, SoSensor *)
 {
   SoCenterballDragger *thisp = (SoCenterballDragger*)d;
+
+  // Save center variable and translate dragger to correct position
+  thisp->savedcenter = thisp->center.getValue();
+  SbMatrix centermat;
+  centermat.setTranslate(thisp->savedcenter);
+  SoMatrixTransform * mt =
+    SO_GET_ANY_PART(thisp, "translateToCenter", SoMatrixTransform);
+  mt->matrix = centermat;
+
   SbMatrix matrix = thisp->getMotionMatrix();
   thisp->workFieldsIntoTransform(matrix);
   thisp->setMotionMatrix(matrix);
@@ -518,14 +531,37 @@ SoCenterballDragger::valueChangedCB(void *, SoDragger * d)
   SbMatrix matrix = thisp->getMotionMatrix();
   SbVec3f t, s;
   SbRotation r, so;
-  matrix.getTransform(t, r, s, so, thisp->center.getValue());
+
+  // Eliminate center variable of matrix
+  if (thisp->savedcenter != SbVec3f(0.0f, 0.0f, 0.0f)) {
+    SbMatrix trans;
+    trans.setTranslate(thisp->savedcenter);
+    matrix.multLeft(trans);
+    trans.setTranslate(-(thisp->savedcenter));
+    matrix.multRight(trans);
+  }
+
+  // Do an inverse rotation, using matrix with center eliminated
+  // to obtain correct translation
+  matrix.getTransform(t, r, s, so);
+  SbMatrix rotmat;
+  rotmat.setRotate(r);
+  //SbMatrix tmp = matrix;
+  matrix.multLeft(rotmat.inverse());
+
+  // Update center of object if dragger has translated
+  SbVec3f difftrans(matrix[3][0], matrix[3][1], matrix[3][2]);
+  if (difftrans != SbVec3f(0.0f, 0.0f, 0.0f)) {
+    thisp->centerFieldSensor->detach();
+    thisp->center.setValue(thisp->savedcenter + difftrans);
+    thisp->centerFieldSensor->attach(&thisp->center);
+  }
 
   thisp->rotFieldSensor->detach();
   if (thisp->rotation.getValue() != r) {
     thisp->rotation = r;
   }
   thisp->rotFieldSensor->attach(&thisp->rotation);
-  thisp->transferCenterDraggerMotion(thisp->getActiveChildDragger());
 }
 
 /*!
