@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -138,11 +138,11 @@
 
   \verbatim
   CLASS SoDragger
-  PVT   "this",  SoDragger  --- 
-        "callbackList",  SoNodeKitListPart [ SoCallback, SoEventCallback ] 
-  PVT   "topSeparator",  SoSeparator  --- 
-  PVT   "motionMatrix",  SoMatrixTransform  --- 
-  PVT   "geomSeparator",  SoSeparator  --- 
+  PVT   "this",  SoDragger  ---
+        "callbackList",  SoNodeKitListPart [ SoCallback, SoEventCallback ]
+  PVT   "topSeparator",  SoSeparator  ---
+  PVT   "motionMatrix",  SoMatrixTransform  ---
+  PVT   "geomSeparator",  SoSeparator  ---
   \endverbatim
 
   \NODEKIT_POST_TABLE
@@ -302,6 +302,7 @@ public:
   const SoEvent * currentevent;
   SoPath * pickedpath;
   SoDraggerCache * draggercache;
+  SbBool didmousemove;
 
   SoCallbackList startCB;
   SoCallbackList motionCB;
@@ -359,6 +360,7 @@ SoDragger::SoDragger(void)
   THIS->surrogateownerpath = NULL;
   THIS->surrogatepath = NULL;
   THIS->cbaction = NULL;
+  THIS->didmousemove = FALSE;
 }
 
 /*!
@@ -370,6 +372,7 @@ SoDragger::~SoDragger()
   if (THIS->surrogateownerpath) THIS->surrogateownerpath->unref();
   if (THIS->surrogatepath) THIS->surrogatepath->unref();
   delete THIS->draggercache;
+  delete THIS->cbaction;
   delete this->pimpl;
 }
 
@@ -771,10 +774,12 @@ SoDragger::unregisterChildDraggerMovingIndependently(SoDragger * child)
 SbMatrix
 SoDragger::getLocalToWorldMatrix(void)
 {
-  assert(THIS->draggercache);
-  SbMatrix m = THIS->draggercache->draggerToWorld;
-  m.multLeft(this->getMotionMatrix());
-  return m;
+  if (THIS->draggercache) {
+    SbMatrix m = THIS->draggercache->draggerToWorld;
+    m.multLeft(this->getMotionMatrix());
+    return m;
+  }
+  return SbMatrix::identity();
 }
 
 
@@ -784,10 +789,12 @@ SoDragger::getLocalToWorldMatrix(void)
 SbMatrix
 SoDragger::getWorldToLocalMatrix(void)
 {
-  assert(THIS->draggercache);
-  SbMatrix m = THIS->draggercache->worldToDragger;
-  m.multRight(this->getMotionMatrix().inverse());
-  return m;
+  if (THIS->draggercache) {
+    SbMatrix m = THIS->draggercache->worldToDragger;
+    m.multRight(this->getMotionMatrix().inverse());
+    return m;
+  }
+  return SbMatrix::identity();
 }
 
 /*!
@@ -1262,15 +1269,19 @@ SoDragger::appendTranslation(const SbMatrix & matrix, const SbVec3f & translatio
 SbMatrix
 SoDragger::appendScale(const SbMatrix & matrix, const SbVec3f & scale, const SbVec3f & scalecenter, const SbMatrix * conversion)
 {
+  // Clamp the dragger values so that huge scaling in one go does not
+  // work, and negative scale values are made positive. The explicit
+  // casts are done to humour the HPUX aCC compiler, which will
+  // otherwise say ``Template deduction failed to find a match for the
+  // call to 'SbMax'''.  mortene.
   SbVec3f clampedscale;
-  // The explicit casts are done to humour the HPUX aCC compiler,
-  // which will otherwise say ``Template deduction failed to find a
-  // match for the call to 'SbMax'''.  mortene.
   clampedscale[0] = SbMax((float)scale[0], SoDragger::minscale);
   clampedscale[1] = SbMax((float)scale[1], SoDragger::minscale);
   clampedscale[2] = SbMax((float)scale[2], SoDragger::minscale);
 
   SbMatrix transform, tmp;
+  // Calculate the scaled matrix without doing any testing if the
+  // scale of the resulting matrix is above SoDragger::minscale.
   transform.setTranslate(-scalecenter);
   tmp.setScale(clampedscale);
   transform.multRight(tmp);
@@ -1282,7 +1293,89 @@ SoDragger::appendScale(const SbMatrix & matrix, const SbVec3f & scale, const SbV
     transform.multLeft(conversion->inverse());
   }
   SbMatrix res = matrix;
-  return res.multLeft(transform);
+  res.multLeft(transform);
+
+  // Decompose the calculated matrix to be able to clamp the scale.
+  SbVec3f t, s;
+  SbRotation r, so;
+  res.getTransform(t, r, s, so);
+
+  // If the scale of the resulting matrix is smaller in x,y or z than
+  // SoDragger::minscale, then the scale value has to be corrected. If
+  // no correction is needed, the res matrix is returned without more
+  // calculations.
+  if (s[0] < SoDragger::minscale ||
+      s[1] < SoDragger::minscale ||
+      s[2] < SoDragger::minscale) {
+
+    // Clamp the values of the scale to be SoDragger::minscale or
+    // above.
+    int i;
+    for (i = 0; i < 3; i++) {
+      if (s[i] < SoDragger::minscale) {
+        s[i] = SoDragger::minscale;
+      }
+    }
+
+    // Set the new transform with the adjusted scale
+    transform.setTransform(t, r, s, so);
+    // Just returning this matrix does not work as expected.
+    // The dragger gets translated along the screen while trying
+    // to scale below SoDragger::minscale. This is why further
+    // processing is needed. We use the relationship between the
+    // matrices to calculate the required scale vector.
+    //
+    // The scale matrix is found by the following equality:
+    //
+    // M = C^-1 * P^-1 * S * P * C * Mold
+    //
+    // By having the scale on one side and moving all other matrices
+    // to the other side, we end up with this equation:
+    //
+    // S = P * C * M * Mold^-1 * C^-1 * P^-1
+    //
+    // Where S is the scale matrix which has a diagonal containing
+    // the required scale vector.
+
+    // Calculate the scale vector:
+    transform.multRight(matrix.inverse());
+    if (conversion) {
+      transform.multLeft(*conversion);
+      transform.multRight(conversion->inverse());
+    }
+    tmp.setTranslate(scalecenter);
+    transform.multLeft(tmp);
+    tmp.setTranslate(-scalecenter);
+    transform.multRight(tmp);
+
+    // SbMatrix does not have a getScale method, so just read the
+    // values directly out of the matrix. This matrix does not only
+    // contain scale. It might also have some other junk that we are
+    // not interested in, so just read the scale values and ignore the
+    // rest.
+    SbVec3f newscale;
+    newscale[0] = transform[0][0];
+    newscale[1] = transform[1][1];
+    newscale[2] = transform[2][2];
+
+    // Calling appendScale here might cause a neverending loop if the
+    // adjusted scale is just below SoDragger::minscale, so redo the
+    // calculation of the resulting matrix without recursion.
+    transform.setTranslate(-scalecenter);
+    tmp.setScale(newscale);
+    transform.multRight(tmp);
+    tmp.setTranslate(scalecenter);
+    transform.multRight(tmp);
+
+    if (conversion) {
+      transform.multRight(*conversion);
+      transform.multLeft(conversion->inverse());
+    }
+    res = matrix;
+    res.multLeft(transform);
+  }
+
+  return res;
 }
 
 /*!
@@ -1409,7 +1502,7 @@ typedef struct {
   SbViewVolume vv;
 } sodragger_vv_data;
 
-static SoCallbackAction::Response 
+static SoCallbackAction::Response
 sodragger_vv_cb(void * userdata, SoCallbackAction * action, const SoNode * node)
 {
   sodragger_vv_data * data = (sodragger_vv_data*) userdata;
@@ -1458,6 +1551,12 @@ SoDragger::handleEvent(SoHandleEventAction * action)
     else
       this->updateDraggerCache(NULL);
   }
+  // try child draggers first
+  if (action->getGrabber() != this) {
+    inherited::handleEvent(action);
+  }
+  // return if handled by a child
+  if (action->isHandled()) return;
 
   // this is a special case, to be able to detect when somebody
   // clicks ctrl over a dragger. This has a special meaning for
@@ -1506,6 +1605,7 @@ SoDragger::handleEvent(SoHandleEventAction * action)
         this->updateDraggerCache(NULL);
 
       this->isActive = TRUE;
+      THIS->didmousemove = FALSE;
       this->setCameraInfo(action);
       this->setStartingPoint(pp);
       this->eventHandled(event, action);
@@ -1521,7 +1621,10 @@ SoDragger::handleEvent(SoHandleEventAction * action)
   }
   else if (this->isActive.getValue() && SO_MOUSE_RELEASE_EVENT(event, BUTTON1)) {
     this->isActive = FALSE;
-    this->eventHandled(event, action);
+    if (THIS->didmousemove) {
+      this->eventHandled(event, action);
+      THIS->didmousemove = FALSE;
+    }
     if (THIS->isgrabbing) this->grabEventsCleanup();
     if (THIS->pickedpath) {
       THIS->pickedpath->unref();
@@ -1542,6 +1645,7 @@ SoDragger::handleEvent(SoHandleEventAction * action)
   }
   else if (this->isActive.getValue() && event->isOfType(SoLocation2Event::getClassTypeId())) {
     this->eventHandled(event, action);
+    THIS->didmousemove = TRUE;
     THIS->motionCB.invokeCallbacks(this);
     if (!THIS->isgrabbing) {
       this->grabEventsSetup();
