@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -44,6 +44,8 @@
 // FIXME: Should include an URL-link to the default geometry-file?
 // Plus a small usage example.  20011113 mortene.
 
+#include <string.h>
+
 #include <Inventor/draggers/SoHandleBoxDragger.h>
 #include <Inventor/nodekits/SoSubKitP.h>
 #include <Inventor/nodes/SoDrawStyle.h>
@@ -384,7 +386,7 @@ SoHandleBoxDragger::SoHandleBoxDragger(void)
   if (SO_KIT_IS_FIRST_INSTANCE()) {
     SoInteractionKit::readDefaultParts("handleBoxDragger.iv",
                                        HANDLEBOXDRAGGER_draggergeometry,
-                                       sizeof(HANDLEBOXDRAGGER_draggergeometry));
+                                       (int)strlen(HANDLEBOXDRAGGER_draggergeometry));
   }
 
   SO_KIT_ADD_FIELD(translation, (0.0f, 0.0f, 0.0f));
