@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -44,6 +44,13 @@
   Rotations are invoked by clicking and dragging the line parts of the
   3 principal "axes" of the dragger geometry.
 
+  <b>NB:</b> How to rotate the dragger might be abit unintuitive for
+  the end user. If the dragger is to be used 'out of the box', a good
+  documentation should be added to your application. An alternative is
+  to modify the geometry to increase the affordance of the dragger.
+  See the SoDragger documentation on how to change the geometry of a
+  dragger.
+
   An "action shot" of the SoJackDragger used within an SoClipPlaneManip:
   <center>
   <img src="http://doc.coin3d.org/images/Coin/draggers/jack-in-action.png">
@@ -58,6 +65,8 @@
   also uses this dragger, for controlling an SoClipPlane node.
 */
 
+#include <string.h>
+
 #include <Inventor/draggers/SoJackDragger.h>
 #include <Inventor/nodekits/SoSubKitP.h>
 #include <Inventor/draggers/SoDragPointDragger.h>
@@ -178,7 +187,7 @@ SoJackDragger::SoJackDragger(void)
   if (SO_KIT_IS_FIRST_INSTANCE()) {
     SoInteractionKit::readDefaultParts("jackDragger.iv",
                                        JACKDRAGGER_draggergeometry,
-                                       sizeof(JACKDRAGGER_draggergeometry));
+                                       (int)strlen(JACKDRAGGER_draggergeometry));
   }
 
   SO_KIT_ADD_FIELD(rotation, (SbRotation(SbVec3f(0.0f, 0.0f, 1.0f), 0.0f)));
