@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -50,6 +50,8 @@
   \sa SoDirectionalLightDragger, SoPointLightDragger
 */
 
+#include <string.h>
+
 #include <Inventor/draggers/SoSpotLightDragger.h>
 #include <Inventor/nodekits/SoSubKitP.h>
 #include <Inventor/draggers/SoDragPointDragger.h>
@@ -195,7 +197,7 @@ SoSpotLightDragger::SoSpotLightDragger(void)
   if (SO_KIT_IS_FIRST_INSTANCE()) {
     SoInteractionKit::readDefaultParts("spotLightDragger.iv",
                                        SPOTLIGHTDRAGGER_draggergeometry,
-                                       sizeof(SPOTLIGHTDRAGGER_draggergeometry));
+                                       (int)strlen(SPOTLIGHTDRAGGER_draggergeometry));
   }
 
   SO_KIT_ADD_FIELD(rotation, (SbRotation(SbVec3f(0.0f, 0.0f, 1.0f), 0.0f)));
@@ -216,10 +218,9 @@ SoSpotLightDragger::SoSpotLightDragger(void)
   // create this part here so that we don't add this node to the scene
   // graph during handleEvent() (causes changes to the scene graph
   // during traversal)
-  this->setPart("translatorRotInv", new SoRotation);
+  (void)SO_GET_ANY_PART(this, "translatorRotInv", SoRotation);
 
-  SoSwitch *sw;
-  sw = SO_GET_ANY_PART(this, "beamSwitch", SoSwitch);
+  SoSwitch * sw = SO_GET_ANY_PART(this, "beamSwitch", SoSwitch);
   SoInteractionKit::setSwitchValue(sw, 0);
 
   this->setBeamScaleFromAngle(1.0f);
