@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -41,6 +41,7 @@
   \sa SoTabBoxDragger
 */
 
+#include <string.h>
 
 #include <Inventor/draggers/SoTabPlaneDragger.h>
 #include <Inventor/nodekits/SoSubKitP.h>
@@ -217,7 +218,7 @@ SoTabPlaneDragger::SoTabPlaneDragger(void)
   if (SO_KIT_IS_FIRST_INSTANCE()) {
     SoInteractionKit::readDefaultParts("tabPlaneDragger.iv",
                                        TABPLANEDRAGGER_draggergeometry,
-                                       sizeof(TABPLANEDRAGGER_draggergeometry));
+                                       (int)strlen(TABPLANEDRAGGER_draggergeometry));
   }
 
   SO_KIT_ADD_FIELD(translation, (0.0f, 0.0f, 0.0f));
@@ -346,12 +347,11 @@ SoTabPlaneDragger::valueChangedCB(void *, SoDragger * d)
 void
 SoTabPlaneDragger::GLRender(SoGLRenderAction * action)
 {
-  //
-  // I think it is best to always recalculate tabs. In OIV, you'll need to
-  // click into the dragger sometimes to make the dragger recalculate tabs
-  // (after zooming, for instance).
-  //
-  if (1 || this->adjustTabs) {
+  // FIXME: it'd be better to adjust even when scaling, but it's a bit
+  // complicated to calculate the correct scale factor adjustment (to
+  // make the pointer stay at the exact same position in the tab no
+  // matter how the dragger is scaled). pederb, 2004-09-22
+  if (this->whatkind != WHATKIND_SCALE) {
     // disable notification do avoid multiple redraws, but
     // remember to invalidate open caches.
     SbBool oldnotify = this->enableNotify(FALSE);
@@ -538,7 +538,12 @@ SoTabPlaneDragger::dragStart(void)
   }
 
   if (this->whatkind == WHATKIND_SCALE) {
-    this->lineProj->setLine(SbLine(this->scaleCenter, startpt));
+    // startpt might have a non-zero z-component that could cause
+    // trouble when scaling down to very small sizes, so create a
+    // point equal to startpt, but with the z-component set to zero.
+    // scaleCenter already has a zero z-value.
+    SbVec3f linept(startpt[0], startpt[1], 0.0f);
+    this->lineProj->setLine(SbLine(this->scaleCenter, linept));
   }
   else { // translate
     this->planeProj->setPlane(SbPlane(SbVec3f(0.0f, 0.0f, 1.0f), startpt));
@@ -563,13 +568,19 @@ SoTabPlaneDragger::drag(void)
     SbVec3f projpt = this->lineProj->project(this->getNormalizedLocaterPosition());
 
     SbVec3f center = this->scaleCenter;
+    SbVec3f orgvec = startpt - center;
+    SbVec3f currvec = projpt - center;
 
-    float orglen = (startpt-center).length();
-    float currlen = (projpt-center).length();
+    // Ignore the possible z-component of the vectors
+    orgvec[2] = 0.0f;
+    currvec[2] = 0.0f;
+
+    float orglen = orgvec.length();
+    float currlen = currvec.length();
     float scale = 0.0f;
 
     if (orglen > 0.0f) scale = currlen / orglen;
-    if (scale > 0.0f && (startpt-center).dot(projpt-center) <= 0.0f) scale = 0.0f;
+    if (scale > 0.0f && orgvec.dot(currvec) <= 0.0f) scale = 0.0f;
 
     SbVec3f scalevec(scale, scale, 1.0f);
     if (this->constraintState == CONSTRAINT_X) {
@@ -751,6 +762,7 @@ SoTabPlaneDragger::createPrivateParts(void)
   // change very often.
   SoSeparator *sep = SO_GET_ANY_PART(this, "scaleTabs", SoSeparator);
   sep->renderCaching = SoSeparator::OFF;
+  sep->renderCaching.setDefault(TRUE);
   // this is the default, so don't write it
   this->scaleTabs.setDefault(TRUE);
 }
