@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -56,6 +56,8 @@
   \sa SoTrackballManip
 */
 
+#include <string.h>
+
 #include <Inventor/draggers/SoTrackballDragger.h>
 #include <Inventor/nodekits/SoSubKitP.h>
 #include <Inventor/nodes/SoAntiSquish.h>
@@ -275,7 +277,7 @@ SoTrackballDragger::SoTrackballDragger(void)
   if (SO_KIT_IS_FIRST_INSTANCE()) {
     SoInteractionKit::readDefaultParts("trackballDragger.iv",
                                        TRACKBALLDRAGGER_draggergeometry,
-                                       sizeof(TRACKBALLDRAGGER_draggergeometry));
+                                       (int)strlen(TRACKBALLDRAGGER_draggergeometry));
   }
 
   SO_KIT_ADD_FIELD(rotation, (SbRotation(SbVec3f(0.0f, 0.0f, 1.0f), 0.0f)));
@@ -336,6 +338,7 @@ SoTrackballDragger::~SoTrackballDragger()
 {
   delete this->rotFieldSensor;
   delete this->scaleFieldSensor;
+  delete THIS->timerSensor;
   delete THIS->sphereProj;
   delete THIS->cylProj;
   delete THIS->lineProj;
