@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -69,6 +69,7 @@
   \sa SoTransformerManip
 */
 
+#include <string.h>
 
 #include <Inventor/draggers/SoTransformerDragger.h>
 #include <Inventor/nodekits/SoSubKitP.h>
@@ -96,12 +97,8 @@
 #include <coindefs.h> // COIN_STUB() & COIN_OBSOLETED()
 
 // FIXME, bugs or missing features (pederb, 20000224):
-// o when appending a rotation and scale is not uniform, the cube
-//   is sometimes sheared. Seems to be a scaleorientation problem.
 // o some feedback is missing (mostly crosshair)
 // o detect if disc or cylinder rotator should be used (disc-only right now)
-// o not possible to go from free rotate to disc/cylinder rotate (marked
-//   by COIN_STUB())
 //
 // Also the translation feedback is a bit different from OIV. Coin
 // always places the feedback axes at the center of the face being
@@ -109,11 +106,23 @@
 // strategy is better, since when switching between constrained
 // translations and unconstrained translation, the OIV feedback axes
 // can easily be positioned outside the face being dragged.
-
-// FIXME: one additional bug I found which is not mentioned above;
-// when doing uniform scaling towards a cornerpoint (holding CTRL),
-// the dragger jumps back to it's initial centerposition when CTRL is
-// released. This basically renders this feature unusable. 20011208 mortene.
+//
+// MATRICES AND SPACES:
+// There are many matrices and spaces that can take some time to get
+// understand. The matrices calculated are:
+//
+// localToWorld = motionMatrix * draggerToWorld
+// worldToLocal = worldToDragger * motionMatrix^-1
+// draggerToWorld = worldToDragger^-1
+//
+// localToWorking = surroundScaleMatrix^-1
+// workingToLocal = surroundScaleMatrix
+//
+// workingToWorld = surroundScaleMatrix * localToWorld
+// worldToWorking = worldToLocal * surroundScaleMatrix
+// 
+// boxPointInWorldSpace = p * surroundScaleMatrix * localToWorld
+// worldPointInBoxSpace = p * worldToLocal * surroundScaleMatrix
 
 /*!
   \enum SoTransformerDragger::State
@@ -187,11 +196,11 @@ public:
   SbVec3f ctrlOffset;
   SbBool ctrlDown;
   SbBool shiftDown;
+  SbVec2f normalizedStartLocaterPosition;
 
   SbBool locateHighlighting;
   static int colinearThreshold;
   int constraintState;
-  SbVec3f worldRestartPt;
 
   int whatkind;
   int whatnum;
@@ -700,7 +709,7 @@ SoTransformerDragger::SoTransformerDragger(void)
   if (SO_KIT_IS_FIRST_INSTANCE()) {
     SoInteractionKit::readDefaultParts("transformerDragger.iv",
                                        TRANSFORMERDRAGGER_draggergeometry,
-                                       sizeof(TRANSFORMERDRAGGER_draggergeometry));
+                                       (int)strlen(TRANSFORMERDRAGGER_draggergeometry));
   }
 
   SO_KIT_ADD_FIELD(rotation, (SbRotation(SbVec3f(0.0f, 0.0f, 1.0f), 0.0f)));
@@ -1172,7 +1181,6 @@ SoTransformerDragger::dragStart(void)
       this->lineProj->setLine(SbLine(startpt, startpt + n));
       THIS->constraintState = CONSTRAINT_OFF;
       if (THIS->shiftDown) {
-        this->getWorkingToWorldMatrix().multVecMatrix(startpt, THIS->worldRestartPt);
         THIS->constraintState = CONSTRAINT_WAIT;
       }
       SbLine myline(SbVec3f(0.0f, 0.0f, 0.0f), n);
@@ -1207,7 +1215,7 @@ SoTransformerDragger::dragStart(void)
         feedbackrot->rotation = SbRotation(SbVec3f(1.0f, 0.0f, 0.0f), -float(M_PI)*0.5f);
         break;
       }
-      this->setDynamicTranslatorSwitches(event);
+      (void) this->setDynamicTranslatorSwitches(event);
     }
     break;
   case WHATKIND_SCALE:
@@ -1217,7 +1225,6 @@ SoTransformerDragger::dragStart(void)
       this->lineProj->setLine(SbLine(SbVec3f(0.0f, 0.0f, 0.0f), startpt));
       THIS->constraintState = CONSTRAINT_OFF;
       if (THIS->shiftDown) {
-        this->getWorkingToWorldMatrix().multVecMatrix(startpt, THIS->worldRestartPt);
         THIS->constraintState = CONSTRAINT_WAIT;
       }
 
@@ -1225,7 +1232,7 @@ SoTransformerDragger::dragStart(void)
       str.sprintf("scale%dSwitch", THIS->whatnum);
       this->setAllPartSwitches(0, SO_SWITCH_NONE, SO_SWITCH_NONE);
       this->setSwitchValue(str.getString(), 1);
-      this->setDynamicScaleSwitches(event);
+      (void) this->setDynamicScaleSwitches(event);
     }
     break;
   case WHATKIND_ROTATE:
@@ -1258,6 +1265,7 @@ SoTransformerDragger::dragStart(void)
         THIS->constraintState = CONSTRAINT_WAIT;
         // this plane is only used to find constraint direction
         this->planeProj->setPlane(SbPlane(startpt, startpt));
+        THIS->normalizedStartLocaterPosition = this->getNormalizedLocaterPosition();
       }
       SoAntiSquish *squish = SO_GET_ANY_PART(this, "circleFeedbackAntiSquish", SoAntiSquish);
       SoAntiSquish::Sizing sizing;
@@ -1270,7 +1278,7 @@ SoTransformerDragger::dragStart(void)
       squish->sizing = sizing;
       squish->recalc();
       this->setAllPartSwitches(SO_SWITCH_NONE, 0, SO_SWITCH_NONE);
-      this->setDynamicRotatorSwitches(event);
+      (void) this->setDynamicRotatorSwitches(event);
     }
     break;
   default:
@@ -1315,12 +1323,14 @@ SoTransformerDragger::dragTranslate()
   if (event->wasShiftDown() && THIS->constraintState == CONSTRAINT_OFF) {
     THIS->constraintState = CONSTRAINT_WAIT;
     this->setStartLocaterPosition(event->getPosition());
-    this->getWorkingToWorldMatrix().multVecMatrix(projpt, THIS->worldRestartPt);
   }
   else if (!event->wasShiftDown() && THIS->constraintState != CONSTRAINT_OFF) {
     THIS->constraintState = CONSTRAINT_OFF;
   }
 
+  // Every time something changes (shift or ctrl is pressed), resave
+  // the state so that the transformation to come starts with blank
+  // sheets.
   if (this->setDynamicTranslatorSwitches(event)) {
     this->saveStartParameters();
     SbVec3f n(0.0f, 0.0f, 0.0f);
@@ -1340,46 +1350,48 @@ SoTransformerDragger::dragTranslate()
     motion = projpt - startpt;
   }
   else {
-    SbVec3f localrestartpt;
-    if (THIS->constraintState != CONSTRAINT_OFF) {
-      this->getWorldToLocalMatrix().multVecMatrix(THIS->worldRestartPt,
-                                                  localrestartpt);
-      motion = localrestartpt - startpt;
-    }
-    else motion = projpt - startpt;
+    motion = projpt - startpt;
+
     switch(THIS->constraintState) {
     case CONSTRAINT_OFF:
       break;
     case CONSTRAINT_WAIT:
       if (this->isAdequateConstraintMotion()) {
-        SbVec3f newmotion = projpt - localrestartpt;
         int biggest = 0;
-        double bigval = fabs(newmotion[0]);
-        if (fabs(newmotion[1]) > bigval) {
+        double bigval = fabs(motion[0]);
+        if (fabs(motion[1]) > bigval) {
           biggest = 1;
-          bigval = fabs(newmotion[1]);
+          bigval = fabs(motion[1]);
         }
-        if (fabs(newmotion[2]) > bigval) {
+        if (fabs(motion[2]) > bigval) {
           biggest = 2;
         }
-        motion[biggest] += newmotion[biggest];
+
+        // Set all but the constraint axis to 0
+        motion[(biggest + 1) % 3] = 0.0f;
+        motion[(biggest + 2) % 3] = 0.0f;
+
         THIS->constraintState = CONSTRAINT_X + biggest;
-        this->setDynamicTranslatorSwitches(event);
       }
       else {
         return;
       }
       break;
     case CONSTRAINT_X:
-      motion[0] += projpt[0] - localrestartpt[0];
+      motion[1] = 0.0f;
+      motion[2] = 0.0f;
       break;
     case CONSTRAINT_Y:
-      motion[1] += projpt[1] - localrestartpt[1];
+      motion[0] = 0.0f;
+      motion[2] = 0.0f;
       break;
     case CONSTRAINT_Z:
-      motion[2] += projpt[2] - localrestartpt[2];
+      motion[0] = 0.0f;
+      motion[1] = 0.0f;
+      break;
     }
   }
+
   SbMatrix mat, inv;
   this->getSurroundScaleMatrices(mat, inv);
   this->setMotionMatrix(this->appendTranslation(this->getStartMotionMatrix(), motion, &mat));
@@ -1400,7 +1412,6 @@ SoTransformerDragger::dragScale()
   if (event->wasShiftDown() && THIS->constraintState == CONSTRAINT_OFF) {
     THIS->constraintState = CONSTRAINT_WAIT;
     this->setStartLocaterPosition(event->getPosition());
-    this->getWorkingToWorldMatrix().multVecMatrix(projpt, THIS->worldRestartPt);
   }
   else if (!event->wasShiftDown() && THIS->constraintState != CONSTRAINT_OFF) {
     this->saveStartParameters();
@@ -1453,7 +1464,19 @@ SoTransformerDragger::dragScale()
     this->setStartingPoint(worldpt);
   }
 
-  this->setDynamicScaleSwitches(event);
+  // If the control key is pressed or released,
+  // setDynamicScaleSwitches will return TRUE. When this happens, we
+  // have to resave our start motionmatrix to be able to do correct
+  // scaling. If we do not do this, the scaled object will jump
+  // because the scale center is changed, but not the scale.
+  if (this->setDynamicScaleSwitches(event)) {
+    this->saveStartParameters();
+    startpt = projpt;
+
+    SbVec3f worldpt;
+    this->getWorkingToWorldMatrix().multVecMatrix(projpt, worldpt);
+    this->setStartingPoint(worldpt);
+  }
 
   if (THIS->constraintState == CONSTRAINT_WAIT) return;
 
@@ -1492,9 +1515,59 @@ SoTransformerDragger::dragScale()
   this->unsquishKnobs();
 }
 
+// The dragRotate method has been implemented somewhat differently
+// than the other drag functions. This is because of the unfortunate
+// effect that happens when a non-uniform scale has been applied to
+// the motion-matrix, and rotation is attempted to be done before the
+// scale. Both operations are perfectly valid, but the end result is
+// not what you'd expect; the transformation comes out sheared. To
+// prevent this from happening, the rotation matrices are applied in
+// world space, that is after all other transformations have been
+// applied (scale and translation are applied in local
+// space). What is basically done is this:
+//
+// Calculates the matrix (as done in SoDragger::appendRotation()):
+// 
+// C := conversion
+// P := rotcenter
+// R := rot
+// M := new motion matrix
+// Mold := previous motionmatrix
+// 
+// M = C^-1 * P^-1 * R * P * C * Mold
+// 
+// What essentially happens is that we transform into C's coordinate
+// system, then we move the rotation center to origo and apply the
+// new rotation.  The rotation has now been applied, but we are in
+// the wrong coordinate system, so we reapply the rotation center
+// and the conversion. Finally we transform the matrix by the old
+// transformation, which gives us the new rotated transformation.
+// 
+// The rotation happens in the local coordinate system of the object
+// if C = P = I, but if P is specified, then the rotation center
+// will be adjusted before the rotation is applied. If the
+// conversion matrix has been specified to be e.g:
+// 
+// C = (Mold * W)^-1
+// 
+// Then the resulting matrix looks something like this:
+// 
+// M = ((Mold * W)^-1)^-1 * P^-1 * R * P * (Mold * W)^-1 * Mold
+//   = Mold * W * P^-1 * R * P * W^-1 * Mold^-1 * Mold
+//   = Mold * W * P^-1 * R * P * W^-1
+//
+// Take a closer look at the resulting matrix: It basically reverses
+// the transformation. Instead of having the rotation applied before
+// the old transformation, the rotation is applied after the old
+// transformation. The rotation also happens in world space - that is
+// the coordinate system is transformed to the world coordinate system
+// before the rotation is applied around rotationcenter. Finally the
+// matrix is transformed back to the local coordinate system.
 void
 SoTransformerDragger::dragRotate(void)
 {
+  // Update the sphere projector to the current view so that it
+  // accurately can react to events.
   this->sphereProj->setViewVolume(this->getViewVolume());
   this->sphereProj->setWorkingSpace(this->getWorkingToWorldMatrix());
 
@@ -1504,6 +1577,8 @@ SoTransformerDragger::dragRotate(void)
   startpt = this->getLocalStartingPoint();
   startpt = this->localToWorking(startpt);
 
+  // If shift was pressed and not in non-constrained state, then
+  // we switch to non-constrained state.
   if (event->wasShiftDown() && THIS->constraintState != CONSTRAINT_OFF) {
     THIS->constraintState = CONSTRAINT_OFF;
     projpt = this->sphereProj->project(this->getNormalizedLocaterPosition());
@@ -1512,31 +1587,68 @@ SoTransformerDragger::dragRotate(void)
     this->saveStartParameters();
     this->setStartingPoint(THIS->prevWorldHitPt);
   }
+  // If shift was released while in non-constrained state, then the
+  // state jumps back to constrained.
   else if (!event->wasShiftDown() && THIS->constraintState == CONSTRAINT_OFF) {
-    // FIXME: functionality of dragger is incomplete. This stub is
-    // called up whenever SHIFT is used for free-form rotation and
-    // then SHIFT is released before the mousebutton drag. This
-    // _really_ need to be fixed. 20011208 mortene.
-    COIN_STUB();
+    THIS->constraintState = CONSTRAINT_WAIT;
+    this->setStartingPoint(THIS->prevWorldHitPt);
+    startpt = this->getLocalStartingPoint();
+    startpt = this->localToWorking(startpt);
+    // This plane is only used to find the constraint direction
+    this->planeProj->setPlane(SbPlane(startpt, startpt));
+    this->setStartLocaterPosition(event->getPosition());
+    THIS->normalizedStartLocaterPosition = this->getNormalizedLocaterPosition();
+    this->saveStartParameters();
   }
 
   SbVec3f center(0.0f, 0.0f, 0.0f);
   if (THIS->ctrlDown) {
+    // Adjust the center to be on the other side of the bounding box
+    // when the control key has been pressed.
     center -= THIS->ctrlOffset * KNOB_DISTANCE;
   }
 
-  this->setDynamicRotatorSwitches(event);
+  // Show the necessary feedback geometry for rotation.
+  (void) this->setDynamicRotatorSwitches(event);
 
   if (THIS->constraintState == CONSTRAINT_OFF) {
-    this->getWorldToWorkingMatrix().multVecMatrix(THIS->prevWorldHitPt, startpt);
+    SbVec3f worldcenter = this->getBoxPointInWorldSpace(center);
+    SbVec3f prevworldprojpt = THIS->prevWorldHitPt;
+    SbVec3f worldprojpt;
+
+    // Find locater position in world space
     projpt = this->sphereProj->project(this->getNormalizedLocaterPosition());
-    this->getWorkingToWorldMatrix().multVecMatrix(projpt, THIS->prevWorldHitPt);
-    SbRotation rot = this->sphereProj->getRotation(startpt, projpt);
-    SbMatrix mat, inv;
-    this->getSurroundScaleMatrices(mat, inv);
-    THIS->prevMotionMatrix = this->appendRotation(THIS->prevMotionMatrix, rot,
-                                                  center, &mat);
-    this->setMotionMatrix(THIS->prevMotionMatrix);
+    this->getWorkingToWorldMatrix().multVecMatrix(projpt, worldprojpt);
+    SbVec3f wppt = worldprojpt;
+
+    // Find the projection vectors from the box center in world space.
+    worldprojpt -= worldcenter;
+    prevworldprojpt -= worldcenter;
+
+    // Normalize the vectors before finding the dotproduct angle
+    // between them.
+    worldprojpt.normalize();
+    prevworldprojpt.normalize();
+
+    // FIXME: Without this test, the rotation behaves very strange,
+    // especially at the edge/outside of the sphere. I will
+    // investigate this some more to understand what happens. 
+    // 20040804 jornskaa
+    if (worldprojpt.dot(prevworldprojpt) > 0.8f) {
+      THIS->prevWorldHitPt = wppt;
+
+      // Calculate the rotation from previous prevworldprojpt to
+      // worldprojpt
+      SbRotation rot(prevworldprojpt, worldprojpt);
+      
+      // Calculate new motionmatrix by rotating in world space to
+      // prevent shearing with non-uniform scale.
+      SbMatrix mat = this->getWorldToLocalMatrix();
+      THIS->prevMotionMatrix = this->appendRotation(THIS->prevMotionMatrix, rot,
+                                                    worldcenter, &mat);
+      
+      this->setMotionMatrix(THIS->prevMotionMatrix);
+    }
   }
   else if (THIS->constraintState == CONSTRAINT_WAIT && this->isAdequateConstraintMotion()) {
     this->planeProj->setViewVolume(this->getViewVolume());
@@ -1544,6 +1656,8 @@ SoTransformerDragger::dragRotate(void)
     projpt = this->planeProj->project(this->getNormalizedLocaterPosition());
 
     SbVec3f diff = projpt - startpt;
+    diff[THIS->dimension] = 0.0f;
+
     int biggest = 0;
     double bigval = fabs(diff[0]);
     if (fabs(diff[1]) > bigval) {
@@ -1558,28 +1672,83 @@ SoTransformerDragger::dragRotate(void)
     n[biggest] = 1.0f;
     SbVec3f dim(0.0f, 0.0f, 0.0f);
     dim[THIS->dimension] = 1.0f;
+
     // set plane to do disc-rotate in
     this->planeProj->setPlane(SbPlane(SbVec3f(0.0f, 0.0f, 0.0f), dim, dim+n));
-    this->setDynamicRotatorSwitches(event);
+    (void) this->setDynamicRotatorSwitches(event);
+
+    // Initialize prevWorldHitPt and prevMotionMatrix.
+    projpt = this->planeProj->project(THIS->normalizedStartLocaterPosition);
+    this->getWorkingToWorldMatrix().multVecMatrix(projpt, THIS->prevWorldHitPt);
+    THIS->prevMotionMatrix = this->getMotionMatrix();
   }
   if (THIS->constraintState >= CONSTRAINT_X) {
-    // project startpt into plane to get nicer disc-rotation
-    // SbPlane really should have had a getClosestPoint() method.
-    const SbPlane &plane = this->planeProj->getPlane();
-    startpt -= plane.getNormal() * plane.getDistance(startpt);
-
     this->planeProj->setViewVolume(this->getViewVolume());
     this->planeProj->setWorkingSpace(this->getWorkingToWorldMatrix());
 
+    SbVec3f worldcenter = this->getBoxPointInWorldSpace(center);
+    SbVec3f prevworldprojpt;
+    SbVec3f prevprojpt;
+    SbVec3f worldprojpt;
+
+    prevworldprojpt = THIS->prevWorldHitPt;
+
+    // Find current locater position projected onto the plane in world
+    // space.
     projpt = this->planeProj->project(this->getNormalizedLocaterPosition());
-    startpt -= center;
+    this->getWorkingToWorldMatrix().multVecMatrix(projpt, worldprojpt);
+
+    // Project the vectors onto the projection plane in world space.
+    // This is needed to make the rotation vectors more numerically
+    // stable especially when the dragger has been scaled to be very
+    // small before rotation is attempted.
+    SbPlane plane = this->planeProj->getPlane();
+    plane.transform(this->getWorkingToWorldMatrix());
+    prevworldprojpt -= plane.getNormal() * plane.getDistance(prevworldprojpt);
+    worldprojpt -= plane.getNormal() * plane.getDistance(worldprojpt);
+
+    // Save the values for the new projection point before it is
+    // altered.
+    SbVec3f wppt = worldprojpt;
+
+    // Adjust the rotation vectors of the dragger to match the center
+    // of the dragger in working space and world space.
     projpt -= center;
-    SbRotation rot(startpt, projpt);
-    SbMatrix mat, inv;
-    this->getSurroundScaleMatrices(mat, inv);
-    this->setMotionMatrix(this->appendRotation(this->getStartMotionMatrix(),
-                                               rot,
-                                               center, &mat));
+    prevworldprojpt -= worldcenter;
+    worldprojpt -= worldcenter;
+
+    // Make sure the length of the projected vector is greater than
+    // a scalar constant. This ensures that the rotation is defined
+    // and there is little room for error that might happen when the
+    // vectors come very close to the rotation center.
+    if (projpt.sqrLength() > 0.1f) {
+      // Normalize the vectors before finding the dotproduct angle
+      // between them.
+      prevworldprojpt.normalize();
+      worldprojpt.normalize();
+
+      // Since we are using incremental changes, the changes will
+      // never be very big, and the dot product between the
+      // rotation-vectors should always be above zero, and never
+      // negative because that indicates the angle between the vectors
+      // is above PI/2. This might happen if dragging is done over the
+      // rotation center, but we do not allow that and just wait until
+      // the locater comes into the valid range before rotating.
+      if (prevworldprojpt.dot(worldprojpt) > 0.3f) { // 0.3 == 72.5degrees
+        THIS->prevWorldHitPt = wppt;
+        
+        // Rotate between the two points in the plane
+        SbRotation rot(prevworldprojpt, worldprojpt);
+        
+        // Rotate in world space to prevent shearing when having
+        // non-uniform scale.
+        SbMatrix mat = this->getWorldToLocalMatrix();
+        THIS->prevMotionMatrix = (this->appendRotation(THIS->prevMotionMatrix, rot,
+                                                       worldcenter, &mat));
+        
+        this->setMotionMatrix(THIS->prevMotionMatrix);
+      }
+    }
   }
   this->unsquishKnobs();
 }
@@ -1624,6 +1793,7 @@ SoTransformerDragger::dragFinish(void)
   this->setSwitchValue("zAxisFeedbackSwitch", SO_SWITCH_NONE);
 
 #if COIN_DEBUG && 0 // used to debug motion matrix (pederb, 20000225)
+  SbMatrix m = this->getMotionMatrix();
   SbRotation r,so;
   SbVec3f t,s;
 
