@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -30,28 +30,38 @@
   (hopefully) work in the same way as the Inventor class though.
 */
 
+// FIXME: is the above class doc comment still correct? Or do we have
+// an implementation of this class in the same manner as the other
+// Inventors now? 20040702 mortene.
+
+// *************************************************************************
+
 #include <Inventor/elements/SoGLLazyElement.h>
-#include <Inventor/elements/SoShapeStyleElement.h>
-#include <Inventor/elements/SoGLTextureImageElement.h>
-#include <Inventor/elements/SoGLTextureEnabledElement.h>
-#include <Inventor/elements/SoGLTexture3EnabledElement.h>
-#include <Inventor/elements/SoTextureCombineElement.h>
-#include <Inventor/misc/SoState.h>
-#include <Inventor/actions/SoGLRenderAction.h>
-#include <Inventor/C/glue/gl.h>
-#include <Inventor/misc/SoGLImage.h>
-#include <Inventor/SbImage.h>
-#include <Inventor/elements/SoGLCacheContextElement.h>
-#include <Inventor/errors/SoDebugError.h>
-#include <Inventor/bundles/SoMaterialBundle.h>
-#include <Inventor/errors/SoDebugError.h>
-#include <Inventor/SoFullPath.h>
-#include <Inventor/nodes/SoNode.h>
 
-#include <coindefs.h> // COIN_OBSOLETED
 #include <assert.h>
 #include <string.h>
 
+#include <Inventor/C/glue/gl.h>
+#include <Inventor/SbImage.h>
+#include <Inventor/SoFullPath.h>
+#include <Inventor/actions/SoGLRenderAction.h>
+#include <Inventor/bundles/SoMaterialBundle.h>
+#include <Inventor/elements/SoGLCacheContextElement.h>
+#include <Inventor/elements/SoGLDisplayList.h>
+#include <Inventor/elements/SoGLTexture3EnabledElement.h>
+#include <Inventor/elements/SoGLTextureEnabledElement.h>
+#include <Inventor/elements/SoGLTextureImageElement.h>
+#include <Inventor/elements/SoShapeStyleElement.h>
+#include <Inventor/elements/SoTextureCombineElement.h>
+#include <Inventor/errors/SoDebugError.h>
+#include <Inventor/errors/SoDebugError.h>
+#include <Inventor/misc/SoGLImage.h>
+#include <Inventor/misc/SoState.h>
+#include <Inventor/nodes/SoNode.h>
+#include <coindefs.h> // COIN_OBSOLETED
+
+// *************************************************************************
+
 #define FLAG_FORCE_DIFFUSE      0x0001
 #define FLAG_DIFFUSE_DEPENDENCY 0x0002
 
@@ -567,7 +577,7 @@ SoGLLazyElement::sendDiffuseByIndex(const int index) const
       first = 0;
     }
 
-    safeindex = SbClamp(index, 0, this->coinstate.numdiffuse-1);
+    safeindex = SbClamp((long) index, (long) 0, (long) (this->coinstate.numdiffuse-1));
   }
 #endif // COIN_DEBUG
 
@@ -576,7 +586,11 @@ SoGLLazyElement::sendDiffuseByIndex(const int index) const
   }
   else {
     uint32_t col = this->packedpointer[safeindex] | this->transpmask;
-    if (col != GLSTATE(this)->diffuse) this->sendPackedDiffuse(col);
+    // this test is really not necessary. SoMaterialBundle does the
+    // same test.  We also need to send the color here to work around
+    // an nVIDIA bug
+    // if (col != GLSTATE(this)->diffuse) 
+    this->sendPackedDiffuse(col);
   }
 }
 
