@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2004 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  *
  *  See <URL:http://www.coin3d.org/> for more information.
  *
- *  Systems in Motion, Teknobyen, Abels Gate 5, 7030 Trondheim, NORWAY.
+ *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
  *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
@@ -32,10 +32,13 @@
 #include <Inventor/elements/SoGLTextureImageElement.h>
 #include <Inventor/elements/SoTextureQualityElement.h>
 #include <Inventor/elements/SoGLCacheContextElement.h>
+#include <Inventor/elements/SoShapeStyleElement.h>
 #include <Inventor/elements/SoLazyElement.h>
 #include <Inventor/actions/SoGLRenderAction.h>
 #include <Inventor/misc/SoGL.h> // GL wrapper.
+#include <Inventor/C/glue/glp.h>
 #include <Inventor/misc/SoGLImage.h>
+#include <Inventor/misc/SoGLBigImage.h>
 #include <Inventor/SbImage.h>
 #include <Inventor/C/tidbits.h>
 #include <stdlib.h>
@@ -141,7 +144,6 @@ SoGLTextureImageElement::set(SoState * const stateptr, SoNode * const node,
 {
   SoGLTextureImageElement * elem = (SoGLTextureImageElement*)
     stateptr->getElement(classStackIndex);
-  if (!elem) return;
 
   if (elem->glimage && elem->glimage->getImage()) elem->glimage->getImage()->readUnlock();
   if (image) {
@@ -163,6 +165,10 @@ SoGLTextureImageElement::set(SoState * const stateptr, SoNode * const node,
     elem->glimage = NULL;
     inherited::setDefault(stateptr, node);
   }
+  SoShapeStyleElement::setBigImageEnabled(stateptr,
+                                          image && image->isOfType(SoGLBigImage::getClassTypeId()));
+  SoShapeStyleElement::setTransparentTexture(stateptr, elem->hasTransparency());
+  
   elem->updateLazyElement();
 }
 
@@ -218,12 +224,14 @@ SoGLTextureImageElement::getMaxGLTextureSize(void)
 /*!
   Returns true if the texture with the given dimensions is supported
   by the current OpenGL context.
+
   If zsize==0, 2D texturing is assumed, else 3D texturing is assumed.
 
-  This function uses PROXY textures and will fall back to getMaxGLTextureSize()
-  if PROXY textures are not supported (OpenGL < 1.1 and GL_EXT_texture not
-  available). In the 3D case, 3D textures need to be supported (OpenGL >= 1.2
-  or GL_EXT_texture3D).
+  This function uses OpenGL proxy textures and will fall back to what
+  you get from SoGLTextureImageElement::getMaxGLTextureSize() if proxy
+  textures are not supported (i.e. OpenGL version < 1.1 and
+  GL_EXT_texture not available). In the 3D case, 3D textures need to
+  be supported (OpenGL >= 1.2 or GL_EXT_texture3D).
 
   Note that this function needs an OpenGL context to be made current
   for it to work. Without that, you will most likely get a faulty
@@ -237,70 +245,11 @@ SbBool
 SoGLTextureImageElement::isTextureSizeLegal(int xsize, int ysize, int zsize,
                                             int bytespertexel)
 {
-  // FIXME: the technique we are using doesn't really match what is recommended at
-  //  http://www.opengl.org/developers/documentation/OGL_userguide/OpenGLonWin-13.html
-  // (see "Testing Whether Textures Fit: The Texture Proxy Mechanism").
-  // 20020701 mortene.
-
-  // FIXME: mipmaps must be handled specifically, which we are not
-  // doing. 20020701 mortene.
-
   const cc_glglue * glw = sogl_glue_instance(this->state);
-  if (zsize==0) { // 2D textures
-    if (COIN_MAXIMUM_TEXTURE2_SIZE > 0) {
-      if (xsize > COIN_MAXIMUM_TEXTURE2_SIZE) return FALSE;
-      if (ysize > COIN_MAXIMUM_TEXTURE2_SIZE) return FALSE;
-      return TRUE;
-    }
-    if (cc_glglue_has_2d_proxy_textures(glw)) {
-      GLint w;
-      glTexImage2D(GL_PROXY_TEXTURE_2D, 0, bytespertexel,
-                   xsize, ysize, 0,
-                   GL_RGBA, GL_UNSIGNED_BYTE,
-                   NULL);
-      glGetTexLevelParameteriv(GL_PROXY_TEXTURE_2D, 0,
-                               GL_TEXTURE_WIDTH, &w);
-      if (w==0) return FALSE;
-      return TRUE;
-    }
-    else {
-      // Falls back to using the old method
-      int maxsize = SoGLTextureImageElement::getMaxGLTextureSize();
-      if (xsize > maxsize || ysize > maxsize) return FALSE;
-      return TRUE;
-    }
-  }
-  else { // 3D textures
-    if (cc_glglue_has_3d_textures(glw)) {
-      if (COIN_MAXIMUM_TEXTURE3_SIZE > 0) {
-        if (xsize > COIN_MAXIMUM_TEXTURE3_SIZE) return FALSE;
-        if (ysize > COIN_MAXIMUM_TEXTURE3_SIZE) return FALSE;
-        if (zsize > COIN_MAXIMUM_TEXTURE3_SIZE) return FALSE;
-        return TRUE;
-      }
-      cc_glglue_glTexImage3D(glw,
-                             GL_PROXY_TEXTURE_3D, 0, (GLenum) bytespertexel,
-                             xsize, ysize, zsize, 0,
-                             GL_RGBA, GL_UNSIGNED_BYTE,
-                             NULL);
-      GLint w;
-      glGetTexLevelParameteriv(GL_PROXY_TEXTURE_3D, 0,
-                               GL_TEXTURE_WIDTH, &w);
-      if (w==0) return FALSE;
-      return TRUE;
-    }
-    else {
-#if COIN_DEBUG
-      static SbBool first = TRUE;
-      if (first) {
-        SoDebugError::post("SoGLTextureImageElement::isTextureSizeLegal",
-                           "3D not supported with this OpenGL driver");
-        first = FALSE;
-      }
-#endif // COIN_DEBUG
-      return FALSE;
-    }
-  }
+  SbBool compress = 
+    this->glimage ? this->glimage->getFlags() & SoGLImage::COMPRESSED : FALSE;
+  return coin_glglue_is_texture_size_legal(glw, xsize, ysize, zsize, 
+                                           bytespertexel, TRUE, compress);
 }
 
 void 
